Package: MortalityLaws
Type: Package
Title: Parametric Mortality Models, Life Tables and HMD
Version: 2.2.0
Authors@R: c(
  person("Marius D.", "Pascariu", role = c("aut", "cre", "cph"), email = "mpascariu@outlook.com", comment = c(ORCID = "0000-0002-2568-6489")),
  person("Vladimir", "Canudas-Romo", role = "ctb"))
Maintainer: Marius D. Pascariu <mpascariu@outlook.com>
Description: Fit the most popular human mortality 'laws', and construct 
  full and abridge life tables given various input indices. A mortality
  law is a parametric function that describes the dying-out process of 
  individuals in a population during a significant portion of their 
  life spans. For a comprehensive review of the most important mortality 
  laws see Tabeau (2001) <doi:10.1007/0-306-47562-6_1>. 
  Practical functions for downloading data from various human mortality 
  databases are provided as well.  
License: MIT + file LICENSE
LazyData: TRUE
Depends: R (>= 3.0.0)
Imports: RCurl (>= 1.95), pbapply (>= 1.3-4), tidyr (>= 0.8.1), rvest
        (>= 1.0.3), httr (>= 1.4.5), methods
Suggests: testthat, knitr, rmarkdown
URL: https://github.com/mpascariu/MortalityLaws
BugReports: https://github.com/mpascariu/MortalityLaws/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-05 06:43:17 UTC; masax
Author: Marius D. Pascariu [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-2568-6489>),
  Vladimir Canudas-Romo [ctb]
Repository: CRAN
Date/Publication: 2026-05-05 07:10:08 UTC
Built: R 4.6.0; ; 2026-05-05 08:34:49 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(AIC,MortalityLaw)
S3method(deviance,MortalityLaw)
S3method(df.residual,MortalityLaw)
S3method(logLik,MortalityLaw)
S3method(plot,MortalityLaw)
S3method(predict,MortalityLaw)
S3method(print,LifeTable)
S3method(print,MortalityLaw)
S3method(print,ReadAHMD)
S3method(print,ReadCHMD)
S3method(print,ReadHMD)
S3method(print,ReadJMD)
S3method(print,availableLF)
S3method(print,availableLaws)
S3method(print,summary.MortalityLaw)
S3method(summary,MortalityLaw)
export(HP)
export(HP2)
export(HP3)
export(HP4)
export(LawTable)
export(LifeTable)
export(MortalityLaw)
export(ReadAHMD)
export(ReadCHMD)
export(ReadHMD)
export(ReadJMD)
export(availableHMD)
export(availableLF)
export(availableLaws)
export(beard)
export(beard_makeham)
export(carriere1)
export(carriere2)
export(convertFx)
export(ggompertz)
export(gompertz)
export(gompertz0)
export(invgompertz)
export(invweibull)
export(kannisto)
export(kannisto_makeham)
export(kostaki)
export(makeham)
export(makeham0)
export(martinelle)
export(opperman)
export(perks)
export(quadratic)
export(rogersplanck)
export(siler)
export(strehler_mildvan)
export(thiele)
export(vandermaen)
export(vandermaen2)
export(weibull)
export(wittstein)
import(httr)
import(rvest)
importFrom(RCurl,getURL)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,box)
importFrom(graphics,hist)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,rect)
importFrom(methods,is)
importFrom(pbapply,closepb)
importFrom(pbapply,setpb)
importFrom(pbapply,startpb)
importFrom(stats,coef)
importFrom(stats,df.residual)
importFrom(stats,fitted)
importFrom(stats,nlminb)
importFrom(stats,optim)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,pt)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,vcov)
importFrom(tidyr,spread)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,read.table)
importFrom(utils,tail)

# <img src="inst/figures/hex_MoralityLaws.png" align="right" width="175" height="175" />MortalityLaws: Parametric Mortality Models, Life Tables and HMD
[![CRAN_Version](https://www.r-pkg.org/badges/version/MortalityLaws)](https://cran.r-project.org/package=MortalityLaws)
[![codecov](https://codecov.io/github/mpascariu/MortalityLaws/branch/master/graphs/badge.svg)](https://app.codecov.io/github/mpascariu/MortalityLaws)
[![issues](https://img.shields.io/github/issues-raw/mpascariu/MortalityLaws.svg)](https://github.com/mpascariu/MortalityLaws/issues)

[![lifecycle](https://img.shields.io/badge/lifecycle-stable-green.svg)](https://lifecycle.r-lib.org/articles/stages.html)
[![license](https://img.shields.io/badge/License-MIT-blue.svg)](https://github.com/mpascariu/MortalityLaws/blob/master/LICENSE)
[![CRAN_Download_Badge1](https://cranlogs.r-pkg.org/badges/grand-total/MortalityLaws)](https://CRAN.R-project.org/package=MortalityLaws)
[![CRAN_Download_Badge2](https://cranlogs.r-pkg.org/badges/MortalityLaws)](https://CRAN.R-project.org/package=MortalityLaws)


Fit the most popular human mortality `laws`, and construct full and abridged life tables given various input indices. A mortality law is a parametric function that describes the dying-out process of individuals in a population during a significant portion of their 
life spans. For a comprehensive review of the most important mortality laws see [Tabeau (2001)](https://doi.org/10.1007/0-306-47562-6_1). Function for downloading demographic data from the following sources are provided:

- the [Human Mortality Database (HMD)](https://www.mortality.org/)
- the [Australian Human Mortality Database (AHMD)](https://aushd.org/)
- the [Canadian Human Mortality Database (CHMD)](http://www.bdlc.umontreal.ca/chmd/index.htm)
- the [Japanese Mortality Database (JMD)](https://www.ipss.go.jp/p-toukei/JMD/index-en.asp)


## Installation

1. Make sure you have the most recent version of R
2. Run the following code in your R console 

```R
install.packages("MortalityLaws")
```

## Updating to the latest version of the package

You can track and contribute to the development of `MortalityLaws` on [GitHub](https://github.com/mpascariu/MortalityLaws). To install it:

1. Install the release version of `devtools` from CRAN with `install.packages("devtools")`.

2. Make sure you have a working development environment.
    * **Windows**: Install [Rtools](https://CRAN.R-project.org/bin/windows/Rtools/).
    * **Mac**: Install `Xcode` from the Mac App Store.
    * **Linux**: Install a compiler and various development libraries (details vary across different flavors of Linux).

3. Install the development version of `MortalityLaws`.

   ```R
   devtools::install_github("mpascariu/MortalityLaws")
   ```

## Help
All functions are documented in the standard way, which means that once you load the package using ```library(MortalityLaws)``` you can just type ```?MortalityLaw``` to see the help file. 

# -------------------------------------------------------------- #
# Author: Marius D. PASCARIU
# Last Update: Thu Jul 20 21:11:11 2023
# -------------------------------------------------------------- #

#' Convert Life Table Indicators
#'
#' Easily convert between different life table indicators (e.g., from death 
#' rates \code{mx} to death probabilities \code{qx}, or from survivorship 
#' \code{lx} to life expectancy \code{ex}). The function wraps 
#' \code{\link{LifeTable}} internally, so the conversion relies on the 
#' same constant-force-of-mortality (CFM) assumption and life-table 
#' methodology used throughout the package.
#'
#' @details
#' This function provides a convenient interface for converting a single 
#' mortality indicator into another, without having to call 
#' \code{\link{LifeTable}} directly and extract the desired column.
#'
#' The supported \strong{input} types (\code{from}) are:
#' \code{mx}, \code{qx}, \code{dx}, and \code{lx}.
#'
#' The supported \strong{output} types (\code{to}) are:
#' \code{mx}, \code{qx}, \code{dx}, \code{lx}, \code{Lx}, \code{Tx}, and 
#' \code{ex}.
#'
#' There are 28 possible \code{from}-\code{to} combinations (4 inputs 
#' \eqn{\times} 7 outputs). All conversions pass through the full life-table 
#' computation; for example, converting \code{mx} to \code{ex} will 
#' internally compute \code{qx}, \code{lx}, \code{dx}, \code{Lx}, and 
#' \code{Tx} in sequence.
#'
#' When \code{data} is a \code{vector}, the function returns a named vector. 
#' When \code{data} is a \code{matrix} or \code{data.frame} with multiple 
#' columns, the function applies the conversion column-wise and returns a 
#' matrix with the same row and column names as the input.
#'
#' @usage convertFx(x, data, from, to, ...)
#'
#' @inheritParams LifeTable
#'
#' @param data A numeric \code{vector}, \code{matrix}, or \code{data.frame} 
#'   containing the mortality indicator to be converted. Each row should 
#'   correspond to an age, each column to a separate population or time 
#'   period.
#'
#' @param from The type of indicator supplied in \code{data}. One of:
#'   \code{"mx"}, \code{"qx"}, \code{"dx"}, or \code{"lx"}.
#'
#' @param to The desired output indicator. One of:
#'   \code{"mx"}, \code{"qx"}, \code{"dx"}, \code{"lx"}, \code{"Lx"}, 
#'   \code{"Tx"}, or \code{"ex"}.
#'
#' @param ... Further arguments passed to \code{\link{LifeTable}} that may 
#'   affect the results, such as \code{sex}, \code{lx0}, or \code{ax}.
#'
#' @return A numeric vector or matrix containing the converted life table 
#'   indicator. If the input was a named object, the output retains those 
#'   names.
#'
#' @seealso
#' \code{\link{LifeTable}} for the underlying life-table construction; 
#' \code{\link{LawTable}} for generating life tables from parametric 
#'   mortality laws.
#'
#' @author Marius D. Pascariu
#'
#' @examples
#' # ---- Basic conversions ----
#'
#' x  <- 0:110
#' mx <- ahmd$mx
#'
#' # Convert death rates to death probabilities
#' qx <- convertFx(x, data = mx, from = "mx", to = "qx")
#'
#' # Convert death rates to death distribution
#' dx <- convertFx(x, data = mx, from = "mx", to = "dx")
#'
#' # Convert death rates to survivorship
#' lx <- convertFx(x, data = mx, from = "mx", to = "lx")
#'
#' # ---- All 28 possible conversions ----
#'
#' from <- c("mx", "qx", "dx", "lx")
#' to   <- c("mx", "qx", "dx", "lx", "Lx", "Tx", "ex")
#' K    <- expand.grid(from = from, to = to)
#'
#' for (i in 1:nrow(K)) {
#'   In  <- as.character(K[i, "from"])
#'   Out <- as.character(K[i, "to"])
#'   N   <- paste0(Out, "_from_", In)
#'   cat(i, " Create", N, "\n")
#'   assign(N, convertFx(x = x, data = get(In), from = In, to = Out))
#' }
#'
#' @export
convertFx <- function(x,
                      data,
                      from = c("mx", "qx", "dx", "lx"),
                      to = c("mx", "qx", "dx", "lx", "Lx", "Tx", "ex"),
                      ...) {

  from <- match.arg(from)
  to   <- match.arg(to)

  LifeTable_foo <- switch(
    from,
    mx = function(x, w, ...) LifeTable(x, mx = w, ...),
    qx = function(x, w, ...) LifeTable(x, qx = w, ...),
    dx = function(x, w, ...) LifeTable(x, dx = w, ...),
    lx = function(x, w, ...) LifeTable(x, lx = w, ...)
    )

  if (is.vector(data)) {
    if (length(x) != length(data))
      stop("The 'x' and 'data' do not have the same length", call. = FALSE)

    out <- LifeTable_foo(x = x, data, ...)$lt[, to]
    names(out) <- names(data)

  } else {
    if (length(x) != nrow(data))
      stop("The length of 'x' must be equal to the number of rows in 'data'",
           call. = FALSE)

    LT  <- function(D) LifeTable_foo(x = x, as.numeric(D), ...)$lt[, to]
    out <- apply(X = data, 2, FUN = LT)
    dimnames(out) <- dimnames(data)
  }

  return(out)
}

# --------------------------------------------------- #
# Author: Marius D. PASCARIU
# Last update: Sun May 02 16:37:15 2021
# --------------------------------------------------- #
remove(list = ls())

# Data ---
x  <- 0:105
mx <- ahmd$mx[paste0(x),]

# mx to qx
qx <- convertFx(x, data = mx, from = "mx", to = "qx")
# mx to dx
dx <- convertFx(x, data = mx, from = "mx", to = "dx")
# mx to lx
lx <- convertFx(x, data = mx, from = "mx", to = "lx")


# There are 28 possible combinations --------------------------------
# Let generate all of them.
from <- c("mx", "qx", "dx", "lx")
to   <- c("mx", "qx", "dx", "lx", "Lx", "Tx", "ex")
K    <- expand.grid(from = from, to = to) # all possible cases/combinations

for (i in 1:nrow(K)) {
  In  <- as.character(K[i, "from"])
  Out <- as.character(K[i, "to"])
  N <- paste0(Out, "_from_", In)
  cat(i, " Create", N, "\n")
  # Create the 28 sets of results
  assign(N, convertFx(x = x, data = get(In), from = In, to = Out))
}


# Tests -------------
expect_true(all(round(ex_from_dx - ex_from_lx, 10) == 0))
expect_true(all(round(ex_from_dx - ex_from_qx, 10) == 0))
expect_true(all(round(ex_from_dx - ex_from_mx, 10) == 0))


# Expect errors --------
expect_error(convertFx(x, data = mx, from = "mx", to = "qxx"))
expect_error(convertFx(x, data = mx, from = "mxx", to = "qx"))
expect_error(convertFx(10:15, data = mx, from = "mx", to = "qx"))
expect_error(convertFx(x, data = x, from = "mx", to = "qx"))

# ----------------------------------------------

expect_true(all(convertFx(x, data = mx[, 1], from = "mx", to = "qx") >= 0))


# All the possible errors associated with this functions should be solved
# in LifeTable(). convertFx is just a wrapper.

# ----------------------------------------------------------------------------
# Test messages

# Error: The length of 'x' must be equal to the numebr of rows in 'data'
expect_error(
  convertFx(x[1], data = mx, from = "mx", to = "qx")
)

# --------------------------------------------
# Author: Marius D PASCARIU
# Date: 2026-05-04 23:31:33
# --------------------------------------------

#' Download The Human Mortality Database (HMD)
#'
#' Download detailed mortality and population data for different countries
#' and regions in a single object from the \href{https://www.mortality.org/}{
#' Human Mortality Database}.
#'
#' @details
#' The Human Mortality Database (HMD) was created to provide detailed mortality
#' and population data to researchers, students, journalists, policy analysts,
#' and others interested in the history of human longevity. The project began
#' as an outgrowth of earlier projects in the Department of Demography at the
#' University of California, Berkeley, USA, and at the Max Planck Institute for
#' Demographic Research in Rostock, Germany (see history). It is the work of two
#' teams of researchers in the USA and Germany (see research teams), with the
#' help of financial backers and scientific collaborators from around the world
#' (see acknowledgements). The Center on the Economics and Development of Aging
#' (CEDA) French Institute for Demographic Studies (INED) has also supported the
#' further development of the database in recent years.
#'
#' @param what What type of data are you looking for? The following options
#' might be available for some or all the countries and regions: \itemize{
#'   \item{\code{"births"}} -- birth records;
#'   \item{\code{"Dx_lexis"}} -- deaths by Lexis triangles;
#'   \item{\code{"Ex_lexis"}} -- exposure-to-risk by Lexis triangles;
#'   \item{\code{"population"}} -- population size;
#'   \item{\code{"Dx"}} -- death counts;
#'   \item{\code{"Ex"}} -- exposure-to-risk;
#'   \item{\code{"mx"}} -- central death-rates;
#'   \item{\code{"LT_f"}} -- period life tables for females;
#'   \item{\code{"LT_m"}} -- period life tables for males;
#'   \item{\code{"LT_t"}} -- period life tables both sexes combined;
#'   \item{\code{"e0"}} -- period life expectancy at birth;
#'   \item{\code{"Exc"}} -- cohort exposures;
#'   \item{\code{"mxc"}} -- cohort death-rates;
#'   \item{\code{"LT_fc"}} -- cohort life tables for females;
#'   \item{\code{"LT_mc"}} -- cohort life tables for males;
#'   \item{\code{"LT_tc"}} -- cohort life tables both sexes combined;
#'   \item{\code{"e0c"}} -- cohort life expectancy at birth;
#'   }
#' @param countries Specify the country data you want to download by adding the
#' HMD country code/s. Options:
#' \code{ 
#' "AUS"    "AUT",    "BEL",   "BGR", 
#' "BLR",   "CAN",    "CHL",   "HRV",
#' "HKG",   "CHE",    "CZE",   "DEUTNP", 
#' "DEUTE", "DEUTW",  "DNK",   "ESP", 
#' "EST",   "FIN",    "FRATNP","FRACNP", 
#' "GRC",   "HUN",    "IRL",   "ISL"    
#' "ISR",   "ITA",    "JPN",   "KOR", 
#' "LTU",   "LUX",    "LVA",   "NLD",   
#' "NOR",   "NZL_NP", "NZL_MA" "NZL_NM", 
#' "POL",   "PRT"     "RUS",   "SVK", 
#' "SVN",   "SWE",    "TWN",   "UKR",   
#' "GBR_NP","GBRTENW","GBRCENW","GBR_SCO",
#' "GBR_NIR","USA"}.
#'  If \code{NULL} data for all the countries are downloaded at once;
#' @param interval Datasets are given in various age and time formats based on
#' which the records are aggregated. Interval options:
#' \itemize{
#'   \item{\code{"1x1"}} -- by age and year;
#'   \item{\code{"1x5"}} -- by age and 5-year time interval;
#'   \item{\code{"1x10"}} -- by age and 10-year time interval;
#'   \item{\code{"5x1"}} -- by 5-year age group and year;
#'   \item{\code{"5x5"}} -- by 5-year age group and 5-year time interval;
#'   \item{\code{"5x10"}} --by 5-year age group and 10-year time interval.
#'   }
#' @param username Your HMD username. If you don't have one you can sign up
#' for free on the Human Mortality Database website.
#' @param password Your HMD password.
#' @param save Do you want to save a copy of the dataset on your local machine?
#' Logical. Default: \code{FALSE}.
#' @param show Choose whether to display a progress bar. Logical.
#' Default: \code{TRUE}.
#' @return A \code{ReadHMD} object that contains:
#'  \item{input}{List with the input values (except the password).}
#'  \item{data}{Data downloaded from HMD.}
#'  \item{download.date}{Time stamp.}
#'  \item{years}{Numerical vector with the years covered in the data.}
#'  \item{ages}{Numerical vector with ages covered in the data.}
#' @author Marius D. Pascariu
#' @examples
#' \dontrun{
#'
#'
#' # Download demographic data for 3 countries in 1x1 format
#' age_int  <- 1  # age interval: 1,5
#' year_int <- 1  # year interval: 1,5,10
#' interval <- paste0(age_int, "x", year_int)  # --> 1x1
#' # And the 3 countries: Sweden Denmark and USA. We have to use the HMD codes
#' cntr  <- c('SWE', 'DNK', 'USA')
#'
#' # Download death counts. We don't want to export data outside R.
#' HMD_Dx <- ReadHMD(what = "Dx",
#'                   countries = cntr,
#'                   interval  = interval,
#'                   username  = "user@email.com",
#'                   password  = "password",
#'                   save = FALSE)
#' HMD_Dx
#'
#' # Download life tables for female population and export data.
#' LTF <- ReadHMD(what = "LT_f",
#'                countries = cntr,
#'                interval  = interval,
#'                username  = "user@email.com",
#'                password  = "password",
#'                save = TRUE)
#' LTF
#' }
#' @export
ReadHMD <- function(what, countries = NULL, interval = "1x1",
                    username, password, save = FALSE, show = TRUE){
  # Step 1 - Validate input & Progress bar setup
  if (is.null(countries)) {
    countries <- HMDcountries()
  }
  
  input <- list(what = what, countries = countries, interval = interval,
                username = username, save = save, show = show)
  check_input_ReadHMD(input)
  nr <- length(countries)
  
  if (show) {
    pb <- startpb(0, nr + 1)
    on.exit(closepb(pb))
    setpb(pb, 0)
  }
  
  # Step 2 - Do the loop for the other countries
  D <- data.frame()
  for (i in 1:nr) {
    if (show) {
      setpb(pb, i)
      cat(paste("      :Downloading", countries[i], "    "))
    }
    
    D <- rbind(D, ReadHMD.core(what, country = countries[i], interval,
                               username, password,
                               link = "https://www.mortality.org/File/GetDocument/hmd.v6/"))
  }
  
  if (length(D) != 0) {
    out <- list(input = input,
                data = D,
                download.date = date(),
                years = sort(unique(D$Year)),
                ages = unique(D$Age))
    out <- structure(class = "ReadHMD", out)
    
    # Step 3 - Write a file with the database in your working directory
    if (show) setpb(pb, nr + 1)
    if (save) saveOutput(out, show, prefix = "HMD")
    
  } else {
    out <- NULL
  }
  
  # Exit
  return(out)
}


#' Save Output in the working directory
#' @param out Output file
#' @inheritParams ReadHMD
#' @return No return value, called for side effects
#' @keywords internal
saveOutput <- function(out, show, prefix) {
  fn  <- paste0(prefix, "_", out$input$what) # file name
  assign(fn, value = out)
  save(list = fn, file = paste0(fn, ".Rdata"))
  if (show) saveMsg()
}


#' Print message when saving an object
#' @inherit saveOutput return
#' @keywords internal
saveMsg <- function() {
  wd  <- getwd()
  n   <- nchar(wd)
  wd_ <- paste0("...", substring(wd, first = n - 45, last = n))
  message(paste("\nThe dataset is saved in your working directory:\n  ", wd_),
          appendLF = FALSE)
  message("\nDownload completed!\n")
}


#' Function to Download Data for a one Country
#' @inheritParams ReadHMD
#' @param country HMD country code for the selected country. Character;
#' @param link the main link to the database.
#' @return A data.frame containing demographic data
#' @keywords internal
ReadHMD.core <- function(what, country, interval, username, password, link){
  
  if (what == "e0" & interval == "1x1") {
    whichFile <- "E0per"
    
  } else if (what == "e0c" & interval == "1x1"){
    whichFile <- "E0coh"
    
  } else {
    whichFile <- switch(what,
                        births = "Births",
                        population = "Population",
                        Dx_lexis = "Deaths_lexis",
                        Ex_lexis = "Exposures_lexis",
                        Dx   = paste0("Deaths_", interval),    # deaths
                        Ex   = paste0("Exposures_", interval), # exposure
                        mx   = paste0("Mx_", interval),        # death rates
                        LT_f = paste0("fltper_", interval),    # Life tables, Females
                        LT_m = paste0("mltper_", interval),    # Life tables, Males
                        LT_t = paste0("bltper_", interval),    # Life tables, Both sexes
                        e0   = paste0("E0per_", interval),     # Life expectancy
                        # Cohort data
                        Exc = paste0("cExposures_", interval),
                        mxc = paste0("cMx_", interval),
                        LT_fc = paste0("fltcoh_", interval),
                        LT_mc = paste0("mltcoh_", interval),
                        LT_tc = paste0("bltcoh_", interval),
                        e0c = paste0("E0coh_", interval)
    )}
  
  
  if (link %in% c("https://www.mortality.org/File/GetDocument/hmd.v6/",
                  "https://www.ipss.go.jp/p-toukei/JMD/")) {
    interlude <- "/STATS/"
    
  } else {
    interlude <- "/"
  }
  
  path     <- paste0(link, country, interlude, whichFile, ".txt")
  response <- ""
  
  if (is.null(username) | is.null(password)) {
    response <- try(silent = TRUE, GET(url = path))

  } else {
    # "We did not find the right method to login yet. Following the HMD website
    # change the line below no longer works."
    response <- try(silent = TRUE,
               RCurl::getURL(url = path, userpwd = paste0(username, ":", password))
    )
  }
  
  status_code_response <- if(is(response, "response")) status_code(response) else 404
  
  # Check if the request was successful (status code 200 means success)
  if (status_code_response == 200) {
    # Read the content of the text file
    txt <- content(response, "text", encoding = "UTF-8")
    
    con  <- try(textConnection(txt),
                stop("\nThe function failed to connect to ", link,
                     " Maybe the website is down at this moment?", call. = FALSE))
    
    JPNcodes <- substrRight(paste0(0, 0:47), 2)
    if (any(country %in% JPNcodes)) {
      country <- JPNregions()[as.numeric(country) + 1]
    }
    
        dat  <- try(read.table(con, skip = 2, header = TRUE, na.strings = "."),
                stop("\n", what, " data for ", country, " in the ", interval,
                     " format could not be found. Looked here:\n",
                     path, call. = FALSE))
    
    close(con)
    out <- cbind(country, dat)
    if (any(interval %in% c("1x1", "1x5", "1x10")) &
        !any(what %in% c("births", "Dx_lexis", "Ex_lexis", "e0", "e0c"))) {
      out$Age <- 0:110
    }
    
  } else {
    message("\nThe internet connection to ", link, " failed with status code:", 
            status_code_response, "!",
            "\nMaybe the website is down at this moment?")
    out <- NULL
  }
  
  return(out)
}


#' Country codes
#' @return a vector
#' @keywords internal
HMDcountries <- function() {
  c("AUS","AUT","BEL","BGR","BLR",
    "CAN","CHL","HRV","CHE","CZE",
    "DEUTNP","DEUTE", "DEUTW","DNK","ESP",
    "EST","FIN","FRATNP","FRACNP","GRC",
    "HUN", "HKG", "IRL","ISL", "ISR",
    "ITA","JPN","KOR","LTU","LUX",
    "LVA","NLD","NOR","NZL_NP","NZL_MA",
    "NZL_NM","POL","PRT","RUS","SVK",
    "SVN","SWE","TWN","UKR","GBR_NP",
    "GBRTENW", "GBRCENW","GBR_SCO","GBR_NIR","USA")
}

#' Data formats
#' @return a vector
#' @keywords internal
data_format <- function() c("1x1", "1x5", "1x10", "5x1", "5x5","5x10")


#' HMD Indices
#' @return a vector
#' @keywords internal
HMDindices <- function() c("births", "population", "Dx_lexis", "Ex_lexis", "Dx",
                           "mx", "Ex", "LT_f", "LT_m", "LT_t", "e0",
                           "mxc", "Exc", "LT_fc", "LT_mc", "LT_tc", "e0c")

#' Check input ReadHMD
#' @param x a list containing the input arguments from ReadHMD function
#' @return No return value, called for validating input data
#' @keywords internal
check_input_ReadHMD <- function(x) {
  coh_countries <- c("DNK", "FIN", "FRATNP", "FRACNP", "ISL", "ITA", "NLD",
                     "NOR", "SWE", "CHE", "GBRTENW", "GBRCENW", "GBR_SCO")
  
  if (!any(x$interval %in% data_format())) {
    stop("The interval ", x$interval, " does not exist in HMD ",
         "Try one of these options:\n", paste(data_format(), collapse = ", "),
         call. = FALSE)
  }
  
  if (!any(x$what %in% HMDindices())) {
    stop(x$what, " does not exist in HMD. Try one of these options:\n",
         paste(HMDindices(), collapse = ", "), call. = FALSE)
  }
  
  if (all(!(x$countries %in% HMDcountries()))) {
    stop("Something is wrong in the country/countries added by you.\n",
         "Try one or more of these options:\n",
         paste(HMDcountries(), collapse = ", "), call. = FALSE)
  }
  
  # Availability of Cohort Data
  if (any(x$what %in% c("LT_fc", "LT_mc", "LT_tc", "e0c")) &
      !(all(x$countries %in% coh_countries))) {
    stop("Data type ", x$what,
         " is not available for one or more countries specified in input.\n",
         "Check one of these countries:\n",
         paste(coh_countries, collapse = ", "), call. = FALSE)
  }
  
  # Availability of Life Expectancy Data
  if (any(x$what %in% c("e0", "e0c")) &
      !(x$interval %in% c("1x1", "1x5", "1x10"))) {
    stop("Data type ", x$what,
         " is available only in the following formats: '1x1', '1x5', '1x10'.",
         call. = FALSE)
  }
}


#' Print ReadHMD
#' @param x An object of class \code{"ReadHMD"}
#' @param ... Further arguments passed to or from other methods.
#' @return Print data on the console
#' @keywords internal
#' @export
print.ReadHMD <- function(x, ...){
  what <- x$input$what
  cat("Human Mortality Database (https://www.mortality.org)\n")
  cat("Downloaded by :", x$input$username, "\n")
  cat("Download Date :", x$download.date, "\n")
  cat("Type of data  :", what, "\n")
  cat(paste("Interval      :", x$input$interval, "\n"))
  cat(paste("Years         :", x$years[1], "--", rev(x$years)[1], "\n"))
  cat(paste("Ages          :", ageMsg(what, x), "\n"))
  cat("Countries     :", x$input$countries, "\n")
  cat("\nData:\n")
  print(head_tail(x$data, hlength = 5, tlength = 5))
}


#' What age(s) are we looking at?
#' @inheritParams print.ReadHMD
#' @return A scalar or character indicating age groups
#' @keywords internal
ageMsg <- function(what, x) {
  if (any(what %in% c("e0", "e0c"))) {
    0
    
  } else if (what %in% c("births")){
    "all ages"
    
  } else {
    paste(x$ages[1], "--", rev(x$ages)[1])
  }
}

# -------------------------------------------------------------- #
# Author: Marius D. PASCARIU
# Last Update: Mon Jan 15 18:12:12 2024
# -------------------------------------------------------------- #
remove(list = ls())

# Wrong user & password
expect_message(ReadHMD(what     = "Dx",
                       countries = "AUS",
                       username = "fake_user",
                       password = "fake_password"))

# Wrong index
expect_error(ReadHMD(what     = "DxDD",
                     countries = "AUS",
                     username = "username",
                     password = "password"))
# Wrong country
expect_error(ReadHMD(what      = "Dx",
                     countries = "AUSS",
                     username  = "username",
                     password  = "password"))

# Wrong interval
expect_error(ReadHMD(what      = "Dx",
                     countries = "AUS",
                     interval  = "1x50",
                     username  = "username",
                     password  = "password"))

# Wrong country for the index
expect_error(ReadHMD(what      = "LT_fc",
                     countries = "AUS",
                     interval  = "1x1",
                     username  = "username",
                     password  = "password"))

# Wrong interval for the index
expect_error(ReadHMD(what      = "e0",
                     countries = "SWE",
                     interval  = "5x1",
                     username  = "username",
                     password  = "password"))

expect_error(ReadHMD(what      = "e0c",
                     countries = "SWE",
                     interval  = "5x1",
                     username  = "username",
                     password  = "password"))

expect_output(
  print(HMD_sample)
)

# ----------------------------------------------------------------------------


# Marius D. Pascariu --- Sun Oct  7 14:37:24 2018 ------------------------------

# cntr <- c('SWE')
# int <- c("1x10")
# wht <- c("births", "population", "Dx_lexis", "Ex_lexis", "Dx",
#          "mx", "Ex", "LT_f", "LT_m", "LT_t", "e0",
#          "mxc", "Exc", "LT_fc", "LT_mc", "LT_tc", "e0c")
# u = "..."
# p = "..."
# expand.grid(wht, int)
#
#
# for (w in wht) {
#   cat(w, "\n")
#   assign(w, ReadHMD(what = w, countries = cntr, interval  = int,
#                     username  = u, password  = p))
# }
#
#
# births
# population
# Dx_lexis
# Ex_lexis
# Dx
# mx
# Ex
# LT_f
# LT_m
# LT_t
# e0
# mxc
# Exc
# LT_fc
# LT_mc
# LT_tc
# e0c

# --------------------------------------------
# Author: Marius D PASCARIU
# Date: 2026-05-04 23:30:55
# --------------------------------------------

#' Function to check input data in MortalityLaw
#' @inheritParams choose_optim
#' @return No return value, called for side effects
#' @keywords internal
#'
check.MortalityLaw <- function(input){
  with(input,
       {
         # Errors ---
         if (!is.logical(show)) stop("'show' should be TRUE or FALSE")

         if (!is.null(mx)) {
           if (length(x) != length(mx))
             stop('x and mx do not have the same length!', call. = FALSE)
         }

         if (!is.null(Dx)) {
           if (length(x) != length(Dx) | length(x) != length(Ex) )
             stop('x, Dx and Ex do not have the same length!', call. = FALSE)
         }

         function_to_optimize <- availableLF()$table[, 'CODE']
         if (!(opt.method %in% function_to_optimize)) {
           m1 <- 'Choose a different objective function to optimize\n'
           m2 <- 'Check one of the following options:\n'
           err2 <- paste(m1, m2, paste(function_to_optimize, collapse = ', '))
           stop(err2, call. = FALSE)
         }
         if (length(fit.this.x) < 2) {
           stop(paste("More observations needed in order to start the fitting.",
                      "Increase the length of 'fit.this.x'"), call. = FALSE)
         }
         if (!all(fit.this.x %in% x)) {
           stop("'fit.this.x' should be a subset of 'x'", call. = FALSE)
         }

         # Messages ---
                  if (law %in% c('HP', 'HP2', 'HP3', 'HP4', 'kostaki') & opt.method != "LF2") {
           message("\nFor models like ", law, ", the optimisation method 'LF2'",
                   " has been observed to return reliable estimates.")
         }
       })
}

# --------------------------------------------
# Author: Marius D PASCARIU
# Date: 2026-05-04 23:31:17
# --------------------------------------------

#' Check Available Mortality Laws
#'
#' The function returns information about the parametric models that can be
#' called and fitted in the \code{\link{MortalityLaw}} function.
#' For a comprehensive review of the most important mortality laws,
#' Tabeau (2001) is a good starting point.
#'
#' @param law Optional. Default: \code{NULL}. One can extract details about
#' a certain model by specifying its codename.
#' @return The output is of the \code{"availableLaws"} class with the following
#' components:
#'  \item{table}{Table with mortality models and codes to be used in \code{\link{MortalityLaw}}.}
#'  \item{legend}{Table with details about the section of the mortality curve.}
#' @references
#' \enumerate{
#' \item{Gompertz, B. (1825). \href{https://www.jstor.org/stable/107756}{On the
#' Nature of the Function Expressive of the Law of Human Mortality, and on a
#' New Mode of Determining the Value of Life Contingencies.}
#' Philosophical Transactions of the Royal Society of London, 115, 513-583.}
#' \item{Makeham, W. (1860).
#' On the Law of Mortality and Construction of Annuity Tables.
#' The Assurance Magazine and Journal of the Institute of Actuaries, 8(6),
#' 301-310. \doi{10.1017/S204616580000126X}}
#' \item{Thiele, T. (1871). On a Mathematical Formula to express the
#' Rate of Mortality throughout the whole of Life, tested by a Series of
#' Observations made use of by the Danish Life Insurance Company of 1871.
#' Journal of the Institute of Actuaries and Assurance Magazine, 16(5), 313-329.
#' \doi{10.1017/S2046167400043688}}
#' \item{Oppermann, L. H. F. (1870). On the graduation of life tables,
#' with special application to the rate of mortality in infancy and childhood.
#' The Insurance Record Minutes from a meeting in the Institute of Actuaries, 42.}
#' \item{Wittstein, T. and D. Bumsted. (1883).
#' \href{https://www.cambridge.org/core/journals/journal-of-the-institute-of-actuaries/article/the-mathematical-law-of-mortality/57A7403B578C84769A463EA2BC2F7ECD}{
#' The Mathematical Law of Mortality.}
#' Journal of the Institute of Actuaries and Assurance Magazine, 24(3), 153-173.}
#' \item{Steffensen, J. (1930). Infantile mortality from an actuarial point of
#' view. Skandinavisk Aktuarietidskrift 13, 272-286.
#' \doi{10.1080/03461238.1930.10416902}}
#' \item{Perks, W. (1932).
#' On Some Experiments in the Graduation of Mortality Statistics.
#' Journal of the Institute of Actuaries, 63(1), 12-57.
#' \doi{10.1017/S0020268100046680}}
#' \item{Harper, F. S. (1936). An actuarial study of infant mortality.
#' Scandinavian Actuarial Journal 1936 (3-4), 234-270.
#' \doi{10.1080/03461238.1936.10405113}}
#' \item{Weibull, W. (1951). A statistical distribution function of wide applicability.
#' Journal of applied mechanics 103, 293-297.
#'  \doi{10.1115/1.4010337}}
#' \item{Beard, R. E. (1971).
#' \href{http://longevity-science.org/Beard-1971.pdf}{
#' Some aspects of theories of mortality, cause of
#' death analysis, forecasting and stochastic processes.}
#' Biological aspects of demography 999, 57-68.}
#' \item{Vaupel, J., Manton, K.G., and Stallard, E. (1979).
#' The impact of heterogeneity in individual frailty on the dynamics of
#' mortality. Demography 16(3): 439-454.
#' \doi{10.2307/2061224}}
#' \item{Siler, W. (1979),
#' A Competing-Risk Model for Animal Mortality. Ecology, 60: 750-757.
#' \doi{10.2307/1936612}}
#' \item{Heligman, L., & Pollard, J. (1980). The age pattern of mortality.
#' Journal of the Institute of Actuaries, 107(1), 49-80.
#' \doi{10.1017/S0020268100040257}}
#' \item{Rogers A and Planck F (1983).
#' \href{https://pure.iiasa.ac.at/id/eprint/2210/1/WP-83-102.pdf}{
#' MODEL: A General Program for Estimating Parametrized Model Schedules of Fertility,
#' Mortality, Migration, and Marital and Labor Force Status Transitions.}
#' IIASA Working Paper. IIASA, Laxenburg, Austria: WP-83-102}
#' \item{Martinelle S. (1987). A generalized Perks formula for old-age mortality.
#' Stockholm, Sweden, Statistiska Centralbyran, 1987. 55 p.
#' (R&D Report, Research-Methods-Development, U/STM No. 38)}
#' \item{Carriere J.F. (1992). Parametric models for life tables.
#' Transactions of the Society of Actuaries. Vol.44}
#' \item{Kostaki A. (1992).
#' A nine-parameter version of the Heligman-Pollard formula.
#' Mathematical Population Studies. Vol. 3 277-288.
#' \doi{10.1080/08898489209525346}}
#' \item{Thatcher AR, Kannisto V and Vaupel JW (1998).
#' The force of mortality at ages 80 to 120. Odense Monographs on
#' Population Aging Vol. 5, Odense University Press, 1998. 104, 20 p.
#' Odense, Denmark}
#' \item{Tabeau E. (2001). A Review of Demographic Forecasting Models for
#' Mortality. In: Tabeau E., van den Berg Jeths A., Heathcote C. (eds)
#' Forecasting Mortality in Developed Countries.
#' European Studies of Population, vol 9. Springer, Dordrecht.
#' \doi{10.1007/0-306-47562-6_1}}
#' \item{Finkelstein M. (2012)
#' Discussing the Strehler-Mildvan model of mortality
#' Demographic Research, Vol. 26(9), 191-206.
#' \doi{10.4054/DemRes.2012.26.9}}
#' }
#' @seealso \code{\link{MortalityLaw}}
#' @author Marius D. Pascariu
#' @examples
#' availableLaws()
#' @export
availableLaws <- function(law = NULL){

  if (is.null(law)) {

    table <- as.data.frame(
      matrix(
        ncol = 7,
        byrow = TRUE,
        data = c(
          1825, 'Gompertz', 'mu[x] = A exp[Bx]', 3, 'gompertz', 'mu[x]', TRUE,
          NA, 'Gompertz', 'mu[x] = 1/sigma * exp[(x-M)/sigma)]', 3, 'gompertz0', 'mu[x]', TRUE,
          NA, 'Inverse-Gompertz', 'mu[x] = [1- exp(-(x-M)/sigma)] / [exp(-(x-M)/sigma) - 1]', 2, 'invgompertz', 'mu[x]', TRUE,
          1860, 'Makeham', 'mu[x] = A exp[Bx] + C', 3, 'makeham', 'mu[x]', TRUE,
          NA, 'Makeham', 'mu[x] = 1/sigma * exp[(x-M)/sigma)] + C', 3, 'makeham0', 'mu[x]', TRUE,
          1870, 'Opperman', 'mu[x] = A/sqrt(x) - B + C*sqrt(x)', 1, 'opperman', 'mu[x]', FALSE,
          1871, 'Thiele', 'mu[x] = A exp(-Bx) + C exp[-.5D (x-E)^2] + F exp(Gx)', 6, 'thiele', 'mu[x]', F,
          1883, 'Wittstein', 'q[x] = (1/B) A^-[(Bx)^N] + A^-[(M-x)^N]', 6, 'wittstein', 'q[x]', FALSE,
          1932, 'Perks', 'mu[x] = [A + BC^x] / [BC^-x + 1 + DC^x]', 3, 'perks', 'mu[x]', TRUE,
          1939, 'Weibull', 'mu[x] = 1/sigma * (x/M)^(M/sigma - 1)', 1, 'weibull', 'mu[x]', FALSE,
          NA, 'Inverse-Weibull', 'mu[x] = 1/sigma * (x/M)^[-M/sigma - 1] / [exp((x/M)^(-M/sigma)) - 1]', 2, 'invweibull', 'mu[x]', TRUE,
          1943, 'Van der Maen', 'mu[x] = A + Bx + Cx^2 + I/[N - x]', 4, 'vandermaen', 'mu[x]', TRUE,
          1943, 'Van der Maen', 'mu[x] = A + Bx + I/[N - x]', 5, 'vandermaen2', 'mu[x]', TRUE,
          1960, 'Strehler-Mildvan', 'mu[x] = K * exp[-V0 * (1 - Bx)/D]', 3, 'strehler_mildvan', 'mu[x]', TRUE,
          NA, 'Quadratic', 'mu[x] = A + Bx + Cx^2', 5, 'quadratic', 'mu[x]', TRUE,
          1971, 'Beard', 'mu[x] = A exp(Bx) / [1 + KA exp(Bx)]', 4, 'beard', 'mu[x]', TRUE,
          1971, 'Beard-Makeham', 'mu[x] = A exp(Bx) / [1 + KA exp(Bx)] + C', 4, 'beard_makeham', 'mu[x]', TRUE,
          1979, 'Gamma-Gompertz', 'mu[x] = A exp(Bx) / (1 + AG/B * [exp(Bx) - 1])', 4, 'ggompertz', 'mu[x]', TRUE,
          1979, 'Siler', 'mu[x] = A exp(-Bx) + C + D exp(Ex)', 6, 'siler', 'mu[x]', FALSE,
          1980, 'Heligman-Pollard', 'q[x]/p[x] = A^[(x + B)^C] + D exp[-E log(x/F)^2] + G H^x', 6, 'HP', 'q[x]', FALSE,
          1980, 'Heligman-Pollard', 'q[x] = A^[(x + B)^C] + D exp[-E log(x/F)^2] + GH^x / [1 + GH^x]', 6, 'HP2', 'q[x]', FALSE,
          1980, 'Heligman-Pollard', 'q[x] = A^[(x + B)^C] + D exp[-E log(x/F)^2] + GH^x / [1 + KGH^x]', 6, 'HP3', 'q[x]', FALSE,
          1980, 'Heligman-Pollard', 'q[x] = A^[(x + B)^C] + D exp[-E log(x/F)^2] + GH^(x^K) / [1 + GH^(x^K)]', 6, 'HP4', 'q[x]', FALSE,
          1983, 'Rogers-Planck', 'q[x] = A0 + A1 exp[-Ax] + A2 exp[B(x - u) - exp(-C(x - u))] + A3 exp[Dx]', 6, 'rogersplanck', 'q[x]', FALSE,
          1987, 'Martinelle', 'mu[x] = [A exp(Bx) + C] / [1 + D exp(Bx)] + K exp(Bx)', 6, 'martinelle', 'mu[x]', FALSE,
          1992, 'Carriere', 'l[x] = P1 l[x](weibull) + P2 l[x](invweibull) + P3 l[x](gompertz)', 6, 'carriere1', 'q[x]', TRUE,
          1992, 'Carriere', 'l[x] = P1 l[x](weibull) + P2 l[x](invgompertz) + P3 l[x](gompertz)', 6, 'carriere2', 'q[x]', TRUE,
          1992, 'Kostaki', 'q[x]/p[x] = A^[(x+B)^C] + D exp[-(E_i log(x/F_))^2] + G H^x', 6, 'kostaki', 'q[x]', FALSE,
          1998, 'Kannisto', 'mu[x] = A exp(Bx) / [1 + A exp(Bx)]', 5, 'kannisto', 'mu[x]', TRUE,
          1998, 'Kannisto-Makeham', 'mu[x] = A exp(Bx) / [1 + A exp(Bx)] + C', 5, 'kannisto_makeham', 'mu[x]', TRUE
          )
        )
      )

    colnames(table) <- c(
      'YEAR',
      'NAME',
      'MODEL',
      'TYPE',
      'CODE',
      'FIT',
      "SCALE_X"
      )

    legend <- as.data.frame(
      matrix(
        ncol = 2,
        byrow = TRUE,
        data = c(
          1, "Infant mortality",
          2, "Accident hump",
          3, "Adult mortality",
          4, "Adult and/or old-age mortality",
          5, "Old-age mortality",
          6, "Full age range"
          )
        )
      )

    colnames(legend) <- c("TYPE", "Coverage")
  }

  if (!is.null(law)) {
    A <- availableLaws()
    if (!(law %in% A$table$CODE)) {
      stop(
        "The specified 'law' is not available. ",
        "Run 'availableLaws()' to see the implemented models.",
        call. = FALSE)
    }
    table <- A$table[A$table$CODE %in% law, ]
    legend <- A$legend[A$legend$TYPE %in% unique(table$TYPE), ]
  }

  out <- structure(
    class = "availableLaws",
    list(table = table, legend = legend)
    )
  return(out)
}


#' Print availableLaws
#' @param x An object of class \code{"availableLaws"}
#' @param ... Further arguments passed to or from other methods.
#' @return print info on the console
#' @keywords internal
#' @export
print.availableLaws <- function(x, ...) {
  cat("\nMortality laws available in the package:\n\n")
  print(
    x$table[, 1:5],
    right = FALSE,
    row.names = FALSE
    )

  cat("\nLEGEND:\n")
  print(
    x$legend,
    right = FALSE,
    row.names = FALSE
    )
}

# --------------------------------------------------- #
# Author: Marius D. PASCARIU
# Last update: Sun May 02 12:09:25 2021
# --------------------------------------------------- #
remove(list = ls())

# test 1: ---------------------------------------
# Test all models on with ages
yr <- 1950
ages <- list(infancy   = 0:15,
             hump      = 16:30,
             adulthood = 30:75,
             adult_old = 30:100,
             old_age   = 76:100,
             full      = 0:100)

aLaws <- availableLaws()
N     <- nrow(aLaws$table)

k = 30
# Build M models
for (k in 1:N) {
  type <- as.numeric(aLaws$table$TYPE[k])
  X    <- c(ages[type][[1]])
  mx   <- ahmd$mx[paste(X), ]
  sx   <- ifelse(min(X) > 1, TRUE, FALSE)
  LAW  <- as.character(aLaws$table$CODE[k])
  cat("M", k,": ", LAW, "\n", sep = "")

  assign(paste0("M", k), MortalityLaw(x   = X,
                                      mx  = mx[, 1:1],
                                      law = LAW,
                                      opt.method = 'LF2'))

  assign(paste0("P", k), MortalityLaw(x   = X,
                                      mx  = mx[, 1:2],
                                      law = LAW,
                                      opt.method = 'LF2'))
}


testMortalityLaw <- function(Y){
  test_that("Test MortalityLaw function", {
    expect_s3_class(Y, "MortalityLaw")
    expect_output(print(Y))
    expect_output(print(summary(Y)))
    expect_true(all(fitted(Y) >= 0))
    expect_true(all(coef(Y) >= 0))
    pred = predict(Y, x = Y$input$x)
    expect_true(all(pred >= 0))

    if (is.matrix(fitted(Y))) {
      expect_error(plot(Y))
      expect_true(is.matrix(pred))
    } else {
      expect_false(is.null(plot(Y)))
    }
  })
}


for (i in 1:N) testMortalityLaw(get(paste0("M", i)))

for (j in 1:N) testMortalityLaw(get(paste0("P", j)))

# ----------------------------------------------------------------------------
# test qx fit and pb
testMortalityLaw(
  MortalityLaw(x   = X,
               qx  = mx,
               law = LAW,
               opt.method = 'LF2',
               show = TRUE)
)

# test 2: ---------------------------------------
# fit.this.x

x  <- 45:75
Dx <- ahmd$Dx[paste(x), paste(yr)]
Ex <- ahmd$Ex[paste(x), paste(yr)]
T2 <- MortalityLaw(x   = x - 44,
                   Dx  = Dx,
                   Ex  = Ex,
                   law = 'makeham',
                   fit.this.x = 50:70 - 44)

testMortalityLaw(T2)
expect_error(
  MortalityLaw(x   = x,
               Dx  = Dx,
               Ex  = Ex,
               law = 'makeham',
               fit.this.x = 48)
  )


expect_error(
  MortalityLaw(x   = x,
    Dx  = Dx,
    Ex  = Ex,
    law = 'makeham',
    fit.this.x = 40:80)
  )

# Test 3: ---------------------------------------
# custom.law
my_gompertz <- function(x, par = c(b = 0.13, m = 45)){
  hx <- with(as.list(par), b*exp(b*(x - m)) )
  return(as.list(environment()))
}

T3 = MortalityLaw(x  = x,
                  Dx = Dx,
                  Ex = Ex,
                  custom.law = my_gompertz)
testMortalityLaw(T3)

# test 4: ---------------------------------------
mx  <- ahmd$mx[paste(0:100), 1] # select data
expect_message((HP4 = MortalityLaw(x   = 0:100,
                                   mx  = mx,
                                   law = 'HP',
                                   opt.method = "poissonL")))

expect_error(predict(HP4, x = -1:100))
expect_true(is.numeric(AIC(HP4)))
expect_true(is.numeric(logLik(HP4)))
expect_true(is.numeric(df.residual(HP4)))
expect_true(is.numeric(deviance(HP4)))
expect_true(class(summary(HP4)) == "summary.MortalityLaw")

# test 5: ---------------------------------------
# Test that all the laws return positive values
L <- availableLaws()
laws <- L$table$CODE

for (i in laws) {
  hx = eval(call(i, x = 1:100))$hx
  expect_true(all(hx >= 0))
  expect_false(any(is.na(hx)))
}

# test 6: ---------------------------------------
# Test error messages
x <- 0:100
mx <- ahmd$mx[paste(x), 1] # select data
Dx = ahmd$Dx[paste(x), 1]
Ex = ahmd$Ex[paste(x), 1]

expect_error(MortalityLaw(x, mx = mx))
expect_error(MortalityLaw(x, mx = mx, law = 'law_not_available'))
expect_error(MortalityLaw(x, mx = mx, law = 'HP', opt.method = "LF_not_available"))
expect_error(MortalityLaw(x, mx = mx, law = 'HP', show = "TRUEx"))
expect_error(MortalityLaw(0:1000, mx = mx, law = 'HP'))
expect_error(MortalityLaw(x, Dx = Dx, Ex = Ex[-1], law = 'HP'))
expect_error(MortalityLaw(x, Dx = Dx[-1], Ex = Ex, law = 'HP'))

# -------------------------------------------------------------- #
# Author: Marius D. PASCARIU
# Last Update: Wed Jul 19 17:43:27 2023
# -------------------------------------------------------------- #
remove(list = ls())

AL = availableLaws()
expect_true(class(AL) == "availableLaws")
expect_false(is.null(AL$table))
expect_false(is.null(AL$legend))
expect_output(print(AL))

AL2 = availableLaws(law = 'rogersplanck')
expect_output(print(AL2))


A <- availableLaws()
law <- 'rogersplanck'


expect_error(availableLaws(law = "notavailable"))


AF = availableLF()
expect_true(class(AF) == "availableLF")
expect_false(is.null(AF$table))
expect_false(is.null(AF$legend))
expect_output(print(AF))

# -------------------------------------------------------------- #
# Author: Marius D. PASCARIU
# Last Update: Thu Jul 20 22:06:31 2023
# -------------------------------------------------------------- #


#' Summary function - display head and tail in a single data.frame
#' The original code for this function was first written for 'psych' R package
#' here we have modified it a bit
#' @param x A matrix or data frame or free text
#' @param hlength The number of lines at the beginning to show
#' @param tlength The number of lines at the end to show
#' @param digits Round off the data to digits
#' @param ellipsis separate the head and tail with dots
#' @return Print table's head and tail
#' @keywords internal
head_tail <- function(x, hlength = 4, tlength = 4, digits = 4, ellipsis = TRUE){
  if (is.data.frame(x) | is.matrix(x)) {
    if (is.matrix(x)) x = data.frame(unclass(x))
    nvar <- dim(x)[2]
    dots <- rep("...", nvar)
    h    <- data.frame(head(x, hlength))
    t    <- data.frame(tail(x, tlength))
    for (i in 1:nvar) {
      if (is.numeric(h[1, i])) {
        h[i] <- round(h[i], digits)
        t[i] <- round(t[i], digits)
      } else {
        dots[i] <- NA
      }
    }
    out <- if (ellipsis) rbind(h, ... = dots, t) else rbind(h, t)
  } else {
    h <- head(x, hlength)
    t <- tail(x, tlength)
    out <- paste(paste(h, collapse = " "), "...   ...", paste(t, collapse = " "))
  }
  return(out)
}


#' Extracting the last n characters from a string
#' @param x a string
#' @param n number of characters
#' @return A character vector of length n
#' @keywords internal
substrRight <- function(x, n){
  substr(x, nchar(x) - n + 1, nchar(x))
}

# --------------------------------------------
# Author: Marius D PASCARIU
# Date: 2026-05-04 23:33:16
# --------------------------------------------

#' Fit Mortality Laws
#'
#' Fit parametric mortality models given a set of input data. The data can be
#' supplied as death counts and mid-interval population estimates
#' \code{(Dx, Ex)}, age-specific death rates \code{(mx)}, or death
#' probabilities \code{(qx)}. Use the \code{law} argument to specify the
#' model to be fitted. Over 30 parametric models are currently implemented;
#' run \code{\link{availableLaws}} to see the full list. Models can be fitted
#' using maximum likelihood or by optimising a loss function. See the
#' \code{\link{availableLF}} function for the implemented options.
#'
#' @usage
#' MortalityLaw(x, Dx = NULL, Ex = NULL, mx = NULL, qx = NULL,
#'                 law = NULL,
#'                 opt.method = "LF2",
#'                 parS = NULL,
#'                 fit.this.x = x,
#'                 custom.law = NULL,
#'                 show = FALSE, ...)
#'
#' @details
#' \strong{Optimisation:} The PORT routines (via \code{\link{nlminb}}) are used
#' for unconstrained and box-constrained optimisation. Parameters are estimated
#' on the log scale to ensure positivity, and the routine is set to allow up to
#' 5000 iterations. When the optimisation method is \code{"poissonL"} or
#' \code{"binomialL"}, the AIC, BIC and log-likelihood are computed from the
#' likelihood. Otherwise these are set to \code{NaN}.
#'
#' \strong{Scaling of the age vector:} For models that cover only a portion
#' of the lifespan (e.g., adult or old-age mortality), the age vector \code{x}
#' is automatically re-scaled as \code{x = x - min(x) + 1} before fitting.
#' This transformation improves numerical stability and helps the optimisation
#' algorithm converge, especially when the starting age is far from zero.
#' Models that apply this scaling are flagged with \code{SCALE_X = TRUE} in
#' the table returned by \code{\link{availableLaws}}. When using
#' \code{\link{predict.MortalityLaw}} or \code{\link{LawTable}} with such
#' models, the same scaling is applied internally, so predictions remain
#' consistent with the fitted coefficients.
#'
#' \strong{Handling matrix input:} If \code{Dx}, \code{Ex}, \code{mx} or
#' \code{qx} are provided as matrices (with one column per population or
#' time period), the function iterates over the columns and fits a separate
#' model to each, returning a collection of results.
#' @inheritParams LifeTable
#' @param law The name of the mortality law to be used (e.g., \code{"gompertz"},
#' \code{"makeham"}). Run \code{\link{availableLaws}} to see all options.
#' @param opt.method The function to optimise. Available options:
#' \itemize{
#'   \item{\code{"poissonL"}: Poisson log-likelihood.}
#'   \item{\code{"binomialL"}: Binomial log-likelihood.}
#'   \item{\code{"LF1"}: Squared relative error \code{(1 - mu/nu)^2}.}
#'   \item{\code{"LF2"}: Squared log-ratio \code{log(mu/nu)^2}.}
#'   \item{\code{"LF3"}: Chi-squared-type \code{((nu - mu)^2)/nu}.}
#'   \item{\code{"LF4"}: Squared error \code{(nu - mu)^2}.}
#'   \item{\code{"LF5"}: Deviance-type \code{(nu - mu) * log(nu/mu)}.}
#'   \item{\code{"LF6"}: Absolute error \code{abs(nu - mu)}.}
#' }
#' See \code{\link{availableLF}} for details.
#' @param parS Optional starting parameter values for the optimisation. If
#' \code{NULL}, sensible defaults are automatically chosen via
#' \code{\link{bring_parameters}}.
#' @param fit.this.x A subset of \code{x} over which to fit the model. The
#' default is the entire \code{x} vector. Use this to exclude, for example,
#' advanced ages where data are sparse.
#' @param custom.law A user-defined function for fitting a model not included
#' in the package. The function must accept arguments \code{x} (age vector)
#' and \code{par} (named parameter vector) and return a list containing at
#' least an element named \code{hx} (the hazard or force of mortality). See
#' the examples below.
#' @param show Logical. If \code{TRUE}, a progress bar is displayed during
#' fitting. Default: \code{FALSE}.
#' @param ... Additional arguments passed to or from other methods.
#' @return
#' An object of class \code{"MortalityLaw"}, which is a list with the following
#' components:
#' \item{input}{List of input arguments, stored for reproducibility.}
#' \item{info}{Model information (name, formula, date of fitting).}
#' \item{coefficients}{Estimated parameters of the mortality law. A named
#' vector for a single fit, or a matrix for multiple fits.}
#' \item{fitted.values}{Fitted hazard rates (or death probabilities) evaluated
#' at the input ages \code{x}.}
#' \item{residuals}{Deviance residuals, computed as observed minus fitted
#' values.}
#' \item{goodness.of.fit}{List or matrix of goodness-of-fit measures: AIC,
#' BIC and log-likelihood (available only for likelihood-based methods).}
#' \item{opt.diagnosis}{Object returned by the optimisation routine, useful
#' for checking convergence.}
#' \item{df}{Number of parameters and residual degrees of freedom.}
#' \item{deviance}{Sum of squared log-residuals, used as a deviance measure.}
#' @seealso
#' \code{\link{availableLaws}} for a list of all implemented models;
#' \code{\link{availableLF}} for loss function details;
#' \code{\link{LifeTable}} for life table construction;
#' \code{\link{ReadHMD}} for downloading data from the Human Mortality Database.
#' @author Marius D. Pascariu
#' @examples
#' # Example 1: Fitting the Makeham model --------------------------
#' x  <- 45:75
#' Dx <- ahmd$Dx[paste(x), "1950"]
#' Ex <- ahmd$Ex[paste(x), "1950"]
#'
#' M1 <- MortalityLaw(x = x, Dx = Dx, Ex = Ex, law = 'makeham')
#'
#' M1
#' ls(M1)
#' coef(M1)
#' summary(M1)
#' fitted(M1)
#' predict(M1, x = 45:95)
#' plot(M1)
#'
#'
#' # Example 2: --------------------------
#' # We can fit the same model using a different data format
#' # and a different optimization method.
#' x  <- 45:75
#' mx <- ahmd$mx[paste(x), ]
#' M2 <- MortalityLaw(x = x, mx = mx, law = 'makeham', opt.method = 'LF1')
#' M2
#' fitted(M2)
#' predict(M2, x = 55:90)
#'
#' # Example 3: --------------------------
#' # Now let's fit a mortality law that is not defined
#' # in the package, say a reparameterized Gompertz in
#' # terms of modal age at death
#' # hx = b*exp(b*(x-m)) (here b and m are the parameters to be estimated)
#'
#' # A function with 'x' and 'par' as input has to be defined, which returns
#' # at least an object called 'hx' (hazard rate).
#' my_gompertz <- function(x, par = c(b = 0.13, M = 45)){
#'   hx  <- with(as.list(par), b*exp(b*(x - M)) )
#'   return(as.list(environment()))
#' }
#'
#' M3 <- MortalityLaw(x = x, Dx = Dx, Ex = Ex, custom.law = my_gompertz)
#' summary(M3)
#' # predict M3 for different ages
#' predict(M3, x = 85:130)
#'
#'
#' # Example 4: --------------------------
#' # Fit Heligman-Pollard model for a single
#' # year in the dataset between age 0 and 100 and build a life table.
#'
#' x  <- 0:100
#' mx <- ahmd$mx[paste(x), "1950"] # select data
#' M4 <- MortalityLaw(x = x, mx = mx, law = 'HP', opt.method = 'LF2')
#' M4
#' plot(M4)
#'
#' LifeTable(x = x, qx = fitted(M4))
#' @export
MortalityLaw <- function(x,
                         Dx = NULL,
                         Ex = NULL,
                         mx = NULL,
                         qx = NULL,
                         law = NULL,
                         opt.method = 'LF2',
                         parS = NULL,
                         fit.this.x = x,
                         custom.law = NULL,
                         show = FALSE,
                         ...){

  info    <- addDetails(law, custom.law, parS)
  law     <- info$law
  scale.x <- info$scale.x
  parS    <- info$parS
  input   <- c(as.list(environment()))
  K       <- find.my.case(Dx, Ex, mx, qx)

  # TR: if inputs are matrix, then we have class matrix, array, and this
  # throws a warning. If we have a dim attribute then this won't work. Even
  # if it's a 1-column matrix (class =array) or a single-dim length-attribute vector
  # (also array!). Both of those cases are probably things we want to treat as
  # vectors!
  if (any(K$iclass == "numeric")) {

    check.MortalityLaw(input) # Check input

    # Set-up progress bar
    if (show) {pb <- startpb(0, 4); on.exit(closepb(pb)); setpb(pb, 1)}

    # Find optimal coefficients
    optim.model <- choose_optim(input)
    if (show) setpb(pb, 2)

    fit   <- optim.model$hx
    dgn   <- optim.model$opt #diagnosis
    cf    <- optim.model$C
    p     <- length(cf)

    resid <- switch(K$case,
      C1_DxEx = Dx/Ex - fit,
      C2_mx = mx - fit,
      C3_qx = qx - fit
      )
    # Deviance is computed as the sum of squared log-residuals
    dev <- switch(K$case,
      C1_DxEx = log(Dx/Ex) - log(fit),
      C2_mx   = log(mx) - log(fit),
      C3_qx   = log(qx) - log(fit)
      )
    dev  <- sum(dev^2)

    rdf  <- length(x) - p
    df   <- c(n.param = p, df.residual = rdf)
    gof  <- with(
      optim.model,
      c(logLik = logLik, AIC = AIC, BIC = BIC)
      )
    info <- list(model.info = info$model, process.date = date())
    names(fit) = names(resid) <- x

    if (show) setpb(pb, 4)

  } else {# if input is a matrix then iterate here

    N  <- K$nLT
    # Set-up progress bar
    if (show) {pb <- startpb(0, N + 1); on.exit(closepb(pb))}
    cf = fit = gof = resid = dgn = df = dev <- NULL

    for (i in 1:N) {
      if (show) setpb(pb, i)
      M <- suppressMessages(
        MortalityLaw(
          x = x,
          Dx = Dx[, i],
          Ex = Ex[, i],
          mx = mx[, i],
          qx = qx[, i],
          law = law,
          opt.method = opt.method,
          parS = parS,
          fit.this.x = fit.this.x,
          custom.law = custom.law,
          show = FALSE
          )
        )
      fit      <- cbind(fit, fitted(M))
      gof      <- rbind(gof, M$goodness.of.fit)
      dgn[[i]] <- M$dgn
      cf       <- rbind(cf, coef(M))
      resid    <- cbind(resid, resid(M))
      df       <- rbind(df, M$df)
      dev      <- c(dev, M$dev)
    }

    info <- M$info
    rownames(cf)  = rownames(gof) = rownames(df) = names(dev) <- K$LTnames
    dimnames(fit) = dimnames(resid) <- list(x, K$LTnames)
    if (show) setpb(pb, N + 1)
  }

  # Exit
  output <- list(
    input = input,
    info = info,
    coefficients = cf,
    fitted.values = fit,
    residuals = resid,
    goodness.of.fit = gof,
    opt.diagnosis = dgn,
    df = df,
    deviance = dev
    )
  output$info$call <- match.call()
  out <- structure(class = "MortalityLaw", output)
  return(out)
}

#' Retrieve model-specific details for fitting
#'
#' Based on the chosen mortality law (or a custom law), this function retrieves
#' the default starting parameters, the model information table, and whether
#' the age vector should be scaled before fitting.
#' @inheritParams MortalityLaw
#' @return A list with components:
#' \item{law}{Internal law name (e.g., \code{"custom.law"} for user-supplied functions).}
#' \item{parS}{Starting parameter values for the optimisation.}
#' \item{model}{Model information data frame (from \code{\link{availableLaws}})
#' or \code{"Custom Mortality Law"} for user-defined models.}
#' \item{scale.x}{Logical; whether the age vector should be re-scaled
#' (\code{x = x - min(x) + 1}) before fitting.}
#' @keywords internal
addDetails <- function(law,
                       custom.law = NULL,
                       parS = NULL) {

  if (is.null(law) & is.null(custom.law)) {
    stop("Which mortality law do you intend to fit?", call. = FALSE)
  }

  if (!is.null(custom.law)) {
    law  <- "custom.law"
    parS <- custom.law(1)$par
    MI   <- "Custom Mortality Law"
    sx   <- TRUE

  } else {
    law  <- law
    parS <- parS
    A    <- availableLaws(law)[["table"]]
    MI   <- data.frame(A[A$CODE == law, ], row.names = "")
    sx   <- as.logical(MI$SCALE_X)
  }

  out <- list(
    law = law,
    parS = parS,
    model = MI,
    scale.x = sx
    )
  return(out)
}

#' Objective function to minimise during optimisation
#'
#' Given a set of parameters (on the log scale), this function evaluates the
#' chosen loss function or negative log-likelihood by comparing observed
#' mortality values (Dx/Ex, mx, or qx) against the hazard rates predicted by
#' the specified mortality law.
#'
#' Parameters are transformed back to the original scale via \code{exp(par)}
#' when calling the mortality law function. Infinite hazard values are capped
#' to 1, and large penalties are applied for missing or out-of-range values
#' to guide the optimiser away from invalid regions.
#'
#' @inheritParams MortalityLaw
#' @param par Parameter vector on the log scale.
#' @return A scalar loss value to be minimised.
#' @keywords internal
objective_fun <- function(par, x, Dx, Ex, mx, qx,
                          law, opt.method, custom.law) {

  C  <- find.my.case(Dx, Ex, mx, qx)$case
  mu <- eval(call(law, x, par = exp(par)))$hx
  mu[is.infinite(mu)] <- 1

  if (is.null(Ex))    Ex = 1
  if (C == "C1_DxEx") nu = Dx/Ex
  if (C == "C2_mx")   nu = Dx <- mx
  if (C == "C3_qx")   nu = Dx <- qx

  # compute likelihoods or loss functions
  loss <- switch(
    EXPR = opt.method,
    poissonL  = -(Dx * log(mu) - mu*Ex),
    binomialL = -(Dx * log(1 - exp(-mu)) - (Ex - Dx)*mu),
    LF1       =  (1 - mu/nu)^2,
    LF2       =  log(mu/nu)^2,
    LF3       =  ((nu - mu)^2)/nu,
    LF4       =  (nu - mu)^2,
    LF5       =  (nu - mu) * log(nu/mu),
    LF6       =  abs(nu - mu)
    )

  # Here I want to make sure that the optimisation algorithm is not returning
  # NaN values when it converges (because that is possible).
  loss[is.infinite(loss)] <- 10^5
  if (sum(is.na(mu)) != 0) loss = loss + 10^5
  out <- sum(loss, na.rm = TRUE)
  return(out)
}

#' Scale the age vector for stable optimisation
#'
#' For mortality laws that cover only a portion of the lifespan (e.g., adult
#' or old-age mortality), the age vector is rescaled so that the minimum age
#' becomes 1. This improves numerical stability by keeping the exponentiated
#' terms in the hazard function within a reasonable range.
#'
#' @param x A numeric vector of ages.
#' @return A numeric vector of scaled ages, where min(x) == 1.
#' @keywords internal
scale_x <- function(x) {
  x - min(x) + 1
}

#' Run the optimisation routine
#'
#' This is the core optimisation function for \code{\link{MortalityLaw}}. It:
#' \enumerate{
#'   \item Subsets the data to the fitting ages (\code{fit.this.x}).
#'   \item Scales the age vector if required by the chosen model.
#'   \item Obtains default starting parameters (if not provided).
#'   \item Minimises the objective function using \code{\link{nlminb}} (PORT
#'         routines) with the parameters on the log scale.
#'   \item Transforms parameters back to the original scale, computes the
#'         fitted hazard, and derives goodness-of-fit measures (AIC, BIC,
#'         log-likelihood) where applicable.
#' }
#'
#' @param input A list containing all input arguments to \code{\link{MortalityLaw}}.
#' @return A list with components:
#' \item{x}{Age vector (original).}
#' \item{new.x}{Age vector after optional scaling.}
#' \item{opt}{Object returned by \code{\link{nlminb}}.}
#' \item{C}{Estimated parameters on the original scale.}
#' \item{hx}{Fitted hazard values evaluated at \code{new.x}.}
#' \item{logLik, AIC, BIC}{Goodness-of-fit measures (\code{NaN} for
#' non-likelihood methods).}
#' @keywords internal
choose_optim <- function(input){
  with(as.list(input), {
    # Subset the data
    select.x <- x %in% fit.this.x

    if (scale.x) {
      new.fit.this.x = scale_x(fit.this.x)
      d = fit.this.x[1] - new.fit.this.x[1]
      new.x = x - d

    } else {
      new.fit.this.x <- fit.this.x
      new.x <- x
    }
    # Starting parameters
    if (is.null(parS)) parS <- bring_parameters(law, parS)
    
    # Objective function setup
    foo <- function(pars) {
      objective_fun(
        par = pars,
        x = new.fit.this.x,
        Dx = Dx[select.x],
        Ex = Ex[select.x],
        mx = mx[select.x],
        qx = qx[select.x],
        law,
        opt.method,
        custom.law)
    }

    # Optimization algorithm
      if (law == 'invweibull'){
        opt <- optim(par = log(parS), fn = foo, method = 'Nelder-Mead')
        opt$fnvalue <- opt$value

      } else {
        opt <- nlminb(start = log(parS), objective = foo, control = list(eval.max = 5000, iter.max = 5000))
        opt$fnvalue <- opt$objective
      }

    # Return the optimal parameters
    C <- exp(opt$par)

    if (law == 'kostaki') { #kostaki hack
      if (C[5] >= 50*C[6]) C[6] <- C[5]/50
    }

    # Hazard function of the fitted model
    hx     <- do.call(law, list(x = new.x, par = C))$hx

    # Compute goodness of fit measures  
    logLik <- log(opt$fnvalue)
    AIC    <- 2 * length(parS) - 2 * logLik
    BIC    <- log(length(fit.this.x)) * length(parS) - 2 * logLik

    if (!any(opt.method %in% c('poissonL', 'binomialL'))) {
      logLik = AIC  = BIC  <- NaN
    }
    out <- as.list(environment())
    return(out)
  })
}

# --------------------------------------------
# Author: Marius D PASCARIU
# Date: 2026-05-04 23:31:48
# --------------------------------------------

#' Download the Japanese Mortality Database (JMD)
#'
#' Download detailed mortality and population data of the 47 prefectures in
#' Japan, in a single object. The source of data is the
#' \href{https://www.ipss.go.jp/p-toukei/JMD/index-en.asp}{
#' Japanese Mortality Database}.
#'
#' @details
#' (Description taken from the JMD website).
#'
#' The Japanese Mortality Database is a comprehensively-reorganized mortality
#' database that is optimized for mortality research and consistent with the
#' Human Mortality Database. This database is provided as a part of the research
#' project "Demographic research on the causes and the socio-economic
#' consequence of longetivity extension in Japan" (2011-2013), "Demographic
#' research on longevity extension, population aging, and their effects on the
#' social security and socio-economic structures in Japan" (2014-2016), and
#' "Comprehensive research from a demographic viewpoint on the longevity
#' revolution" (2017-2019) at the National Institute of Population and Social
#' Security Research.
#'
#' The Japanese Mortality Database is designed to provide the life tables to all
#' the people who are interested in Japanese mortality including domestic and
#' foreign mortality researchers for the purpose of mortality research.
#' Especially because we have structured it to conform with the HMD, our
#' database is suitable for international comparison, we put emphasis on the
#' compatibility with the HMD more than our country's particular
#' characteristics. Therefore, the life tables by JMD do not necessarily
#' exhibit the same values as ones by the official life tables prepared and
#' released by the Statistics and Information Department, Minister's
#' Secretariat, Ministry of Health, Labor and Welfare according to the different
#' base population or the methods for estimating the tables. When doing things
#' other than mortality research, if life table that statistically displays our
#' country's mortality situation is necessary, please use the official life
#' table that has been prepared by the Statistics and Information Department,
#' Minister's Secretariat, Ministry of Health, Labor and Welfare.
#'
#' At the present time, we offer the data for All Japan and by prefecture.
#' The project team is studying the methodology for estimating life tables
#' along with data preparation. Therefore, the data may be updated when a
#' new methodology is adopted. Please refer to "Methods" for further
#' information.
#'
#' @inheritParams ReadHMD
#' @param regions Specify the region specific data you want to download by
#' adding the JMD region code/s. Options: \code{"Japan", "Hokkaido", "Aomori",
#' "Iwate", "Miyagi","Akita", "Yamagata", "Fukushima", "Ibaraki", "Tochigi",
#' "Gunma", "Saitama", "Chiba", "Tokyo", "Kanagawa", "Niigata", "Toyama",
#' "Ishikawa", "Fukui", "Yamanashi", "Nagano", "Gifu", "Shizuoka","Aichi",
#' "Mie", "Shiga", "Kyoto", "Osaka", "Hyogo", "Nara", "Wakayama", "Tottori",
#' "Shimane", "Okayama", "Hiroshima", "Yamaguchi", "Tokushima", "Kagawa",
#' "Ehime", "Kochi", "Fukuoka", "Saga", "Nagasaki", "Kumamoto", "Oita",
#' "Miyazaki", "Kagoshima", "Okinawa"}.
#' If \code{NULL} data for all the regions are downloaded at once.
#' @return A \code{ReadJMD} object that contains:
#'  \item{input}{List with the input values;}
#'  \item{data}{Data downloaded from JMD;}
#'  \item{download.date}{Time stamp;}
#'  \item{years}{Numerical vector with the years covered in the data;}
#'  \item{ages}{Numerical vector with ages covered in the data.}
#' @author Marius D. Pascariu
#' @seealso
#' \code{\link{ReadHMD}}
#' \code{\link{ReadCHMD}}
#' @examples
#' \donttest{
#' # Download demographic data for Fukushima and Tokyo regions in 1x1 format
#'
#' # Death counts. We don't want to export data outside R.
#' JMD_Dx <- ReadJMD(what = "Dx",
#'                   regions = c('Fukushima', 'Tokyo'),
#'                   interval  = "1x1",
#'                   save = FALSE)
#' JMD_Dx
#'
#' # Download life tables for female population in all the states and export data.
#' LTF <- ReadJMD(what = "LT_f", interval  = "5x5", save = FALSE)
#' LTF
#' }
#' @export
ReadJMD <- function(what,
                    regions = NULL,
                    interval = "1x1",
                    save = FALSE,
                    show = TRUE){
  # Step 1 - Validate input & Progress bar setup
  if (is.null(regions)) {
    regions <- JPNregions()
  }

  input <- as.list(environment())
  check_input_ReadJMD(input)
  nr <- length(regions)

  if (show) {
    pb <- startpb(0, nr + 1)
    on.exit(closepb(pb))
    setpb(pb, 0)
  }

  # Step 2 - Do the loop for the other regions
  D <- data.frame()
  for (i in 1:nr) {
    if (show) {
      setpb(pb, i)
      cat(paste("      :Downloading", regions[i], "    "))
    }
    region_code <- substrRight(paste0(0, match(regions[i], JPNregions()) - 1), 2)
    d <- ReadHMD.core(
      what = what,
      country = region_code,
      interval = interval,
      username = NULL,
      password = NULL,
      link = "https://www.ipss.go.jp/p-toukei/JMD/")
    
    if(!is.null(d)) {
      colnames(d)[colnames(d) == "country"] <- "region"
      D <- rbind(D, d)
    }
  }

  if (length(D) != 0) {
    
    out <- list(
      input = input,
      data = D,
      download.date = date(),
      years = sort(unique(D$Year)),
      ages = unique(D$Age)
      )
    out <- structure(class = "ReadJMD", out)
  
    # Step 3 - Write a file with the database in your working directory
    if (show) setpb(pb, nr + 1)
    if (save) saveOutput(out, show, prefix = "JMD")
    
  } else {
    out <- NULL
  }

  # Exit
  return(out)
}


#' region codes
#' @return the vector
#' @keywords internal
JPNregions <- function() {
  c("Japan",

    "Aichi",
    "Akita",
    "Aomori",
    "Chiba",
    "Ehime",
    "Fukushima",
    "Fukui",
    "Fukuoka",
    "Gifu",
    "Gunma",
    "Hyogo",
    "Hokkaido",
    "Hiroshima",
    "Iwate",
    "Ibaraki",
    "Ishikawa",
    "Kagawa",
    "Kanagawa",
    "Kagoshima",
    "Kyoto",
    "Kochi",
    "Kumamoto",
    "Miyazaki",
    "Miyagi",
    "Mie",
    "Nara",
    "Nagano",
    "Nagasaki",
    "Niigata",
    "Oita",
    "Okayama",
    "Okinawa",
    "Osaka",
    "Saitama",
    "Saga",
    "Shizuoka",
    "Shiga",
    "Shimane",
    "Tochigi",
    "Tokyo",
    "Tokushima",
    "Toyama",
    "Tottori",
    "Wakayama",
    "Yamagata",
    "Yamaguchi",
    "Yamanashi"
    )
}



#' Check input ReadJMD
#' @param x a list containing the input arguments from ReadJMD function
#' @return No return value, called for validating input data
#' @keywords internal
check_input_ReadJMD <- function(x) {

  if (!(x$interval %in% data_format())) {
    stop(
      "The interval ",
      x$interval,
      " does not exist in JMD ",
      "Try one of these options:\n",
      paste(data_format(), collapse = ", "),
      call. = FALSE
      )
  }

  if (!(x$what %in% HMDindices())) {
    stop(
      x$what,
      " does not exist in JMD. Try one of these options:\n",
      paste(HMDindices(), collapse = ", "),
      call. = FALSE
      )
  }

  if (all(!(x$regions %in% JPNregions()))) {
    stop(
      "Something is wrong in the region codes supplied.\n",
      "Try one or more of these options:\n",
      paste(JPNregions(), collapse = ", "),
      call. = FALSE
      )
  }
}



#' Print ReadJMD
#' @param x An object of class \code{"ReadJMD"}
#' @param ... Further arguments passed to or from other methods.
#' @return Print info on the console
#' @keywords internal
#' @export
print.ReadJMD <- function(x, ...){
  what <- x$input$what
  cat("Japanese Mortality Database\n")
  cat("Web Address   : https://www.ipss.go.jp/p-toukei/JMD/index-en.asp\n")
  cat("Download Date :", x$download.date, "\n")
  cat("Type of data  :", what, "\n")
  cat(paste("Interval      :", x$input$interval, "\n"))
  cat(paste("Years         :", x$years[1], "--", rev(x$years)[1], "\n"))
  cat(paste("Ages          :", ageMsg(what, x), "\n"))
  cat("Regions       :", x$input$regions, "\n")
  cat("\nData:\n")
  print(head_tail(x$data, hlength = 5, tlength = 5))
}

# -------------------------------------------------------------- #
# Author: Marius D. PASCARIU
# Last Update: Mon Jan 15 16:54:54 2024
# -------------------------------------------------------------- #
remove(list = ls())

# Wrong index
expect_error(ReadJMD(what = "DxDD"))

# Wrong region
expect_error(ReadJMD(what = "Dx",
                     regions = "Kyotooooooo"))

# Wrong interval
expect_error(ReadJMD(what = "Dx",
                     regions = "Kyoto",
                     interval = "1x50"))

# Wrong interval for the index
expect_message(ReadJMD(what = "Ex_lexis",
                     regions = "Japan",
                     interval = "1x1"))

expect_message(ReadJMD(what = "e0",
                     regions = "Japan",
                     interval = "5x1"))

# remove test since the case is now valid and available in the database.
# expect_error(ReadJMD(what = "LT_f",
#                      regions = "Kyoto",
#                      interval = "1x1"))

expect_output(
  print(JMD_sample)
)

# The tests below have been removed because in case the internet source is
# temporary not working the CRAN will consider it as a software failure and will
# demand correction. Unfortunately, we can test only the error messages i.e. the
# automated checks put in place.

# expect_output(ReadJMD(what = "e0",
#                       regions = "Japan",
#                       interval = "1x5"))
#
# expect_output(ReadJMD(what = "LT_f",
#                       regions = "Kyoto",
#                       interval = "5x5"))
#
# expect_silent(D <- ReadJMD(what = "LT_f",
#                            regions = "Kyoto",
#                            interval = "5x5",
#                            show = FALSE))
# expect_output(print(D))

# --------------------------------------------
# Author: Marius D PASCARIU
# Date: 2026-05-04 23:30:36
# --------------------------------------------

#' MortalityLaws Test Data
#'
#' Dataset containing altered death rates (mx), death counts (Dx)
#' and exposures (Ex) for the female population living in
#' England & Wales in four different years: 1850, 1900, 1950 and 2010.
#' This dataset is provided for testing purposes only.
#' Download the actual data free of charge from \url{https://www.mortality.org}.
#' Once a username and a password are created on the website, the function
#' \code{\link{ReadHMD}} can be used for downloading.
#'
#' @source \href{https://www.mortality.org}{Human Mortality Database}
#' @seealso \code{\link{ReadHMD}}
#' @examples head(ahmd$mx)
"ahmd"


#' HMD sample
#' Data object generated by the \code{ReadHMD()} function.
#' @return A sample of demographic data in a data.frame
#' @keywords internal
"HMD_sample"

#' AHMD sample
#' Data object generated by the \code{ReadAHMD()} function.
#' @return A sample of demographic data in a data.frame
#' @keywords internal
"AHMD_sample"

#' CHMD sample
#' Data object generated by the \code{ReadCHMD()} function.
#' @return A sample of demographic data in a data.frame
#' @keywords internal
"CHMD_sample"

#' JMD sample
#' Data object generated by the \code{ReadJMD()} function.
#' @return A sample of demographic data in a data.frame
#' @keywords internal
"JMD_sample"

# --------------------------------------------
# Author: Marius D PASCARIU
# Date: 2026-05-03 21:42:58
# --------------------------------------------

# MortalityLaws Package

#' @details
#' To learn more about the package, start with the vignettes:
#' \code{browseVignettes(package = "MortalityLaws")}
#'
#' @importFrom RCurl getURL
#' @importFrom tidyr spread
#' @importFrom graphics plot abline axis barplot box hist layout legend lines par rect
#' @importFrom utils read.table read.csv head tail
#' @importFrom stats fitted coef optim predict quantile nlminb pt printCoefmat
#' df.residual qt vcov
#' @importFrom pbapply startpb closepb setpb
#' @importFrom methods is
#' @import rvest httr
#' @name MortalityLaws
#' @docType package
"_PACKAGE"

# --------------------------------------------
# Author: Marius D PASCARIU
# Date: 2026-05-04 23:38:03
# --------------------------------------------

#' Plot Method for MortalityLaw
#' @param x An object of class \code{"MortalityLaw"}.
#' @param ... Further arguments passed to graphical methods, such as
#' parameters (see \code{\link{par}}).
#' @return A plot is generated as a side effect.
#' @seealso \code{\link{MortalityLaw}}
#' @author Marius D. Pascariu
#' @examples
#' # See complete example in MortalityLaw help page
#' @export
plot.MortalityLaw <- function(x, ...){
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))   #- reset to default

  with(
    data = x$input,
    if (find.my.case(Dx, Ex, mx, qx)$iclass != "numeric") {
      stop("Plot function not available for multiple mortality curves",
           call. = FALSE)})

  age      <- x$input$x
  age2     <- x$input$fit.this.x
  select.x <- age %in% age2
  law      <- x$input$law

  if (law == "custom.law") {
    lawN <- "Custom Mortality"

  } else {
    lawN = unlist(availableLaws(law)$table['NAME'])
  }

  lay_mat <- matrix(c(1, 2, 3, 1, 2, 3), ncol = 3, byrow = TRUE)
  layout(
    lay_mat,
    widths = 1.5 * c(6, 6, 1.5),
    heights = 1.5 * c(1.5, 6, 6),
    respect = TRUE)

  # ----- Plot 1 -----
  par(
    mar = c(5, 5, 4, 1),
    cex.lab = 1.1,
    cex.axis = 1.3
    )

  if (!is.null(x$input$qx)) {
    y = x$input$qx

  } else {
    y = with(x$input, if (is.null(mx)) Dx/Ex else mx)
  }

  fit_y <- x$fitted.values
  pos_x <- quantile(
    x = age,
    p = seq(0, 1, by = 0.25),
    type = 1,
    na.rm = TRUE)
  pos_y <- round(
    quantile(
      x = log(y),
      p = seq(0, 1, by = 0.25),
      type = 1,
      na.rm = TRUE),
    digits = 1
    )

  plot(
    x = age,
    y = log(y),
    pch = 16,
    cex.main = 1.1,
    ylab = 'Mortality Level',
    xlab = 'x',
    axes = FALSE,
    main = paste('Observed vs. Fitted', lawN, 'law'),
    panel.first = rect(
      xleft = min(age2),
      xright = max(age2),
      ybottom = -1e6,
      ytop = 1e6,
      col = 'grey95',
      border = FALSE
      )
    )
  box(col = 'grey80')
  axis(1, at = pos_x)
  axis(2, at = pos_y)
  lines(
    x = age,
    y = log(fit_y),
    col = 3,
    lwd = 1.5
    )
  legend(
    x = 'bottomright',
    bty = "n",
    legend = c("Observed", paste("Fitted", law)),
    lty = c(NA, 1),
    pch = c(16, NA),
    col = c(1, 3),
    cex = 1.3,
    lwd = 3
    )

  # ----- Plot 2 -----
  # resid <- x$residuals[select.x]
  resid <- x$residuals
  par(
    mar = c(5, 5, 4, 1),
    cex.lab = 1.1,
    cex.axis = 1.3
    )
  plot(
    x = age,
    y = resid,
    pch = 16,
    cex.main = 1.1,
    main = 'Residual plot',
    xlab = 'x',
    ylab = '',
    axes = FALSE,
    panel.first = rect(
      xleft = min(age2),
      xright = max(age2),
      ybottom = -1e6,
      ytop = 1e6,
      col = 'grey95',
      border = FALSE
      )
    )
  box(col = 'grey80')
  axis(1, at = pos_x)
  axis(2, at = pos_y)
  abline(
    h = 0,
    lty = 2,
    col = 'grey80'
    )

  # ----- Plot 3 -----
  xhist <- hist(
    x = resid,
    breaks = max(10, round(length(resid)/5)),
    plot = FALSE
    )
  par(
    mar = c(5, 0, 4, 1),
    cex.lab = 0.9,
    cex.axis = 1.3
    )
  barplot(
    height = xhist$counts,
    axes = TRUE,
    space = 0,
    horiz = TRUE,
    xlab = 'Frequency'
    )
}

# --------------------------------------------
# Author: Marius D PASCARIU
# Date: 2026-05-04 22:59:32
# --------------------------------------------

#' Compute Life Tables from Mortality Data
#'
#' Construct either a full (single-year age intervals) or an abridged 
#' (wider age intervals) life table from a variety of input data types. 
#' The function accepts:
#' \itemize{
#'   \item Death counts and mid-interval population estimates (\code{Dx, Ex})
#'   \item Age-specific death rates (\code{mx})
#'   \item Death probabilities (\code{qx})
#'   \item Survivorship curve (\code{lx})
#'   \item Distribution of deaths (\code{dx})
#' }
#' Only one of these input options needs to be provided; the others 
#' are ignored if present. The input can be a numeric \code{vector}, 
#' \code{matrix}, or \code{data.frame}. When a \code{matrix} or 
#' \code{data.frame} with multiple columns is supplied, the function 
#' computes one life table per column.
#'
#' @details
#' A life table (also called a mortality table or actuarial table) 
#' summarises the mortality experience of a population. For each age 
#' (or age interval) it reports:
#' \itemize{
#'   \item Death rates (\code{mx}) and death probabilities (\code{qx})
#'   \item Survivorship (\code{lx})
#'   \item Distribution of deaths (\code{dx})
#'   \item Person-years lived (\code{Lx}) and total person-years remaining 
#'         (\code{Tx})
#'   \item Life expectancy (\code{ex})
#' }
#' The life table is constructed sequentially: from the input data the 
#' function derives \code{mx}, then \code{qx}, then \code{lx}, \code{dx}, 
#' \code{Lx}, \code{Tx}, and finally \code{ex}. The constant-force-of-mortality 
#' (CFM) assumption is used to convert between \code{mx} and \code{qx}. 
#' If the \code{sex} argument is supplied, the first two values of the 
#' \code{ax} column are adjusted using the Coale-Demeny method, which 
#' accounts for the different infant mortality patterns between males 
#' and females.
#'
#' @usage
#' LifeTable(x, Dx = NULL, Ex = NULL,
#'              mx = NULL,
#'              qx = NULL,
#'              lx = NULL,
#'              dx = NULL,
#'              sex = NULL,
#'              lx0 = 1e5,
#'              ax  = NULL)
#'
#' @param x Numeric vector of ages at the beginning of each age interval. 
#'   For a full life table, use single-year ages (e.g., \code{0:110}). 
#'   For an abridged life table, use the lower bound of each interval 
#'   (e.g., \code{c(0, 1, 5, 10, ..., 110)}).
#'
#' @param Dx Death counts. Each element represents the total number of 
#'   deaths during the calendar year to persons aged \code{x} to 
#'   \code{x + n} (where \code{n} is the length of the age interval). 
#'   Must be provided together with \code{Ex}.
#'
#' @param Ex Exposure-to-risk in the period. This is usually approximated 
#'   by the mid-year population aged \code{x} to \code{x + n}. Must be 
#'   provided together with \code{Dx}.
#'
#' @param mx Age-specific death rate in the age interval \code{[x, x+n)}. 
#'   Defined as \code{Dx / Ex}.
#'
#' @param qx Probability of dying within the age interval \code{[x, x+n)}.
#'
#' @param lx Probability of surviving to exact age \code{x} (if \code{lx0 = 1}), 
#'   or the number of survivors at exact age \code{x} (if \code{lx0 > 1}). 
#'   When \code{lx} is the sole input, the values are re-scaled to the 
#'   chosen radix \code{lx0}.
#'
#' @param dx Number of deaths in the life-table population occurring in 
#'   the age interval \code{[x, x+n)}. When \code{dx} is the sole input, 
#'   the values are re-scaled to sum to \code{lx0}.
#'
#' @param sex Sex of the population. Options are \code{NULL} (default), 
#'   \code{"male"}, \code{"female"}, or \code{"total"}. When specified, 
#'   the first two entries of the \code{ax} column are adjusted using 
#'   Coale-Demeny coefficients, producing more accurate life-table values 
#'   at the youngest ages. The adjustment differs slightly between males 
#'   and females.
#'
#' @param lx0 Radix, the starting population (or probability scale) at 
#'   age 0. Default is \code{100,000}. All subsequent life-table columns 
#'   (\code{lx}, \code{dx}, \code{Lx}, \code{Tx}) are scaled accordingly.
#'
#' @param ax Numeric vector representing the average number of person-years 
#'   lived in the age interval by those who die in that interval. If 
#'   \code{NULL} (the default), \code{ax} is estimated internally using 
#'   a standard formula. You may supply a single value (applied to all 
#'   intervals) or a vector of the same length as \code{x}. A common 
#'   assumption is \code{ax = 0.5}, which places deaths at the midpoint 
#'   of each interval.
#'
#' @return An object of class \code{"LifeTable"} containing the following 
#'   components:
#'   \item{lt}{A \code{data.frame} with the complete life table, including 
#'     columns for age interval (\code{x.int}), exact age (\code{x}), 
#'     death rate (\code{mx}), death probability (\code{qx}), person-years 
#'     lived by decedents (\code{ax}), survivorship (\code{lx}), death 
#'     distribution (\code{dx}), person-years lived (\code{Lx}), total 
#'     person-years remaining (\code{Tx}), and life expectancy (\code{ex}).}
#'   \item{call}{The matched function call.}
#'   \item{process_date}{Timestamp of when the life table was computed.}
#'
#' @seealso
#' \code{\link{LawTable}} for generating life tables from a fitted 
#'   parametric mortality law; 
#'   \code{\link{convertFx}} for converting between mortality measures.
#'
#' @author Marius D. Pascariu
#'
#' @examples
#' # Example 1 --- Full life tables with different inputs ------------
#'
#' y  <- 1900
#' x  <- as.numeric(rownames(ahmd$mx))
#' Dx <- ahmd$Dx[, paste(y)]
#' Ex <- ahmd$Ex[, paste(y)]
#'
#' LT1 <- LifeTable(x, Dx = Dx, Ex = Ex)
#' LT2 <- LifeTable(x, mx = LT1$lt$mx)
#' LT3 <- LifeTable(x, qx = LT1$lt$qx)
#' LT4 <- LifeTable(x, lx = LT1$lt$lx)
#' LT5 <- LifeTable(x, dx = LT1$lt$dx)
#'
#' LT1
#' LT5
#' ls(LT5)
#'
#' # Example 2 --- Compute multiple life tables at once ------------
#'
#' LTs <- LifeTable(x, mx = ahmd$mx)
#' LTs
#' # A warning is printed if the input contains missing values.
#' # Some of the missing values can be handled automatically.
#'
#' # Example 3 --- Abridged life table -----------------------------
#'
#' x  <- c(0, 1, seq(5, 110, by = 5))
#' mx <- c(.053, .005, .001, .0012, .0018, .002, .003, .004,
#'         .004, .005, .006, .0093, .0129, .019, .031, .049,
#'         .084, .129, .180, .2354, .3085, .390, .478, .551)
#' LT6 <- LifeTable(x, mx = mx, sex = "female")
#' LT6
#'
#' # Example 4 --- Abridged life table using a custom 'ax' --------
#' # This example reuses the ages (x) and death rates (mx) from Example 3.
#' # Note that 'ax' must have the same length as 'x', otherwise an error
#' # will be returned.
#'
#' my_ax <- c(0.1, 1.5, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
#'            2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1)
#'
#' LT7 <- LifeTable(x = x, mx = mx, ax = my_ax)
#'
#' @export
LifeTable <- function(x,
                      Dx = NULL,
                      Ex = NULL,
                      mx = NULL,
                      qx = NULL,
                      lx = NULL,
                      dx = NULL,
                      sex = NULL,
                      lx0 = 1e5,
                      ax = NULL){

  input <- c(as.list(environment()))
  X     <- LifeTable.check(input)
  LT    <- NULL

  if (any(X$iclass == "numeric")) {
    LT <- with(X, LifeTable.core(x, Dx, Ex, mx, qx, lx, dx, sex, lx0, ax))

  } else {
    for (i in 1:X$nLT) {
      LTi <- with(
        X,
        LifeTable.core(
          x,
          Dx = Dx[, i],
          Ex = Ex[, i],
          mx = mx[, i],
          qx = qx[, i],
          lx = lx[, i],
          dx = dx[, i],
          sex = sex,
          lx0 = lx0,
          ax = ax
          )
        )

      N   <- X$LTnames
      LTn <- if (is.na(N[i])) i else N[i]
      LTi <- cbind(LT = LTn, LTi)
      LT  <- rbind(LT, LTi)
    }
  }

  # Exit
  out <- list(
    lt = LT,
    call = match.call(),
    process_date = date()
    )
  out <- structure(class = "LifeTable", out)
  return(out)
}


#' LifeTable.core
#' @inheritParams LifeTable
#' @return A data.frame containing life table results
#' @keywords internal
LifeTable.core <- function(x, Dx, Ex, mx, qx, lx, dx, sex, lx0, ax){

  my.case  <- find.my.case(Dx, Ex, mx, qx, lx, dx)$case
  gr_names <- paste0("[", x,",", c(x[-1], "+"), ")")
  N        <- length(x)
  df       <- diff(x)
  nx       <- c(df, df[N - 1])

  if (my.case == "C1_DxEx") {
    Dx <- as.numeric(Dx)
    Ex <- as.numeric(Ex)
    mx <- Dx/Ex
    mx <- uxAbove100(x, mx)
    qx <- mx_qx(x, nx, mx, out = "qx")
    lx <- lx0 * c(1, cumprod(1 - qx)[1:(N - 1)])
    dx <- dx_lx(lx, out = "dx")
  }
  if (my.case == "C2_mx") {
    mx <- as.numeric(mx)
    qx <- mx_qx(x, nx, mx, out = "qx")
    lx <- lx0 * c(1, cumprod(1 - qx)[1:(N - 1)])
    dx <- dx_lx(lx, out = "dx")
  }
  if (my.case == "C3_qx") {
    qx <- as.numeric(qx)
    mx <- mx_qx(x, nx, qx, out = "mx")
    lx <- lx0 * c(1, cumprod(1 - qx)[1:(N - 1)])
    dx <- dx_lx(lx, out = "dx")
  }
  if (my.case == "C4_lx") {
    lx <- as.numeric(lx)
    lx <- lx * lx0/lx[1]
    dx <- dx_lx(lx, out = "dx")
    qx <- dx/lx
    qx <- uxAbove100(x, qx)
    mx <- mx_qx(x, nx, qx, out = "mx")
  }
  if (my.case == "C5_dx") {
    dx <- as.numeric(dx)
    dx <- dx * lx0/sum(dx)
    lx <- dx_lx(dx, out = "lx")
    qx <- dx/lx
    qx <- uxAbove100(x, qx)
    mx <- mx_qx(x, nx, qx, out = "mx")
  }

  if (is.null(ax)) {
    ax <- compute.ax(x, mx, qx)
    if (!is.null(sex)) ax <- coale.demeny.ax(x, mx, ax, sex)

  } else if (length(ax) == 1){
    ax <- rep(ax, N)

  } else if (length(ax) == N){
    ax <- as.numeric(ax)
  }

  Lx    <- nx * lx - (nx - ax) * dx
  Lx[N] <- ax[N] * dx[N]
  Lx[is.na(Lx)] <- 0
  Tx    <- rev(cumsum(rev(Lx)))
  ex    <- Tx/lx
  ex[is.na(ex)] <- 0
  ex[N] <- if (ex[N - 1] == 0) 0 else ax[N]

  last_check = all(is.na(mx)) | all(is.nan(mx)) | all(is.infinite(mx)) | all(mx == 0)
  if (last_check) mx = qx = ax = lx = dx = Lx = Tx = ex <- NA

  out <- data.frame(x.int = gr_names,
                    x = x,
                    mx = mx,
                    qx = qx,
                    ax = ax,
                    lx = lx,
                    dx = dx,
                    Lx = Lx,
                    Tx = Tx,
                    ex = ex)
  return(out)
}


#' Function that identifies the case/problem we have to solve
#' @inheritParams LifeTable
#' @return A list containing problem solving details
#' @keywords internal
find.my.case <- function(Dx = NULL,
                         Ex = NULL,
                         mx = NULL,
                         qx = NULL,
                         lx = NULL,
                         dx = NULL) {

  input   <- c(as.list(environment()))

  # Matrix of possible cases --------------------
  rn  <- c("C1_DxEx", "C2_mx", "C3_qx", "C4_lx", "C5_dx")
  cn  <- c("Dx", "Ex", "mx", "qx", "lx", "dx")
  mat <- matrix(
    ncol = 6,
    byrow = TRUE,
    dimnames = list(rn, cn),
    data = c(T,T,F,F,F,F,
             F,F,T,F,F,F,
             F,F,F,T,F,F,
             F,F,F,F,T,F,
             F,F,F,F,F,T)
    )
  # ----------------------------------------------
  L1 <- !unlist(lapply(input, is.null))
  L2 <- apply(mat, 1, function(x) all(L1 == x))
  my_case <- rn[L2]

  if (sum(L1[c(1, 2)]) == 1) {
    stop("If you input 'Dx' you must input 'Ex' as well, and viceversa",
         call. = FALSE)
  }

  if (!any(L2)) {
    stop("The input is not specified correctly. Check again the function ",
         "arguments and make sure the input data is added properly.",
         call. = FALSE)
  }

  X       <- input[L1][[1]]

  if (length(dim(X)) == 1){
    # TR changed here:
    # The following input isn't detected with is.vector()
    # X = structure(c(0.0036542739116619, 0.000150960486092765, 2.77881983521598e-05,
    # 0.000136941279579316, 0.00018946827083136, 0.00026606712873658,
    # 0.000258838755220895, 0.000557913403869528, 0.000665917509468515,
    # 0.00114979916336982, 0.0021040113433655, 0.00384681333263752,
    # 0.00632225868307671, 0.00937214337262448, 0.0154497258185624,
    # 0.023117866694913, 0.0363617070194365, 0.0609184108563059, 0.120398389987367,
    # 0.221461187214612, 0.420152946468736), .Dim = 21L)
    # nLT would come as NA
    # and iclass would be array
    X <- c(X)
  }

  nLT     <- 1
  LTnames <- NA

  # TR: change from !is.vector
  if (length(dim(X)) == 2 ) {

    nLT     <- ncol(X)     # number of LTs to be created
    LTnames <- colnames(X) # the names to be assigned to LTs
  }

  out <- list(case = my_case,
              iclass = class(X), # TR: if inputs are matrix,
                                 # then this is two elements
              nLT = nLT,
              LTnames = LTnames)
  return(out)
}


#' mx to qx
#'
#' Function to convert mx into qx and back, using the constant force of
#' mortality assumption (CFM).
#' @inheritParams LifeTable
#' @param nx Length of the age-intervals.
#' @param ux A vector of mx or qx.
#' @param out Type of the output: mx or qx.
#' @return A vector of rates
#' @keywords internal
mx_qx <- function(x, nx, ux, out = c("qx", "mx")){
  out <- match.arg(out)

  if (out == "qx") {
    eta <- 1 - exp(-nx * ux)
    eta[length(nx)] <- 1  # The life table should always close with q[x] = 1

  } else {
    eta <- suppressWarnings(-log(1 - ux)/nx)
    # If qx[last-age] = 1 then mx[last-age] = Inf. Not nice to have Inf's;
    # they distort the results in the subsequent processes.
    # We apply a simple extrapolation method of the last mx.
    N <- length(x)
    eta[N] <- eta[N - 1]^2 / eta[N - 2]
  }

  eta <- uxAbove100(x, eta)
  return(eta)
}


#' Educate mx or qx on how to behave above age 100 if it gets in trouble
#' (with NA's, zero's and Inf)
#' @inheritParams LifeTable
#' @inheritParams mx_qx
#' @param omega Threshold age. Default: 100.
#' @param verbose A logical value. Set \code{verbose = FALSE} to silent
#' the process that take place inside the function and avoid progress messages.
#' @return A vector of rates
#' @keywords internal
uxAbove100 <- function(x,
                       ux,
                       omega = 100,
                       verbose = FALSE) {

  if (is.vector(ux)) {
    L <- x >= 100 & (is.na(ux) | is.infinite(ux) | ux == 0)

    if (any(L)) {
      mux   <- max(ux[!L])
      ux[L] <- mux

      if (verbose)
        warning("The input data contains NA's, Inf or zero's over the age of ",
                "100. These have been replaced with maximum observed value: ",
                round(mux, 4), call. = FALSE)
    }

  } else {
    for (i in 1:ncol(ux)) {
      ux[, i] = uxAbove100(x, ux[, i], omega, verbose)
    }

  }

  return(ux)
}


#' dx to lx
#'
#' Function to convert dx into lx and back
#' @param ux A vector of dx or lx data.
#' @param out Type of the output: dx or lx.
#' @return A vector containing dx or lx values
#' @keywords internal
dx_lx <- function(ux, out = c("dx", "lx")) {
  out <- match.arg(out)

  if (out == "dx") {
    ux_ <- rev(diff(rev(ux)))
    d   <- ux[1] - sum(ux_)
    eta <- c(ux_, d)

  } else {
    eta <- rev(cumsum(rev(ux)))
  }
  return(eta)
}


#' Find ax indicator
#'
#' @inheritParams LifeTable
#' @return \code{ax} - the point in the age interval where 50% of the deaths
#' have already occurred
#' @keywords internal
compute.ax <- function(x, mx, qx) {
  nx <- c(diff(x), Inf)
  N  <- length(x)
  ax <- nx + 1/mx - nx/qx

  for (i in 1:(N - 1)) {
    if (is.infinite(ax[i + 1]) | is.na(ax[i + 1])) ax[i + 1] = ax[i]
  }
  return(ax)
}


#' Find ax[1:2] indicators using Coale-Demeny coefficients
#' Here we adjust the first two values of ax to account for infant
#' mortality more accurately
#' @inheritParams LifeTable
#' @return A vector of coefficients
#' @keywords internal
coale.demeny.ax <- function(x, mx, ax, sex) {

  if (mx[1] < 0) stop("'m[1]' must be greater than 0", call. = FALSE)

  nx  <- c(diff(x), Inf)
  m0  <- mx[1]
  a0M <- ifelse(m0 >= 0.107, 0.330, 0.045 + 2.684 * m0)
  a1M <- ifelse(m0 >= 0.107, 0.330, 1.651 - 2.816 * m0)
  a0F <- ifelse(m0 >= 0.107, 0.350, 0.053 + 2.800 * m0)
  a1F <- ifelse(m0 >= 0.107, 0.350, 1.522 - 1.518 * m0)
  a0T <- (a0M + a0F)/2
  a1T <- (a1M + a1F)/2

  f  <- nx[1:2] / c(1, 4)

  if (sex == "male")   ax[1:2] <- c(a0M, a1M) * f
  if (sex == "female") ax[1:2] <- c(a0F, a1F) * f
  if (sex == "total")  ax[1:2] <- c(a0T, a1T) * f

  return(ax)
}


#' Check LifeTable input
#' @param input A list containing the input arguments of the LifeTable functions.
#' @return A list of life table validated data
#' @keywords internal
LifeTable.check <- function(input) {

  with(input, {
    # ----------------------------------------------
    K <- find.my.case(Dx, Ex, mx, qx, lx, dx)
    C <- K$case
    valid_classes <- c("numeric", "matrix", "data.frame", NULL)

    if (!any(K$iclass %in% valid_classes)) {
      stop(paste0("The class of the input should be: ",
                  paste(valid_classes, collapse = ", ")), call. = FALSE)
    }
    # ----------------------------------------------
    SMS <- "contains missing values. These have been replaced with "

    if (!is.null(sex)) {
      if (!any(sex %in% c("male", "female", "total")))
        stop("'sex' should be: 'male', 'female', 'total' or 'NULL'.",
             call. = FALSE)
    }

    if (C == "C1_DxEx") {
      if (any(is.na(Dx))) warning("'Dx'", SMS, 0, call. = FALSE)
      if (any(is.na(Ex))) warning("'Ex'", SMS, 0.01, call. = FALSE)
      Dx[is.na(Dx)] <- 0
      Ex[is.na(Ex) | Ex == 0] <- 0.01
    }

    if (C == "C2_mx") {
      mx <- uxAbove100(x, mx)

    }
    if (C == "C3_qx") {
      qx <- uxAbove100(x, qx)
    }

    if (C == "C4_lx") {
      if (any(is.na(lx))) warning("'lx'", SMS, 0, call. = FALSE)
      lx[is.na(lx) & x >= 100] <- 0
    }

    if (C == "C5_dx") {
      if (any(is.na(dx))) warning("'dx'", SMS, 0, call. = FALSE)
      dx[is.na(dx)] <- 0
    }

    if (!is.null(ax)) {
      if (!is.numeric(ax))
        stop("'ax' must be a numeric scalar (or NULL)", call. = FALSE)

      if (!any(length(ax) %in% c(1, length(x))))
        stop("'ax' must be a scalar of length 1 or a ",
             "vector of the same dimension as 'x'",
             call. = FALSE)
    }

    # Exit
    out <- list(x = x,
                Dx = Dx,
                Ex = Ex,
                mx = mx,
                qx = qx,
                lx = lx,
                dx = dx,
                sex = sex,
                lx0 = lx0,
                ax = ax,
                iclass = K$iclass,
                nLT = K$nLT,
                LTnames = K$LTnames)
    return(out)
  })
}


#' Print LifeTable
#' @param x An object of class \code{"LifeTable"}
#' @param ... Further arguments passed to or from other methods.
#' @return Print data on the console
#' @keywords internal
#' @export
print.LifeTable <- function(x, ...){

  LT <- x$lt
  lt <- with(
    LT,
    data.frame(
      x.int = x.int,
      x = x,
      mx = round(mx, 6),
      qx = round(qx, 6),
      ax = round(ax, 2),
      lx = round(lx),
      dx = round(dx),
      Lx = round(Lx),
      Tx = round(Tx),
      ex = round(ex, 2)
      )
    )

  if (colnames(LT)[1] == "LT") lt <- data.frame(LT = LT$LT, lt)
  dimnames(lt) <- dimnames(LT)
  nx    <- length(unique(LT$x))
  nlt   <- nrow(LT) / nx
  out   <- head_tail(lt, hlength = 6, tlength = 3, ...)
  step  <- diff(LT$x)
  step  <- step[step > 0]
  type1 <- if (all(step == 1)) "Full" else "Abridged"
  type2 <- if (nlt == 1) "Life Table" else "Life Tables"

  cat("\n", type1, " ", type2, "\n\n", sep = "")
  cat("Number of life tables:", nlt, "\n")
  cat("Dimension:", nrow(LT), "x", ncol(LT), "\n")
  cat("Age intervals:", head_tail(lt$x.int, hlength = 3, tlength = 3), "\n\n")
  print(out, row.names = FALSE)
}

# --------------------------------------------------- #
# Author: Marius D. PASCARIU
# Last update: Sun May 02 11:35:52 2021
# --------------------------------------------------- #
remove(list = ls())
library(MortalityLaws)

# Example 1 --- Full life table -----------------
y  <- 1900
x  <- 0:107

Dx <- ahmd$Dx[paste(x), paste(y)]
Ex <- ahmd$Ex[paste(x), paste(y)]

LT1 <- LifeTable(x, Dx = Dx, Ex = Ex)
LT2 <- LifeTable(x, mx = LT1$lt$mx)
LT3 <- LifeTable(x, qx = LT1$lt$qx)
LT4 <- LifeTable(x, lx = LT1$lt$lx)
LT5 <- LifeTable(x, dx = LT1$lt$dx)

LT6  <- LifeTable(x, Dx = Dx, Ex = Ex, ax = 0.5)
LT7  <- LifeTable(x, mx = LT6$lt$mx, ax = 0.5)
LT8  <- LifeTable(x, qx = LT6$lt$qx, ax = 0.5)
LT9  <- LifeTable(x, lx = LT6$lt$lx, ax = 0.5)
LT10 <- LifeTable(x, dx = LT6$lt$dx, ax = 0.5)

# Example 2 --- Abridge life table ------------
x2  = c(0, 1, seq(5, 110, by = 5))
mx2 = c(.053, .005, .001, .0012, .0018, .002, .003, .004,
       .004, .005, .006, .0093, .0129, .019, .031, .049,
       .084, .129, .180, .2354, .3085, .390, .478, .551)
LT11 = LifeTable(x2, mx = mx2, sex = "female")
LT12 = LifeTable(x2, qx = LT11$lt$qx, sex = NULL)
LT13 = LifeTable(x2, lx = LT11$lt$lx, sex = "male")
LT14 = LifeTable(x2, dx = LT11$lt$dx, sex = "total")


x3 = c(0, 1, 2, 3, 4, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 70)
dx = c(11728, 1998, 2190, 1336, 637, 1927, 420, 453, 475, 905, 1168,
       2123, 2395, 3764, 5182, 6555, 8652, 10687, 37405)
LT15 <- LifeTable(x = x3, dx = dx)


# Example 3 --- Abridge life table w ax ------------
ax <- LT15$lt$ax
ax[1] <- 0.1
LT16 <- LifeTable(x = x3, dx = dx, ax = ax)


# TESTS ----------------------------------------------
# expect_warning((LT16 = LifeTable(x = 0:110, mx = ahmd$mx)))

foo.test.lt <- function(X) {
  cn = c("x", "mx", "qx", "ax", "lx", "dx", "Lx", "Tx", "ex")
  test_that("LifeTable works fine", {
    expect_true(all(X$lt[, cn] >= 0))           # All values in LT are positive
    expect_false(all(is.na(X$lt$ex)))           # ex does not contain NA's
    expect_identical(class(X$lt$ex), "numeric") # All ex is of the class numeric
    expect_true(X$lt$ex[1] >= rev(X$lt$ex)[1])  # e[x] at the beginnig is greater or equat with e[x] at the end
    expect_equal(sum(X$lt$dx), X$lt$lx[1])      # The distribution of deaths sums up to unity
    expect_true(X$lt$qx[nrow(X$lt)] >= 0.99999)  # The life table should always close with q[x] = 1
    expect_output(print(X))                     # The print function works
  })
}

for (j in 1:16) {
  print(j)
  foo.test.lt(X = get(paste0("LT", j)))
}


# round(LT1$lt$mx - LT2$lt$mx, 10)
# round(LT1$lt$mx - LT3$lt$mx, 10)
# round(LT1$lt$mx - LT4$lt$mx, 10)
# round(LT1$lt$mx - LT5$lt$mx, 10)

test_lt_consistency <- function(benchmark_LT, LT) {
  n <- nrow(benchmark_LT$lt)   # The last row can be different depending how the LT is closed. Do not test last row.
  B <- round(benchmark_LT$lt[-n, -1], 7)
  L <- round(LT$lt[-n, -1], 7)
  test_that("Identical LT estimates", {
    expect_identical(B$mx, L$mx)
    expect_identical(B$qx, L$qx)
    expect_identical(B$dx, L$dx)
    expect_identical(B$lx, L$lx)
    expect_identical(B$ex, L$ex)
  })
}

for (k in 2:5) test_lt_consistency(LT1, get(paste0("LT", k)))

for (k in 7:10) test_lt_consistency(LT6, get(paste0("LT", k)))




# ----------------------------------------------
# Test messages

# Error: 'ax' must be a numeric scalar (or NULL)
expect_error(
  LifeTable(x, mx = mx, ax = "ax")
)

# Error: 'ax' must be a scalar of lenght 1 or a vector of the same
# dimension as 'x'
expect_error(
  LifeTable(x, mx = mx, ax = rep(0.5, 3))
)

expect_error(
  # If you input 'Dx' you must input 'Ex' as well, and viceversa
  LifeTable(x, Dx = Dx)
)

expect_error(
  # The input is not specified correctly.
  LifeTable(x, Dx = Dx, Ex = Ex, qx = Ex, mx = Ex)
)

# Error: 'sex' should be: 'male', 'female', 'total' or 'NULL'.
mx <- LT1$lt$mx
expect_error(
  LifeTable(x, mx = mx, sex = "Male")
)

# 'Dx'contains missing values. These have been replaced with 0
Dxi <- Dx
Dxi[2] <- NA
expect_warning(
  LifeTable(x, Dx = Dxi, Ex = Ex)
)

# 'Ex'contains missing values
Exi <- Ex
Exi[12] <- NA
expect_warning(
  LifeTable(x, Dx = Dx, Ex = Exi)
)


# 'lx'contains missing values. These have been replaced with 0
lx <- LT1$lt$lx
lx[106] <- NA
expect_warning(
  LifeTable(x, lx = lx)
)

# 'dx'contains missing values.
dx <- LT1$lt$dx
dx[30] <- NA
expect_warning(
  LifeTable(x, dx = dx)
)



# # The input data contains NA's, Inf
# mx <- LT1$lt$mx
# mx[100] <- Inf
# expect_warning(
#   LifeTable(x, mx = mx)
# )


# ----------------------------------------------------------------------------
# Test print function for multiple tables

expect_output(
  print(LifeTable(x = 0:110, mx = ahmd$mx))
)

# --------------------------------------------
# Author: Marius D PASCARIU
# Date: 2026-05-04 23:02:52
# --------------------------------------------

#' Compute Life Tables from Parameters of a Mortality Law
#'
#' Generate a complete life table directly from the fitted parameters of a 
#' parametric mortality model. This function evaluates the mortality law at 
#' the given ages and passes the resulting death rates (\code{mx}) or death 
#' probabilities (\code{qx}) to \code{\link{LifeTable}} for further 
#' computation of all standard life-table columns (\code{lx}, \code{dx}, 
#' \code{Lx}, \code{Tx}, \code{ex}, etc.).
#'
#' @details
#' This function is designed to work with models that have been fitted 
#' externally (e.g., via \code{\link{MortalityLaw}} or by hand). The 
#' \code{par} argument must contain the estimated coefficients of the 
#' mortality law, and \code{law} must be one of the valid codes listed by 
#' \code{\link{availableLaws}}.
#'
#' \strong{Important caveat: age scaling during fitting}
#'
#' Several mortality laws (e.g., Gompertz, Makeham) internally \emph{scale} 
#' the age vector during optimisation to ensure numerical stability. If the 
#' model was fitted using \code{MortalityLaw} over an age range \code{[a, b]}, 
#' the published coefficients correspond to the \emph{scaled} ages, not the 
#' original ages. Consequently, \code{LawTable} will only produce valid life 
#' tables for ages \eqn{\ge a} (the lower bound of the fitting range). 
#' Attempting to use the same coefficients at younger ages will yield 
#' incorrect results (e.g., life expectancy at age 25 will equal that at 
#' age 45).
#'
#' To determine which models apply age scaling, run:
#' \preformatted{
#' A <- availableLaws()$table
#' A[, c("CODE", "SCALE_X")]
#' }
#' Models with \code{SCALE_X = TRUE} rescale the age vector internally. When 
#' using \code{LawTable} with such a model, make sure the \code{x} argument 
#' starts from the same lower age bound used during fitting.
#'
#' For models that do \emph{not} scale (e.g., Heligman-Pollard \code{"HP"}), 
#' this limitation does not apply, and \code{LawTable} can be used for any 
#' age range.
#'
#' @inheritParams MortalityLaw
#' @inheritParams LifeTable
#'
#' @param par The parameters of the mortality model. Can be:
#'   \itemize{
#'     \item A numeric \strong{vector} containing the coefficients (for a 
#'           single life table).
#'     \item A numeric \strong{matrix} or \strong{data.frame} where each row 
#'           corresponds to a separate set of parameters (producing multiple 
#'           life tables). Column names should match the parameter names of 
#'           the chosen law.
#'   }
#'
#' @inherit LifeTable return details
#'
#' @seealso
#' \code{\link{LifeTable}} for constructing life tables from raw mortality 
#'   data; 
#' \code{\link{MortalityLaw}} for fitting parametric mortality models; 
#' \code{\link{availableLaws}} for the list of implemented laws and their 
#'   scaling behaviour.
#'
#' @author Marius D. Pascariu
#'
#' @examples
#' # Example 1 --- Makeham --- multiple life tables from a matrix of parameters
#'
#' x1 <- 45:100
#' L1 <- "makeham"
#' C1 <- matrix(
#'   c(0.00717, 0.07789, 0.00363,
#'     0.01018, 0.07229, 0.00001,
#'     0.00298, 0.09585, 0.00002,
#'     0.00067, 0.11572, 0.00078),
#'   nrow = 4,
#'   dimnames = list(1:4, c("A", "B", "C"))
#' )
#'
#' LawTable(x = x1, par = C1, law = L1)
#'
#' # ---- Important note on age scaling ----
#'
#' # The Makeham model applies internal age scaling during fitting.
#' # If the coefficients above were estimated over ages 45-100, the life
#' # table produced by LawTable is valid only from age 45 onward.
#'
#' # ---- Example 1B: correct usage ----
#' LawTable(x = 45:100, par = c(0.00717, 0.07789, 0.00363), law = L1)
#'
#' # ---- Example 1C: incorrect usage ----
#' # The code below uses the same coefficients but starts at age 25.
#' # Because the model was fitted on scaled ages (starting at 45),
#' # the life table at age 25 will be meaningless (e.g., e25 equals e45).
#' \dontrun{
#' LawTable(x = 25:100, par = c(0.00717, 0.07789, 0.00363), law = L1)
#' }
#'
#' # ---- How to check which laws apply scaling ----
#' A <- availableLaws()$table
#' A[, c("CODE", "SCALE_X")]
#'
#' # Example 2 --- Heligman-Pollard (no scaling) ---
#'
#' x2 <- 0:110
#' L2 <- "HP"
#' C2 <- c(0.00223, 0.01461, 0.12292, 0.00091,
#'         2.75201, 29.01877, 0.00002, 1.11411)
#'
#' LawTable(x = x2, par = C2, law = L2)
#'
#' # Because "HP" does NOT scale the age vector, the output is valid for
#' # any starting age. Compare:
#' LawTable(x = 3:110, par = C2, law = L2)
#' # Note that e3 = 70.31 in both tables, confirming consistency.
#'
#' @export
LawTable <- function(x, par, law, sex = NULL, lx0 = 1e5, ax = NULL) {

  info    <- addDetails(law)
  scale.x <- info$scale.x
  fn      <- get(law)
  xx      <- if (scale.x) scale_x(x) else x

  if (is.matrix(par) | is.data.frame(par)) {
    hx <- NULL
    for (j in 1:nrow(par)) {
      hxj <- fn(xx, par[j, ])$hx
      hx  <- cbind(hx, hxj)
    }
    dimnames(hx) <- list(x, rownames(par))

  } else {
    hx <- fn(xx, par)$hx
  }

  thisIndex  <- info$model["FIT"]

  if (thisIndex == "q[x]") {
    out <- LifeTable(x = xx, qx = hx, sex = sex, lx0 = lx0, ax = ax)
  }
  if (thisIndex == "mu[x]") {
    out <- LifeTable(x = xx, mx = hx, sex = sex, lx0 = lx0, ax = ax)
  }
  out$call <- match.call()
  return(out)
}

# ------------------------------------------------- #
# Author: Marius D. Pascariu
# Last update: Wed Apr  2 11:03:28 2025
# ------------------------------------------------- #

#' Download the Australian Human Mortality Database (AHMD)
#'
#' Download detailed mortality and population data for different
#' provinces and territories in Australia, in a single object from the
#' \href{https://aushd.org/}{Australian Human Mortality Database}.
#'
#' @details
#' (Description taken from the AHMD website).
#'
#' The Australian Human Mortality Database (AHMD) was created to provide
#' detailed Australian mortality and population data to researchers, students,
#' journalists, policy analysts, and others interested in the history of
#' human longevity. The project is an achievement of the Mortality,
#' Ageing & Health research team in the ANU School of Demography under the
#' supervision of Associate Professor Vladimir Canudas-Romo, in collaboration
#' with demographers at the Max Plank Institute for Demographic Research
#' (Rostock, Germany) and the Department of Demography, University of
#' California at Berkeley.
#'
#' The AHMD is a "satellite" of the Human Mortality Database (HMD),
#' an international database which currently holds detailed data for multiple
#' countries or regions. Consequently, the AHMD's underlying methodology
#' corresponds to the one used for the HMD.
#'
#' The AHMD gathers all required data (deaths counts, births counts,
#' population size, exposure-to-risk, death rates) to compute life tables
#' for Australia, its states and its territories. One of the great advantages
#' of the database is to include data that is validated and corrected, when
#' required, and rendered comparable, if possible, for the period ranging
#' from 1971 thru 2016. For comparison purposes, various life tables published
#' by governmental organizations are also available for download in PDF format.
#'
#' @inheritParams ReadHMD
#' @param regions Specify the region specific data you want to download by
#' adding the AHMD region code/s. Options:
#' \itemize{
#'   \item{\code{"ACT"}} -- Australian Capital Territory;
#'   \item{\code{"NSW"}} -- New South Wales;
#'   \item{\code{"NT"}} -- Northern Territory;
#'   \item{\code{"QLD"}} -- Queensland;
#'   \item{\code{"SA"}} -- South Australia;
#'   \item{\code{"TAS"}} -- Tasmania;
#'   \item{\code{"VIC"}} -- Victoria;
#'   \item{\code{"WA"}} -- Western Australia;
#'   \item{\code{NULL}} -- if \code{NULL} data for all the regions are downloaded.
#'   }
#' @return A \code{ReadAHMD} object that contains:
#'  \item{input}{List with the input values;}
#'  \item{data}{Data downloaded from AHMD;}
#'  \item{download.date}{Time stamp;}
#'  \item{years}{Numerical vector with the years covered in the data;}
#'  \item{ages}{Numerical vector with ages covered in the data.}
#' @author Marius D. Pascariu
#' @seealso
#' \code{\link{ReadHMD}}
#' \code{\link{ReadCHMD}}
#' @examples
#' \donttest{
#' # Download demographic data for Australian Capital Territory and
#' # Tasmania regions in 5x1 format
#'
#' # Death counts. We don't want to export data outside R.
#' AHMD_Dx <- ReadAHMD(what = "Dx",
#'                     regions = c('ACT', 'TAS'),
#'                     interval  = "5x1",
#'                     save = FALSE)
#' AHMD_Dx
#'
#' # Download life tables for female population in all the states and export data.
#' LTF <- ReadAHMD(what = "LT_f", interval  = "5x1", save = FALSE)
#' LTF
#' }
#' @export
ReadAHMD <- function(what,
                     regions = NULL,
                     interval = "1x1",
                     save = FALSE,
                     show = TRUE){
  # Step 1 - Validate input & Progress bar setup
  if (is.null(regions)) {
    regions <- AUSregions()
  }

  input <- as.list(environment())
  check_input_ReadAHMD(input)
  nr <- length(regions)

  if (show) {
    pb <- startpb(0, nr + 1)
    on.exit(closepb(pb))
    setpb(pb, 0)
  }

  # Step 2 - Do the loop for the other regions
  D <- data.frame()
  for (i in 1:nr) {
    if (show) {
      setpb(pb, i)
      cat(paste("      :Downloading", regions[i], "    "))
    }

    D <- rbind(D, ReadHMD.core(what = what,
                               country = regions[i],
                               interval = interval,
                               username = NULL,
                               password = NULL,
                               link = "https://aushd.org/assets/txtFiles/humanMortality/"))
  }

  if(length(D) != 0) {
    out <- list(input = input,
                data = D,
                download.date = date(),
                years = sort(unique(D$Year)),
                ages = unique(D$Age))
    out <- structure(class = "ReadAHMD", out)
  
    # Step 3 - Write a file with the database in your working directory
    if (show) setpb(pb, nr + 1)
    if (save) saveOutput(out, show, prefix = "AHMD")
    
  } else {
    out <- NULL
  }

  # Exit
  return(out)
}


#' region codes
#' @return a vector
#' @keywords internal
AUSregions <- function() {
  c("ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA")
}


#' Check input ReadAHMD
#' @param x a list containing the input arguments from ReadAHMD function
#' @return No return value, called for checking stuff
#' @keywords internal
check_input_ReadAHMD <- function(x) {

  if (!(x$interval %in% data_format())) {
    stop("The interval ", x$interval, " does not exist in AHMD ",
         "Try one of these options:\n", paste(data_format(), collapse = ", "),
         call. = FALSE)
  }
  
  if (!(x$what %in% HMDindices())) {
    stop(x$what, " does not exist in AHMD. Try one of these options:\n",
         paste(HMDindices(), collapse = ", "), call. = FALSE)
  }

  if (all(!(x$regions %in% AUSregions()))) {
    stop("Something is wrong in the region codes supplied.\n",
         "Try one or more of these options:\n",
         paste(AUSregions(), collapse = ", "), call. = FALSE)
  }
}



#' Print ReadCHMD
#' @param x An object of class \code{"ReadCHMD"}
#' @param ... Further arguments passed to or from other methods.
#' @return Print data on console
#' @keywords internal
#' @export
print.ReadAHMD <- function(x, ...){
  what <- x$input$what
  cat("Australian Human Mortality Database\n")
  cat("Web Address   : https://aushd.org\n")
  cat("Download Date :", x$download.date, "\n")
  cat("Type of data  :", what, "\n")
  cat(paste("Interval      :", x$input$interval, "\n"))
  cat(paste("Years         :", x$years[1], "--", rev(x$years)[1], "\n"))
  cat(paste("Ages          :", ageMsg(what, x), "\n"))
  cat("Regions       :", x$input$regions, "\n")
  cat("\nData:\n")
  print(head_tail(x$data, hlength = 5, tlength = 5))
}

# ------------------------------------------------- #
# Author: Marius D. Pascariu
# Last update: Wed Apr  2 11:42:35 2025
# ------------------------------------------------- #

remove(list = ls())

# Wrong index
expect_error(ReadAHMD(what = "DxDD"))

# Wrong region
expect_error(ReadAHMD(what    = "Dx",
                      regions = "ACTT"))

# Wrong interval
expect_error(ReadAHMD(what     = "Dx",
                      regions  = "ACT",
                      interval = "1x50"))

# Wrong region for the index
expect_error(
  ReadAHMD(what     = "LT_fc",
           regions  = "TAS_LT_fc",
           interval = "1x1",
           show     = FALSE))

# Wrong interval for the index
expect_error(
  ReadAHMD(what     = "e0",
           regions  = "TAS",
           interval = "5x1",
           show     = FALSE))

expect_output(
  print(AHMD_sample)
)

# expect_error(ReadAHMD(what = "LT_f", regions = "TAS", interval = "1x1"))


# The tests below have been removed because in case the internet source is
# temporary not working the CRAN will consider it as a software failure and will
# demand correction. Unfortunately, we can test only the error messages i.e. the
# automated checks put in place.

# # Test the show arg and print function
# expect_silent(D <- ReadAHMD(what     = "LT_f",
#                             regions  = "ACT",
#                             interval = "5x10",
#                             show     = F))
# expect_output(print(D))

# --------------------------------------------
# Author: Marius D PASCARIU
# Date: 2026-05-04 23:32:19
# --------------------------------------------

#' Print MortalityLaw
#' @param x an object of class \code{"MortalityLaw"}
#' @param ... further arguments passed to or from other methods.
#' @return Print data on console
#' @keywords internal
#' @export
print.MortalityLaw <- function(x, ...) {
  L    <- x$input$law == "custom.law"
  info <- if (L) "Custom Mortality Law" else as.matrix(x$info$model.info[, c(2, 3)])
  cat(paste(info, collapse = " model: "))
  fv <- ifelse(!is.null(x$input$qx), 'qx', 'mx')
  cat("\nFitted values:", fv, "\n")
}


#' Summary MortalityLaw
#' @param object an object of class \code{"MortalityLaw"}
#' @param digits number of digits to display.
#' @param ... additional arguments affecting the summary produced.
#' @return A list of model diagnosis
#' @keywords internal
#' @export
summary.MortalityLaw <- function(object, ...,
                                 digits = max(3L, getOption("digits") - 3L)) {
  x     <- object
  L1    <- x$input$law == "custom.law"
  mi    <- if (L1) "Custom Mortality Law" else as.matrix(x$info$model.info[, c(2, 3)])
  res   <- summary(as.vector(as.matrix(x$residuals)))
  fv    <- ifelse(!is.null(x$input$qx), 'qx', 'mx')
  gof   <- round(x$goodness.of.fit, digits)
  param <- round(coef(x), digits)
  sigma <- mean(sqrt(x$deviance))
  nc    <- nrow(param)
  L2    <- is.null(nc)
  L3    <- x$input$opt.method %in% c("poissonL", "binomialL")

  if (!L2) {
    if (nc > 4) {
      param <- head_tail(param, hlength = 2, tlength = 2, digits = digits)
      gof   <- head_tail(gof, hlength = 2, tlength = 2, digits = digits)
    }
  }
  out  <- list(info = mi, call = x$info$call, gof = gof, sigma = sigma,
               fv = fv, resid = res, param = param, df = x$df, digits = digits,
               L1 = L1, L2 = L2, L3 = L3)
  out  <- structure(class = "summary.MortalityLaw", out)
  return(out)
}


#' Print summary.MortalityLaw
#' @param x an object of class \code{"summary.MortalityLaw"}
#' @param ... additional arguments affecting the summary produced.
#' @return Print data on console
#' @keywords internal
#' @export
print.summary.MortalityLaw <- function(x, ...) {
  with(x, {
    cat(paste(info, collapse = " model: "))
    cat("\nFitted values:", fv)
    cat("\n\nCall: ")
    print(call)
    cat('\nDeviance Residuals:\n')
    print(round(resid, digits))
    cat("\nParameters:\n")
    print(param)
    sg <- format(signif(sigma, digits))
    if (L3) {
      cat("\nGoodness of fit:\n")
      print(gof)
    }
    if (L2) {
      cat("\nResidual standard error:", sg, "on", df[2], "degrees of freedom")
    } else {
      cat("\nAverage residual standard error:", sg, "on", df[1, 2], "degrees of freedom")
    }
  })
}


#' logLik function for MortalityLaw
#' @inheritParams print.MortalityLaw
#' @return Model log-likelihood value
#' @keywords internal
#' @export
logLik.MortalityLaw <- function(object, ...) {
  c(object$goodness.of.fit["logLik"])
}

#' AIC function for MortalityLaw
#' @inheritParams print.MortalityLaw
#' @return model AIC value
#' @keywords internal
#' @export
AIC.MortalityLaw <- function(object, ...) {
  c(object$goodness.of.fit["AIC"])
}

#' deviance function for MortalityLaw
#' @inheritParams print.MortalityLaw
#' @return model deviance value
#' @keywords internal
#' @export
deviance.MortalityLaw <- function(object, ...) {
  object$deviance
}

#' df.residual function for MortalityLaw
#' @inheritParams print.MortalityLaw
#' @return model residual value
#' @keywords internal
#' @export
df.residual.MortalityLaw <- function(object, ...) {
  object$df[2]
}


#' Predict function for MortalityLaw
#' @param object An object of class \code{"MortalityLaw"}
#' @param x Vector of ages to be considered in prediction
#' @param ... Additional arguments affecting the predictions produced.
#' @return A vector of predicted hazard rates
#' @seealso \code{\link{MortalityLaw}}
#' @author Marius D. Pascariu
#' @examples
#' # Extrapolate old-age mortality with the Kannisto model
#' # Fit ages 80-94 and extrapolate up to 120.
#'
#' Mx <- ahmd$mx[paste(80:94), "1950"]
#' M1 <- MortalityLaw(x = 80:94, mx  = Mx, law = 'kannisto')
#' fitted(M1)
#' predict(M1, x = 80:120)
#'
#' # See more examples in MortalityLaw function help page.
#' @export
predict.MortalityLaw <- function(object, x, ...){
  if (min(x) < 0) stop("'x' must be greater or equal to zero.", call. = F)
  law   <- object$input$law
  sx    <- object$input$scale.x
  new.x <- x

  if (sx) {
    fit.this.x <- object$input$fit.this.x
    d <- fit.this.x[1] - scale_x(fit.this.x)[1]
    new.x <- x - d
  }

  Par <- coef(object)

  if (!is.matrix(Par)) {
    Par <- matrix(Par, nrow = 1, dimnames = list("", names(Par)))
  }

  fn <- if (law == "custom.law") object$input$custom.law else get(law)
  hx <- apply(X = Par, 1, FUN = function(X) fn(x = new.x, par = X)$hx)
  rownames(hx) <- x

  if (ncol(hx) == 1) {
    hx <- as.numeric(hx)
    names(hx) <- x
  }
  return(hx)
}

# --------------------------------------------
# Author: Marius D PASCARIU
# Date: 2026-05-04 23:32:44
# --------------------------------------------

# ---- LAWS ---------------------------------------

#' Gompertz Mortality Law - 1825
#'
#' @param x vector of age at the beginning of the age classes
#' @param par parameters of the selected model. If NULL the
#' default values will be assigned automatically.
#' @examples gompertz(x = 45:90)
#' @return A list of rates and model parameters
#' @keywords internal
#' @export
gompertz <- function(x, par = NULL){
  par <- bring_parameters('gompertz', par)
  hx  <- with(as.list(par), A*exp(B*x) )
  Hx  <- with(as.list(par), A/B * (exp(B*x) - 1) )
  Sx  <- exp(-Hx)
  return(list(hx = hx, par = par, Sx = Sx))
}


#' Gompertz Mortality Law - informative parameterization
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples gompertz0(x = 45:90)
#' @keywords internal
#' @export
gompertz0 <- function(x, par = NULL){
  par <- bring_parameters('gompertz0', par)
  hx  <- with(as.list(par), (1/sigma) * exp((x - M)/sigma) )
  Hx  <- with(as.list(par), exp(-M/sigma) * (exp(x/sigma) - 1) )
  Sx  <- exp(-Hx)
  return(list(hx = hx, par = par, Sx = Sx))
}

#' Inverse-Gompertz Mortality Law - informative parameterization
#'
#' m - is a measure of location because it is the mode of the density, m > 0
#' sigma - represents the dispersion of the density about the mode, sigma > 0
#' @inheritParams gompertz 
#' @inherit gompertz return
#' @examples invgompertz(x = 15:25)
#' @keywords internal
#' @export
invgompertz <- function(x, par = NULL){
  par <- bring_parameters('invgompertz', par)
  hx  <- with(as.list(par), 1/sigma * exp(-(x - M)/sigma) / (exp(exp(-(x - M)/sigma)) - 1))
  Sx  <- with(as.list(par), (1 - exp(-exp(-(x - M)/sigma))) / (1 - exp(-exp(M/sigma))))
  Hx  <- -log(Sx)
  return(list(hx = hx, par = par, Sx = Sx))
}

#' Makeham Mortality Law - 1860
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples makeham(x = 45:90)
#' @keywords internal
#' @export
makeham <- function(x, par = NULL){
  par <- bring_parameters('makeham', par)
  hx  <- with(as.list(par), A*exp(B*x) + C)
  Hx  <- with(as.list(par), A/B * (exp(B*x) - 1) + x*C )
  Sx  <- exp(-Hx)
  return(list(hx = hx, par = par))
}


#' Makeham Mortality Law - informative parameterization
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples makeham0(x = 45:90)
#' @keywords internal
#' @export
makeham0 <- function(x, par = NULL){
  par <- bring_parameters('makeham0', par)
  hx <- with(as.list(par), (1/sigma) * exp((x - M)/sigma) + C)
  Hx <- with(as.list(par), exp(-M/sigma) * (exp(x/sigma) - 1) + x*C)
  Sx <- exp(-Hx)
  return(list(hx = hx, par = par, Sx = Sx))
}


#' Opperman Mortality Law - 1870
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples opperman(x = 1:25)
#' @keywords internal
#' @export
opperman <- function(x, par = NULL){
  par <- bring_parameters('opperman', par)
  x  <- x + 1
  hx <- with(as.list(par), A/sqrt(x) - B + C*sqrt(x))
  hx <- pmax(0, hx)
  return(list(hx = hx, par = par))
}


#' Thiele Mortality Law - 1871
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples thiele(x = 0:100)
#' @keywords internal
#' @export
thiele <- function(x, par = NULL){
  par <- bring_parameters('thiele', par)
  mu1 <- with(as.list(par), A*exp(-B*x) )
  mu2 <- with(as.list(par), C*exp(-.5*D*(x - E)^2) )
  mu3 <- with(as.list(par), F_*exp(G*x) )
  hx <- ifelse(x == 0, mu1 + mu3, mu1 + mu2 + mu3)
  return(list(hx = hx, par = par))
}


#' Wittstein Mortality Law - 1883
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples wittstein(x = 0:100)
#' @keywords internal
#' @export
wittstein <- function(x, par = NULL){
  par <- bring_parameters('wittstein', par)
  hx  <- with(as.list(par), (1/B)*A^-((B*x)^N) + A^-((M - x)^N) )
  return(list(hx = hx, par = par))
}


#' Weibull Mortality Law - 1939
#'
#' Note that if sigma > m, then the mode of the density is 0 and hx is a
#' non-increasing function of x, while if sigma < m, then the mode is
#' greater than 0 and hx is an increasing function.
#' m > 0 is a measure of location
#' sigma > 0 is measure of dispersion
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples weibull(x = 1:20)
#' @keywords internal
#' @export
weibull <- function(x, par = NULL){
  par <- bring_parameters('weibull', par)
  hx <- with(as.list(par), 1/sigma * (x/M)^(M/sigma - 1) )
  hx[x == 0] <- 1
  Hx <- with(as.list(par), (x/M)^(M/sigma) )
  Sx <- exp(-Hx)
  return(list(hx = hx, par = par, Sx = Sx))
}


#' Inverse-Weibull Mortality Law
#'
#' The Inverse-Weibull proves useful for modelling the childhood and teenage years,
#' because the logarithm of h(x) is a concave function.
#' m > 0 is a measure of location
#' sigma > 0 is measure of dispersion
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples invweibull(x = 1:20)
#' @keywords internal
#' @export
invweibull <- function(x, par = NULL){
  par <- bring_parameters('invweibull', par)
  hx <- with(as.list(par),
             (1/sigma) * (x/M)^(-M/sigma - 1) / (exp((x/M)^(-M/sigma)) - 1) )
  Hx <- with(as.list(par), -log(1 - exp(-(x/M)^(-M/sigma))) )
  Sx <- exp(-Hx)
  return(list(hx = hx, par = par, Sx = Sx))
}


#' Perks Model - 1932
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples perks(x = 50:100)
#' @keywords internal
#' @export
perks <- function(x, par = NULL){
  par <- bring_parameters('perks', par)
  hx  <- with(as.list(par), (A + B*C^x) / (B*(C^-x) + 1 + D*C^x) )
  return(list(hx = hx, par = par))
}


#' Van der Maen Model - 1943
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples vandermaen(x = 0:100)
#' @keywords internal
#' @export
vandermaen <- function(x, par = NULL){
  par <- bring_parameters('vandermaen', par)
  hx  <- with(as.list(par), A + B*x + C*(x^2) + I/(N - x))
  return(list(hx = hx, par = par))
}


#' Van der Maen 2 Model - 1943
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples vandermaen(x = 0:100)
#' @keywords internal
#' @export
vandermaen2 <- function(x, par = NULL){
  par <- bring_parameters('vandermaen2', par)
  hx  <- with(as.list(par), A + B*x + I/(N - x))
  return(list(hx = hx, par = par))
}


#' Strehler-Mildvan Model - 1960
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples strehler_mildvan(x = 30:85)
#' @keywords internal
#' @export
strehler_mildvan <- function(x, par = NULL){
  par <- bring_parameters('strehler_mildvan', par)
  hx  <- with(as.list(par), K * exp(-V0 * (1 - B * x)/D) )
  return(list(hx = hx, par = par))
}


#' Beard Model - 1971
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples beard(x = 50:100)
#' @keywords internal
#' @export
beard <- function(x, par = NULL){
  par <- bring_parameters('beard', par)
  hx  <- with(as.list(par), (A*exp(B*x)) / (1 + K*A*exp(B*x)) )
  return(list(hx = hx, par = par))
}


#' Makeham-Beard Model - 1971
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples beard_makeham(x = 0:100)
#' @keywords internal
#' @export
beard_makeham <- function(x, par = NULL){
  par <- bring_parameters('beard_makeham', par)
  hx  <- with(as.list(par), A*exp(B*x) / (1 + K*A*exp(B*x)) + C)
  return(list(hx = hx, par = par))
}


#' Gamma-Gompertz Model as in Vaupel et al. (1979)
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples ggompertz(x = 50:120)
#' @keywords internal
#' @export
ggompertz <- function(x, par = NULL){
  par <- bring_parameters('ggompertz', par)
  hx  <- with(as.list(par), (A*exp(B*x)) / (1 + (A*G/B)*(exp(B*x) - 1)) )
  return(list(hx = hx, par = par))
}


#' Quadratic Model
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples quadratic(x = 0:100)
#' @keywords internal
#' @export
quadratic <- function(x, par = NULL){
  par <- bring_parameters('quadratic', par)
  hx  <- with(as.list(par), A + B*x + C*(x^2))
  return(list(hx = hx, par = par))
}


#' Siler Mortality Law - 1979
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples siler(x = 0:100)
#' @keywords internal
#' @export
siler <- function(x, par = NULL){
  par <- bring_parameters('siler', par)
  hx <- with(as.list(par), A*exp(-B*x) + C + D*exp(E*x))
  return(list(hx = hx, par = par))
}


#' Heligman-Pollard Mortality Law - 8 parameters - 1980
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples HP(x = 0:100)
#' @keywords internal
#' @export
HP <- function(x, par = NULL){
  par <- bring_parameters('HP', par)
  mu1 <- with(as.list(par), A^((x + B)^C) + G*H^x )
  mu2 <- with(as.list(par), D*exp(-E*(log(x/F_))^2) )
  eta <- ifelse(x == 0, mu1, mu1 + mu2)
  hx <- eta/(1 + eta)
  return(list(hx = hx, par = par))
}

#' Heligman-Pollard 2 Mortality Law - 8 parameters
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples HP2(x = 0:100)
#' @keywords internal
#' @export
HP2 <- function(x, par = NULL){
  par <- bring_parameters('HP2', par)
  mu1 <- with(as.list(par), A^((x + B)^C) + (G*H^x)/(1 + G*H^x) )
  mu2 <- with(as.list(par), D*exp(-E*(log(x/F_))^2) )
  eta <- ifelse(x == 0, mu1, mu1 + mu2)
  hx <- eta
  return(list(hx = hx, par = par))
}

#' Heligman-Pollard 3 Mortality Law - 9 parameters
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples HP3(x = 0:100)
#' @keywords internal
#' @export
HP3 <- function(x, par = NULL){
  par <- bring_parameters('HP3', par)
  mu1 <- with(as.list(par), A^((x + B)^C) + (G*H^x)/(1 + K*G*H^x) )
  mu2 <- with(as.list(par), D*exp(-E*(log(x/F_))^2) )
  eta <- ifelse(x == 0, mu1, mu1 + mu2)
  hx <- eta
  return(list(hx = hx, par = par))
}

#' Heligman-Pollard 4 Mortality Law - 9 parameters
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples HP4(x = 0:100)
#' @keywords internal
#' @export
HP4 <- function(x, par = NULL){
  par <- bring_parameters('HP4', par)
  mu1 <- with(as.list(par), A^((x + B)^C) + (G*H^(x^K)) / (1 + G*H^(x^K)) )
  mu2 <- with(as.list(par), D*exp(-E*(log(x/F_))^2) )
  eta <- ifelse(x == 0, mu1, mu1 + mu2)
  hx <- eta
  return(list(hx = hx, par = par))
}


#' Martinelle Model - 1987
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples martinelle(x = 0:100)
#' @keywords internal
#' @export
martinelle <- function(x, par = NULL){
  par <- bring_parameters('martinelle', par)
  hx  <- with(as.list(par), (A*exp(B*x) + C) / (1 + D*exp(B*x)) + K*exp(B*x))
  return(list(hx = hx, par = par))
}


#' Rogers-Planck Model - 1983
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples rogersplanck(x = 0:100)
#' @keywords internal
#' @export
rogersplanck <- function(x, par = NULL){
  par <- bring_parameters('rogersplanck', par)
  hx  <- with(as.list(par),
          A0 + A1*exp(-A*x) + A2*exp(B*(x - U) - exp(-C*(x - U))) + A3*exp(D*x))
  return(list(hx = hx, par = par))
}


#' Carriere Mortality Law - 1992
#'
#' Carriere1 = weibull + invweibull + gompertz
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples carriere1(x = 0:100)
#' @keywords internal
#' @export
carriere1 <- function(x, par = NULL){
  par <- bring_parameters('carriere1', par)
  # Compute distribution functions
  S_wei  <- weibull(x, par[c('sigma1', 'M1')])$Sx
  S_iwei <- invweibull(x, par[c('sigma1', 'M2')])$Sx
  S_gom  <- gompertz0(x, par[c('sigma3', 'M3')])$Sx

  f1 <- par['P1'] <- max(0.0001, min(par['P1'], 1))
  f2 <- par['P2'] <- max(0.0001, min(par['P2'], 1))
  f3 <- 1 - f1 - f2

  Sx <- f1*S_wei + f2*S_iwei + f3*S_gom
  Sx <- pmax(0, pmin(1, Sx))
  Hx <- -log(Sx)
  hx <- c(Hx[1], diff(Hx)) # here we will need a numerical solution!
  return(list(hx = hx, par = par))
}


#' Carriere Mortality Law - 1992
#'
#' Carriere2 = weibull + invgompertz + gompertz
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples carriere2(x = 0:100)
#' @keywords internal
#' @export
carriere2 <- function(x, par = NULL){
  par <- bring_parameters('carriere2', par)
  # Compute distribution functions
  S_wei  <- weibull(x, par[c('sigma1', 'M1')])$Sx
  S_igom <- invgompertz(x, par[c('sigma2', 'M2')])$Sx
  S_gom  <- gompertz0(x, par[c('sigma3', 'M3')])$Sx

  f1 <- par['P1'] <- max(0.0001, min(par['P1'], 1))
  f2 <- par['P2'] <- max(0.0001, min(par['P2'], 1))
  f3 <- 1 - f1 - f2

  Sx <- f1*S_wei + f2*S_igom + f3*S_gom
  Sx <- pmax(0, pmin(1, Sx))
  Hx <- -log(Sx)
  hx <- c(Hx[1], diff(Hx)) # here we will need a numerical solution!
  return(list(hx = hx, par = par))
}


#' Kostaki Model - 1992
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples kostaki(x = 0:100)
#' @keywords internal
#' @export
kostaki <- function(x, par = NULL){
  par <- bring_parameters('kostaki', par)
  with(as.list(par), {
    # Sometimes the difference between estimated parameters E1 and E2 is
    # very large, in which case the resulting mortality curve will exhibit
    # a significant artificial jump in one age group. I am imposing a
    # restriction below to limit this behaviour.
    if (E1 >= 50*E2) E2 <- E1/50 # This hack seems to work.

    L   <- x <= F_    # Logical
    mu1 <- A^((x + B)^C) + G*H^x
    e1  <- -(E1*log(x/F_))^2
    e2  <- -(E2*log(x/F_))^2
    mu2 <- D*exp(L*e1 + (!L)*e2)
    eta <- ifelse(x == 0, mu1, mu1 + mu2)
    hx  <- eta/(1 + eta)
    return(list(hx = hx, par = par))
  })
}


#' Kannisto Mortality Law - 1998
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples kannisto(x = 85:120)
#' @keywords internal
#' @export
kannisto <- function(x, par = NULL){
  par <- bring_parameters('kannisto', par)
  with(as.list(par), {
    hx  <- A*exp(B*x) / (1 + A*exp(B*x))
    Hx  <- 1/A * log((1 + B*exp(B*x)) / (1 + A))
    Sx  <- exp(-Hx)
    return(list(hx = hx, par = par))
  })
}


#' Kannisto-Makeham Mortality Law - 1998
#' @inheritParams gompertz
#' @inherit gompertz return
#' @examples kannisto_makeham(x = 85:120)
#' @keywords internal
#' @export
kannisto_makeham <- function(x, par = NULL){
  par <- bring_parameters('kannisto_makeham', par)
  with(as.list(par), {
    hx  <- A*exp(B*x) / (1 + A*exp(B*x)) + C
    return(list(hx = hx, par = par))
  })
}


#' Bring or Rename Starting Parameters in the Law Functions
#' @inheritParams MortalityLaw
#' @inheritParams gompertz
#' @return Vector or initial model parameters
#' @keywords internal
bring_parameters <- function(law, par = NULL) {
  Spar <- switch(law,
            demoivre    = c(A = 105),
            gompertz    = c(A = 0.0002, B = 0.13),
            gompertz0   = c(sigma = 7.7, M = 49),
            invgompertz = c(sigma = 7.7, M = 49),
            makeham     = c(A = .0002, B = .13, C = .001),
            makeham0    = c(sigma = 7.692308, M = 49, C = .001),
            opperman    = c(A = .04, B = .0004, C = .001),
            thiele      = c(A = .02474, B = .3, C = .004, D = .5,
                           E = 25, F_ = .0001, G = .13),
            wittstein   = c(A = 1.5, B = 1, N = .5, M = 100),
            perks       = c(A = .002, B = .13, C = .01, D = .01),
            weibull     = c(sigma = 2, M = 1),
            invweibull  = c(sigma = 10, M = 5),
            vandermaen  = c(A = .01, B = 1, C = .01, I = 100, N = 200),
            vandermaen2 = c(A = .01, B = 1, I = 100, N = 200),
            strehler_mildvan = c(K = .01, V0 = 2.5, B = 0.2, D = 6),
            quadratic   = c(A = .01, B = 1, C = .01),
            beard       = c(A = .002, B = .13, K = 1),
            beard_makeham = c(A = .002, B = .13, C = .01, K = 1),
            ggompertz = c(A = .002, B = .13, G = 1),
            siler     = c(A = .0002, B = .13, C = .001, D = .001, E = .013),
            HP        = c(A = .0005, B = .004, C = .08, D = .001,
                          E = 10, F_ = 17, G = .00005, H = 1.1),
            HP2       = c(A = .0005, B = .004, C = .08, D = .001,
                          E = 10, F_ = 17, G = .00005, H = 1.1),
            HP3       = c(A = .0005, B = .004, C = .08, D = .001,
                          E = 10, F_ = 17, G = .00005, H = 1.1, K = 1),
            HP4       = c(A = .0005, B = .004, C = .08, D = .001,
                          E = 10, F_ = 17, G = .00005, H = 1.1, K = 1),
            rogersplanck = c(A0 = .0001, A1 = .02, A2 = .001, A3 = .0001,
                             A = 2, B = .001, C = 100, D = .1, U = .33),
            martinelle = c(A = .001, B = .13, C = .001, D = 0.1, K = .001),
            kostaki    = c(A = .0005, B = .01, C = .10, D = .001,
                            E1 = 3, E2 = .1, F_ = 25, G = .00005, H = 1.1),
            carriere1  = c(P1 = .003, sigma1 = 15, M1 = 2.7,
                           P2 = .007, sigma2 = 6, M2 = 3,
                           sigma3 = 9.5, M3 = 88),
            carriere2  = c(P1 = .01, sigma1 = 2, M1 = 1,
                           P2 = .01, sigma2 = 7, M2 = 49,
                           sigma3 = 7, M3 = 49),
            kannisto   = c(A = 0.5, B = 0.13),
            kannisto_makeham = c(A = 0.5, B = 0.13, C = 0.001)
            )
  if (is.null(par)) par <- Spar
  # If 'par' is provided, just give them a name anyway.
  names(par) <- names(Spar)
  return(par)
}

# --------------------------------------------
# Author: Marius D PASCARIU
# Date: 2026-05-04 23:32:03
# --------------------------------------------

#' Download the Canadian Human Mortality Database (CHMD)
#'
#' Download detailed mortality and population data for different
#' provinces and territories in Canada, in a single object from the
#' Canadian Human Mortality Database.
#'
#' @details
#' (Description taken from the CHMD website).
#'
#' The Canadian Human Mortality Database (CHMD) was created to provide detailed
#' Canadian mortality and population data to researchers, students, journalists,
#' policy analysts, and others interested in the history of human longevity.
#' The project is an achievement of the Mortality and Longevity research team at
#' the Department of Demography, Universite de Montreal, under the supervision
#' of Professor Robert Bourbeau, in collaboration with demographers at the
#' Max Plank Institute for Demographic Research (Rostock, Germany) and the
#' Department of Demography, University of California at Berkeley.
#' Nadine Ouellette, researcher at the Institut national d'etudes demographiques
#' in Paris and member of the Mortality and Longevity research team at the
#' Universite de Montreal, is in charge of computing all CHMD life tables and
#' updating the CHMD web site.
#'
#' The CHMD is a "satellite" of the Human Mortality Database (HMD), an
#' international database which currently holds detailed data for multiple
#' countries or regions. Consequently, the CHMD's underlying methodology
#' corresponds to the one used for the HMD.
#'
#' The CHMD gathers all required data (deaths counts, births counts, population
#' size, exposure-to-risk, death rates) to compute life tables for Canada,
#' its provinces and its territories. One of the great advantages of the
#' database is to include data that is validated and corrected, when required,
#' and rendered comparable, if possible, for the period ranging from 1921
#' thru 2011. For comparison purposes, various life tables published by
#' governmental organizations are also available for download in PDF format.
#'
#' @inheritParams ReadHMD
#' @param what What type of data are you looking for? The following options are
#' available: \itemize{
#'   \item{\code{"births"}} -- birth records;
#'   \item{\code{"Dx_lexis"}} -- deaths by Lexis triangles;
#'   \item{\code{"population"}} -- population size;
#'   \item{\code{"Dx"}} -- death counts;
#'   \item{\code{"Ex"}} -- exposure-to-risk;
#'   \item{\code{"mx"}} -- central death-rates;
#'   \item{\code{"LT_f"}} -- period life tables for females;
#'   \item{\code{"LT_m"}} -- period life tables for males;
#'   \item{\code{"LT_t"}} -- period life tables both sexes combined;
#'   \item{\code{"e0"}} -- period life expectancy at birth;
#'   }
#' @param regions Specify the region specific data you want to download by
#' adding the CHMD region code/s. Options:
#' \itemize{
#'   \item{\code{"CAN"}} -- Canada - Sum of Canadian provinces and territories;
#'   \item{\code{"NFL"}} -- Newfoundland & Labrador;
#'   \item{\code{"PEI"}} -- Prince Edward Island;
#'   \item{\code{"NSC"}} -- Nova Scotia;
#'   \item{\code{"NBR"}} -- New Brunswick;
#'   \item{\code{"QUE"}} -- Quebec;
#'   \item{\code{"ONT"}} -- Ontario;
#'   \item{\code{"MAN"}} -- Manitoba;
#'   \item{\code{"SAS"}} -- Saskatchewan;
#'   \item{\code{"ALB"}} -- Alberta;
#'   \item{\code{"BCO"}} -- British Columbia;
#'   \item{\code{"NWT"}} -- Northwest Territories & Nunavut;
#'   \item{\code{"YUK"}} -- Yukon;
#'   \item{\code{NULL}} -- if \code{NULL} data for all the regions are downloaded.
#'   }
#' @return A \code{ReadCHMD} object that contains:
#'  \item{input}{List with the input values;}
#'  \item{data}{Data downloaded from CHMD;}
#'  \item{download.date}{Time stamp;}
#'  \item{years}{Numerical vector with the years covered in the data;}
#'  \item{ages}{Numerical vector with ages covered in the data.}
#' @author Marius D. Pascariu
#' @seealso
#' \code{\link{ReadHMD}}
#' \code{\link{ReadAHMD}}
#' @examples
#' \donttest{
#' # Download demographic data for Quebec and Saskatchewan regions in 1x1 format
#'
#' # Death counts. We don't want to export data outside R.
#' CHMD_Dx <- ReadCHMD(what = "Dx",
#'                     regions = c('QUE', 'SAS'),
#'                     interval  = "1x1",
#'                     save = FALSE)
#'
#' # Download life tables for female population. To export data use save = TRUE.
#' LTF <- ReadCHMD(what = "LT_f",
#'                 regions = c('QUE', 'SAS'),
#'                 interval  = "1x1",
#'                 save = FALSE)
#' }
#' @export
ReadCHMD <- function(what,
                     regions = NULL,
                     interval = "1x1",
                     save = FALSE,
                     show = TRUE){
  # Step 1 - Validate input & Progress bar setup
  if (is.null(regions)) {
    regions <- CANregions()
  }

  input <- as.list(environment())
  check_input_ReadCHMD(input)
  nr <- length(regions)

  if (show) {
    pb <- startpb(0, nr + 1)
    on.exit(closepb(pb))
    setpb(pb, 0)
  }

  # Step 2 - Do the loop for the other regions
  D <- data.frame()
  for (i in 1:nr) {
    if (show) {
      setpb(pb, i)
      cat(paste("      :Downloading", regions[i], "    "))
    }

    D <- rbind(D, ReadHMD.core(
      what = what,
      country = regions[i],
      interval = interval,
      username = NULL,
      password = NULL,
      link = "https://www.prdh.umontreal.ca/BDLC/data/"))
  }

  if (length(D) != 0) {
    out <- list(input = input,
                data = D,
                download.date = date(),
                years = sort(unique(D$Year)),
                ages = unique(D$Age))
    out <- structure(class = "ReadCHMD", out)
  
    # Step 3 - Write a file with the database in your working directory
    if (show) setpb(pb, nr + 1)
    if (save) saveOutput(out, show, prefix = "CHMD")
    
  } else {
    out <- NULL
  }

  # Exit
  return(out)
}


#' Country codes
#' @return a vector
#' @keywords internal
CANregions <- function() {
  c("CAN",
    "NFL",
    "PEI",
    "NSC",
    "NBR",
    "QUE",
    "ONT",
    "MAN",
    "SAS",
    "ALB",
    "BCO",
    "NWT",
    "YUK")
}


#' Check input for ReadCHMD
#' @param x A list with the input values for ReadCHMD
#' @return No return value, called for input validation 
#' @keywords internal
#' 
check_input_ReadCHMD <- function(x) {
  wht <- c("births", "population", "Dx_lexis", "Dx",
           "mx", "Ex", "LT_f", "LT_m", "LT_t", "e0")

  if (!(x$interval %in% data_format())) {
        stop("The interval ", x$interval, " does not exist in CHMD. ",
         "Try one of these options:\n", paste(data_format(), collapse = ", "),
         call. = FALSE)
  }

  if (!(x$what %in% wht)) {
    stop(x$what, " does not exist in CHMD. Try one of these options:\n",
         paste(wht, collapse = ", "), call. = FALSE)
  }

  if (all(!(x$regions %in% CANregions()))) {
    stop("Something is wrong in the region codes supplied.\n",
         "Try one or more of these options:\n",
         paste(CANregions(), collapse = ", "), call. = FALSE)
  }

  # Availability of Death and Exposures
  if ((x$what %in% c("Dx", "Ex")) & !(x$interval %in% c("1x1", "5x1"))) {
    stop("Data type ", x$what,
         " is available only in the following format: '1x1' and '5x1'.",
         call. = FALSE)
  }

  if (any(x$region %in% c("NWT", "YUK")) &
    (x$what %in% c("LT_m", "LT_f", "LT_t")) &
      (x$interval %in% c("1x1", "5x1"))) {
    stop("For the regions of Northwest Territories & Nunavut (NWT) and Yukon (YUK),",
         "\ndata type ", x$what, " is NOT available in the following format:",
         "'1x1' and '5x1'.",
         "\nTo download the life-tables for all the other regions use the argument:",
         "\nregions = c('CAN', 'NFL', 'PEI', 'NSC', 'NBR', 'QUE', 'ONT', 'MAN', 'SAS', 'ALB', 'BCO')",
         call. = FALSE)
  }
}



#' Print ReadCHMD
#' @param x An object of class \code{"ReadCHMD"}
#' @param ... Further arguments passed to or from other methods.
#' @return Print data on the console
#' @keywords internal
#' @export
print.ReadCHMD <- function(x, ...){
  what <- x$input$what
  cat("Canadian Human Mortality Database\n")
  cat("Web Address   : https://www.bdlc.umontreal.ca/chmd\n")
  cat("Download Date :", x$download.date, "\n")
  cat("Type of data  :", what, "\n")
  cat(paste("Interval      :", x$input$interval, "\n"))
  cat(paste("Years         :", x$years[1], "--", rev(x$years)[1], "\n"))
  cat(paste("Ages          :", ageMsg(what, x), "\n"))
  cat("Regions       :", x$input$regions, "\n")
  cat("\nData:\n")
  print(head_tail(x$data, hlength = 5, tlength = 5))
}

# -------------------------------------------------------------- #
# Author: Marius D. PASCARIU
# Last Update: Mon Jan 15 16:53:07 2024
# -------------------------------------------------------------- #
remove(list = ls())

# Wrong index
expect_error(ReadCHMD(what = "DxDD"))

# Wrong country
expect_error(ReadCHMD(what = "Dx",
                      regions = "CANN"))

# Wrong interval
expect_error(ReadCHMD(what = "Dx",
                      regions = "CAN",
                      interval = "1x50"))

# Wrong region for the index
expect_error(ReadCHMD(what = "LT_fc",
                      regions = "SAS",
                      interval = "1x1"))

# Wrong interval for the index
expect_message(
  ReadCHMD(what = "e0",
           regions = "CAN",
           interval = "5x1",
           show = F))

expect_error(ReadCHMD(what = "LT_f",
                      regions = "YUK",
                      interval = "5x1",
                      show = F))

expect_output(
  print(CHMD_sample)
)

# The tests below have been removed because in case the internet source is
# temporary not working the CRAN will consider it as a software failure and will
# demand correction. Unfortunately, we can test only the error messages i.e. the
# automated checks put in place.

# # Test the show arg and print function
# expect_silent(D <- ReadCHMD(what = "LT_f",
#                             regions = "CAN",
#                             interval = "5x10",
#                             show = F))
# expect_output(print(D))

# --------------------------------------------
# Author: Marius D PASCARIU
# Date: 2026-05-04 23:38:13
# --------------------------------------------

#' Check Data Availability in HMD
#'
#' Returns information about the data available in the Human Mortality 
#' Database (HMD), including the range of years covered by the life tables 
#' for each country or region.
#' @param link URL to the HMD available data.
#' Default: "https://www.mortality.org/Data/DataAvailability"
#' @return A tibble.
#' @seealso \code{\link{ReadHMD}}
#' @author Marius D. Pascariu
#' @examples
#' availableHMD()
#' 
#' @export
availableHMD <- function(link = "https://www.mortality.org/Data/DataAvailability") {
  out <- NULL
  error_message <- "Website connection failed. Please check your internet connection or the URL."
  
  # Check website connectivity
  response <- make_http_request(link)
  
  # Check if the website is accessible
  if (!is.null(response)) {
    
    if (http_status(response)$message == "Success: (200) OK") {
      # Read the HTML content of the webpage
      webpage <- read_html(response)
      
      # Extract the table from the webpage
      table_data <- html_table(webpage, fill = TRUE)
      
      # from the list of tables extracted above the table of interest is the first one:
      if (length(table_data) > 0) {
        out <- table_data[[1]]
        
      } else {
        message("No tables found on the webpage.")
      }
      
    } else {
      message(error_message)
    }
    
  } else { 
    message(error_message)
    }
  
  return(out)
}



#' Make HTTP request
#' @param url URL
#' @return url response
#' @keywords internal
#' 
make_http_request <- function(url) {
  response <- NULL
  
  tryCatch({
    response <- GET(url)
    
  }, error = function(e) {
    # Error: Display the error message
    paste("Error:", conditionMessage(e))
  })
  
  return(response)
}

# --------------------------------------------
# Author: Marius D PASCARIU
# Date: 2026-05-04 23:30:18
# --------------------------------------------

#' Check Available Loss Functions
#'
#' Returns information about the loss functions implemented for use with the
#' optimisation procedure in the \code{\link{MortalityLaw}} function.
#' @return A list of class \code{availableLF} with the components:
#'  \item{table}{Table with loss functions and codes to be used in \code{\link{MortalityLaw}}.}
#'  \item{legend}{Table with details about the abbreviation used.}
#' @seealso \code{\link{MortalityLaw}}
#' @author Marius D. Pascariu
#' @examples
#' availableLF()
#' @export
availableLF <- function(){
  tab <- as.data.frame(
    matrix(c("L = -[Dx * log(mu) - mu*Ex]", "poissonL",
             "L = -[Dx * log(1 - exp(-mu)) - (Ex - Dx)*mu]  ", "binomialL",
             "L =  [1 - mu/ov]^2", "LF1",
             "L =  log[mu/ov]^2", "LF2",
             "L =  [(ov - mu)^2]/ov", "LF3",
             "L =  [ov - mu]^2", "LF4",
             "L =  [ov - mu] * log[ov/mu]", "LF5",
             "L =  abs(ov - mu)", "LF6"), ncol = 2, byrow = T))
  colnames(tab) <- c("LOSS FUNCTION", "CODE")

  legend <- c("Dx: Death counts",
              "Ex: Population exposed to risk",
              "mu: Estimated value",
              "ov: Observed value")

  out <- structure(class = "availableLF", list(table = tab, legend = legend))
  return(out)
}


#' Print availableLF
#' @param x An object of class \code{"availableLF"}
#' @param ... Further arguments passed to or from other methods.
#' @return print info on the console
#' @keywords internal
#' @export
print.availableLF <- function(x, ...) {
  cat("\nLoss functions available in the package:\n\n")
  print(x$table, right = FALSE, row.names = FALSE)
  cat("\nLEGEND:\n")
  cat(x$legend, sep = '\n')

  message("\nHINT: Most loss functions work well with 'poissonL'. However, for complex ",
          "mortality laws like Heligman-Pollard (HP), a better fit can be obtained using ",
          "other loss functions (e.g. 'LF2'). You are strongly encouraged to test ",
          "different options before deciding on the final version. The results might be ",
          "slightly different.\n")
}

---
title: "Introduction to `MortalityLaws`"
author: "Marius D. Pascariu"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Intro}
  %\VignetteEncoding{UTF-8}
bibliography: Mlaw_Refrences.bib
biblio-style: "apalike"
link-citations: true
---

```{r setup, include=FALSE}
library(MortalityLaws)
library(knitr)
opts_chunk$set(collapse = TRUE)
```

# Package structure

**MortalityLaws** is an R package that provides tools for fitting and analyzing a wide range of parametric mortality models. It exploits various optimisation methods to handle complex models efficiently. The package can construct full and abridged life tables from different inputs and download data directly from the Human Mortality Database (HMD).

The core functions include:

- `MortalityLaw` – fit parametric mortality models.
- `LifeTable` – build life tables.
- `LawTable` – generate life tables from a fitted mortality law.
- `convertFx` – convert between mortality functions (e.g., `mx` to `qx`).
- `ReadHMD` – download HMD data.

Support functions such as `availableLaws`, `availableLF`, and `availableHMD` return information about the implemented mortality laws, loss functions, and HMD data availability, respectively. Standard generics (`predict`, `plot`, `summary`, `fitted`, `residuals`) work on `MortalityLaw` and `LifeTable` objects.

A small built-in dataset (`ahmd`) is provided for testing. All functions are documented in the usual R way; after loading the package with `library(MortalityLaws)`, you can type `?MortalityLaw` to see its help file.

# Downloading HMD data

The `ReadHMD` function downloads data from the Human Mortality Database [@university_of_california_berkeley_human_2016]. To use it, you need a valid HMD account.

```{r LoadPackages, message=FALSE}
library(MortalityLaws)
```

```{r ReadHMD, eval=FALSE}
# Download Swedish death counts (ages 0–110, years 1751–2014)
HMD_Dx <- ReadHMD(
  what      = "Dx",
  countries = "SWE",
  interval  = "1x1",
  username  = "user@email.com",
  password  = "password",
  save      = FALSE
)
```

You can similarly download:

- `births` – birth records  
- `Ex` – exposure-to-risk  
- `lexis` – deaths by Lexis triangles  
- `population` – population size  
- `mx` – death rates  
- `LT_f`, `LT_m`, `LT_t` – life tables by sex  
- `e0` – life expectancy at birth  
- `mxc` – cohort death rates  
- `Exc` – cohort exposures  

for over 40 countries and regions in various formats (`1x1`, `1x5`, `5x1`, `5x5`).

# Model fitting and diagnosis

Once you have data, fitting a model is straightforward. Below we fit a Heligman–Pollard [@heligman1980] model to Swedish mortality in 1950 using a Poisson likelihood (loss function `"LF2"`).

```{r}
year     <- 1950
ages     <- 0:100
deaths   <- ahmd$Dx[paste(ages), paste(year)]
exposure <- ahmd$Ex[paste(ages), paste(year)]

fit <- MortalityLaw(
  x          = ages,
  Dx         = deaths,
  Ex         = exposure,
  law        = "HP",
  opt.method = "LF2"
)
```

## Examining the fitted model

```{r}
ls(fit)   # components of the fitted object
```

A summary of the fit gives parameter estimates and goodness‑of‑fit statistics:

```{r}
summary(fit)
```

Visual inspection is also easy:

```{r, fig.align='center', out.width='80%', fig.width=9}
plot(fit)
```

The default plot shows observed and fitted age‑specific death rates (or probabilities), the residuals, and the fitted parameters.

## Fitting on a subset of ages

Sometimes you may want to fit the model only to a certain age range while still plotting over the full range. Use `fit.this.x` to specify the ages used during optimisation:

```{r, fig.align='center', out.width='80%', fig.width=9}
fit.subset <- MortalityLaw(
  x          = ages,
  Dx         = deaths,
  Ex         = exposure,
  law        = "HP",
  opt.method = "LF2",
  fit.this.x = 0:65
)
plot(fit.subset)
```

The grey area indicates the ages that were actually used in the fit.

## Age scaling for numerical stability

When fitting models with exponential or power terms (e.g., Gompertz, Makeham), large age values can cause numerical overflow or slow convergence. **Centering and/or scaling the age variable** helps stabilise the optimisation.

The `MortalityLaw` function provides the argument `scale.age`. When set to `TRUE`, ages are centered by subtracting the mean age in the fitting range and scaled by dividing by the standard deviation. This transformation often improves convergence speed and parameter identifiability without affecting the shape of the mortality curve.

```{r, eval=FALSE}
# Fit with automatic age scaling
fit_scaled <- MortalityLaw(
  x          = ages,
  Dx         = deaths,
  Ex         = exposure,
  law        = "gompertz",
  scale.age  = TRUE
)
```

**Explanation:**  
- Centering removes a large constant offset, reducing correlations between the intercept and slope parameters.  
- Scaling brings all ages to a comparable magnitude, which is especially beneficial for algorithms like `nlm` or `optim` that assume unit‑scale variables.  
- Internally, the model is fitted on the transformed ages, and the resulting parameters are back‑transformed so that the reported values correspond to the original age scale.

You can also manually scale ages before passing them to `MortalityLaw` (e.g., `(x - mean(x))/sd(x)`), but using the built‑in `scale.age` argument is simpler and ensures correct back‑transformation.

# Mortality laws in the package

The following table lists all parametric mortality laws currently implemented. The **Code** column shows the string used in the `law` argument of `MortalityLaw`.

| Mortality Law | Predictor | Code |
|---------------|-----------|------|
| Gompertz | $\mu(x) = A e^{Bx}$ | `gompertz` |
| Gompertz (reparam.) | $\mu(x) = \frac{1}{\sigma} \exp\left\{\frac{x-M}{\sigma}\right\}$ | `gompertz0` |
| Inverse‑Gompertz | $\mu(x) = \frac{\frac{1}{\sigma}\exp\left\{\frac{x-M}{\sigma}\right\}}{\exp\left\{e^{-(x-M)/\sigma}\right\}-1}$ | `invgompertz` |
| Makeham | $\mu(x) = A e^{Bx} + C$ | `makeham` |
| Makeham (reparam.) | $\mu(x) = \frac{1}{\sigma} \exp\left\{\frac{x-M}{\sigma}\right\} + C$ | `makeham0` |
| Opperman | $\mu(x) = \frac{A}{\sqrt{x}} + B + C \sqrt[3]{x}$ | `opperman` |
| Thiele | $\mu(x) = A_1 e^{-B_1 x} + A_2 e^{-\frac12 B_2 (x-C)^2} + A_3 e^{B_3 x}$ | `thiele` |
| Wittstein | $q(x) = \frac{1}{B} A^{-(B x)^N} + A^{-(M - x)^N}$ | `wittstein` |
| Perks | $\mu(x) = \frac{A + B C^x}{B C^{-x} + 1 + D C^x}$ | `perks` |
| Weibull | $\mu(x) = \frac{1}{\sigma} \left( \frac{x}{M} \right)^{\frac{M}{\sigma}-1}$ | `weibull` |
| Inverse‑Weibull | $\mu(x) = \frac{\frac{1}{\sigma} \left( \frac{x}{M} \right)^{-\frac{M}{\sigma}-1}}{\exp\left\{\left( \frac{x}{M} \right)^{-\frac{M}{\sigma}}\right\}-1}$ | `invweibull` |
| Van der Maen | $\mu(x) = A + Bx + Cx^2 + \frac{I}{N-x}$ | `vandermaen` |
| Van der Maen (simpl.) | $\mu(x) = A + Bx + \frac{I}{N-x}$ | `vandermaen2` |
| Quadratic | $\mu(x) = A + Bx + Cx^2$ | `quadratic` |
| Beard | $\mu(x) = \frac{A e^{Bx}}{1 + K A e^{Bx}}$ | `beard` |
| Makeham‑Beard | $\mu(x) = \frac{A e^{Bx}}{1 + K A e^{Bx}} + C$ | `makehambeard` |
| Gamma‑Gompertz | $\mu(x) = \frac{A e^{Bx}}{1 + \frac{AG}{B}(e^{Bx}-1)}$ | `ggompertz` |
| Siler | $\mu(x) = A_1 e^{-B_1 x} + A_2 + A_3 e^{B_3 x}$ | `siler` |
| Heligman–Pollard | $\frac{q(x)}{p(x)} = A^{(x+B)^C} + D e^{-E (\ln x - \ln F)^2} + G H^x$ | `HP` |
| Heligman–Pollard (HP2) | $q(x) = A^{(x+B)^C} + D e^{-E (\ln x - \ln F)^2} + \frac{G H^x}{1+G H^x}$ | `HP2` |
| Heligman–Pollard (HP3) | $q(x) = A^{(x+B)^C} + D e^{-E (\ln x - \ln F)^2} + \frac{G H^x}{1+K G H^x}$ | `HP3` |
| Rogers–Planck | $q(x) = A_0 + A_1 e^{-Ax} + A_2 e^{B(x-U) - e^{-C(x-U)}} + A_3 e^{D x}$ | `rogersplanck` |
| Martinelle | $\mu(x) = \frac{A e^{Bx} + C}{1 + D e^{Bx}} + K e^{Bx}$ | `martinelle` |
| Carriere 1 | $S(x) = \psi_1 S_1(x) + \psi_2 S_2(x) + \psi_3 S_3(x)$ | `carriere1` |
| Carriere 2 | $S(x) = \psi_1 S_1(x) + \psi_4 S_4(x) + \psi_3 S_3(x)$ | `carriere2` |
| Kostaki | $\frac{q(x)}{p(x)} = A^{(x+B)^C} + D e^{-E_i (\ln x - \ln F)^2} + G H^x$ | `kostaki` |
| Kannisto | $\mu(x) = \frac{A e^{Bx}}{1 + A e^{Bx}} + C$ | `kannisto` |

Use `availableLaws()` in R to see this list dynamically:

```{r, eval=FALSE, warning=FALSE}
availableLaws()
```

# Loss functions in the package

A parametric mortality model is fitted by optimising a loss function (e.g., a negative log‑likelihood or a sum of squared errors). MortalityLaws provides eight loss functions that can emphasise different portions of the mortality curve. Use `availableLF()` to view their descriptions:

```{r, message=FALSE, warning=FALSE}
availableLF()
```

# Custom mortality laws

You are not limited to the built‑in laws – you can define your own hazard function and pass it via `custom.law`. As an example, we fit a reparametrised Gompertz model in terms of the modal age at death $M$ [@missov2015]:

\[
\mu_x = \beta e^{\beta (x - M)}.
\]

First, define the hazard function. The function must return a list containing at least the hazard vector `hx`.

```{r}
my_gompertz <- function(x, par = c(b = 0.13, M = 45)){
  hx <- with(as.list(par), b * exp(b * (x - M)))
  return(as.list(environment()))   # must return a list
}
```

Select data and fit:

```{r}
year     <- 1950
ages     <- 45:85
deaths   <- ahmd$Dx[paste(ages), paste(year)]
exposure <- ahmd$Ex[paste(ages), paste(year)]
```

```{r, warning=FALSE, results='hide'}
my_model <- MortalityLaw(
  x          = ages,
  Dx         = deaths,
  Ex         = exposure,
  custom.law = my_gompertz
)
```

```{r}
summary(my_model)
```

```{r}
plot(my_model)
```

# Full life tables

The `LifeTable` function can build complete or abridged life tables from several types of input:

- Death counts and exposures (`Dx`, `Ex`)
- Age‑specific death rates (`mx`)
- Death probabilities (`qx`)
- Survivorship (`lx`)
- Death distribution (`dx`)

Only one of these inputs needs to be specified – the rest are ignored if present.

```{r}
y  <- 1900
x  <- as.numeric(rownames(ahmd$mx))
Dx <- ahmd$Dx[, paste(y)]
Ex <- ahmd$Ex[, paste(y)]

LT1 <- LifeTable(x, Dx = Dx, Ex = Ex)   # primary input
LT2 <- LifeTable(x, mx = LT1$lt$mx)     # from mx
LT3 <- LifeTable(x, qx = LT1$lt$qx)     # from qx
LT4 <- LifeTable(x, lx = LT1$lt$lx)     # from lx
LT5 <- LifeTable(x, dx = LT1$lt$dx)     # from dx

LT1
```

```{r}
ls(LT1)   # components of the life table object
```

# Abridged life tables

Abridged life tables use wider age intervals (e.g., 0, 1–4, 5–9, ...). The same `LifeTable` function works for any abridged age structure.

```{r}
x  <- c(0, 1, seq(5, 110, by = 5))
mx <- c(.053, .005, .001, .0012, .0018, .002, .003, .004, 
       .004, .005, .006, .0093, .0129, .019, .031, .049, 
       .084, .129, .180, .2354, .3085, .390, .478, .551)

lt <- LifeTable(x, mx = mx, sex = "female")
```

```{r}
lt
```

# Citation

```r
citation(package = "MortalityLaws")
```

# References

---
title: "`MortalityLaws` Installation"
author: "Marius D. Pascariu"
date: "January 10, 2018"
output: 
  pdf_document:
    highlight: tango
    number_sections: true
    toc: false
fontsize: 11pt
geometry: margin=1.1in
bibliography: Mlaw_Refrences.bib
biblio-style: "apalike"
link-citations: true
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Installation}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

The package can be installed from two different sources, `CRAN` or `GitHub`. Most people would choose the first option (see below).

# Install stable version from `CRAN`

The `MortalityLaws` package can be installed from [`CRAN`](https://cran.r-project.org/web/packages/MortalityLaws/index.html) as follows:

1. Make sure you have the most recent version of R
2. Run the following code in your R console 

```{r, eval=FALSE}
    install.packages("MortalityLaws")
```

# Updating to the latest version of the package
You can track and contribute to the development of `MortalityLaws` at https://github.com/mpascariu/MortalityLaws. To install it:

1. Install the release version of `devtools` from CRAN with
```{r, eval=FALSE}
    install.packages("devtools")
```

2. Make sure you have a working development environment.
    * **Windows**: Install [Rtools](https://cran.r-project.org/bin/windows/Rtools/).
    * **Mac**: Install `Xcode` from the Mac App Store.
    * **Linux**: Install a compiler and various development libraries (details vary across different flavors of Linux).

3. Install the development version of devtools.

```{r, eval=FALSE}
    devtools::install_github("mpascariu/MortalityLaws")
```


\vspace{2cm}

Now you are ready to go. Happy modelling!

Check the other vignettes and examples provided in the help pages.

---
title: "Installing `MortalityLaws`"
author: "Marius D. Pascariu"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Installing-MortalityLaws}
  %\VignetteEncoding{UTF-8}
bibliography: Mlaw_Refrences.bib
biblio-style: "apalike"
link-citations: true
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

The `MortalityLaws` package can be installed from either **CRAN** (the official R package repository) or **GitHub** (the development version). Most users will prefer the first option, as it provides a stable, tested release. The sections below explain both approaches.

# Install the stable version from CRAN

The easiest way to get `MortalityLaws` is to install it from CRAN. This gives you the most recent stable release that has passed R's quality checks.

1. **Make sure you have a recent version of R** (≥ 3.5 is recommended). You can check your R version at the console:
   ```r
   R.version.string
   ```

2. **Install the package** by running the following command in your R console:
   ```{r, eval=FALSE}
   install.packages("MortalityLaws")
   ```

   This will automatically download and install the package along with its required dependencies.

3. **Load the package** and verify that it works:
   ```{r, eval=FALSE}
   library(MortalityLaws)
   help(package = "MortalityLaws")
   ```

# Install the development version from GitHub

If you want to try the latest features, bug fixes, or contribute to the development of `MortalityLaws`, you can install the development version directly from [GitHub](https://github.com/mpascariu/MortalityLaws). This version may contain changes that have not yet been submitted to CRAN.

### Step 1: Install `devtools`

The `devtools` package provides the `install_github()` function needed to fetch and install packages from GitHub. Install it from CRAN:

```{r, eval=FALSE}
install.packages("devtools")
```

**What `devtools` does:** It automates the process of downloading source code from GitHub, resolving dependencies, compiling (if needed), and installing the package into your R library.

### Step 2: Set up a development environment

To install packages from source (which is what GitHub packages are), your system needs a compiler and associated tools:

- **Windows:** Install [Rtools](https://cran.r-project.org/bin/windows/Rtools/). Choose the version that matches your R version. Rtools provides the `gcc` compiler and `make` utility required to build R packages from source.
  
- **macOS:** Install **Xcode** from the Mac App Store, or at minimum the Command Line Tools for Xcode. You can install the command line tools alone by running `xcode-select --install` in the Terminal.

- **Linux (Ubuntu/Debian):** Install a compiler and development libraries. A typical command is:
  ```bash
  sudo apt-get install build-essential libcurl4-openssl-dev libssl-dev libxml2-dev
  ```
  The exact packages may vary by distribution. The key requirement is that `gcc` (or `clang`) and `make` are available.

> **Why is this needed?** Unlike CRAN binaries (pre-compiled), GitHub packages are distributed as source code. R must compile them locally, which requires a development toolchain.

### Step 3: Install `MortalityLaws` from GitHub

Once `devtools` is installed and your development environment is ready, run:

```{r, eval=FALSE}
devtools::install_github("mpascariu/MortalityLaws")
```

This command:
1. Clones the repository from `https://github.com/mpascariu/MortalityLaws`.
2. Builds the package from source.
3. Installs it into your R library.

### Step 4: Verify the installation

After installation, load the package and check that everything works:

```{r, eval=FALSE}
library(MortalityLaws)
availableLaws()   # list all implemented mortality laws
```

---

# Updating an existing installation

- **CRAN version:** Simply re-run `install.packages("MortalityLaws")` periodically to get the latest stable release.
- **GitHub version:** Run `devtools::install_github("mpascariu/MortalityLaws")` again to pull and install the most recent commits.

---

You are now ready to use `MortalityLaws`! Happy modelling.

For further guidance, check the other vignettes (e.g., the introductory vignette) and the help pages:

```{r, eval=FALSE}
browseVignettes("MortalityLaws")   # view available vignettes
?MortalityLaw                       # help for the main fitting function
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_models.R
\name{perks}
\alias{perks}
\title{Perks Model - 1932}
\usage{
perks(x, par = NULL)
}
\arguments{
\item{x}{vector of age at the beginning of the age classes}

\item{par}{parameters of the selected model. If NULL the
default values will be assigned automatically.}
}
\value{
A list of rates and model parameters
}
\description{
Perks Model - 1932
}
\examples{
perks(x = 50:100)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_S3.R
\name{deviance.MortalityLaw}
\alias{deviance.MortalityLaw}
\title{deviance function for MortalityLaw}
\usage{
\method{deviance}{MortalityLaw}(object, ...)
}
\arguments{
\item{...}{further arguments passed to or from other methods.}
}
\value{
model deviance value
}
\description{
deviance function for MortalityLaw
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LifeTable.R
\name{dx_lx}
\alias{dx_lx}
\title{dx to lx}
\usage{
dx_lx(ux, out = c("dx", "lx"))
}
\arguments{
\item{ux}{A vector of dx or lx data.}

\item{out}{Type of the output: dx or lx.}
}
\value{
A vector containing dx or lx values
}
\description{
Function to convert dx into lx and back
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAHMD.R
\name{ReadAHMD}
\alias{ReadAHMD}
\title{Download the Australian Human Mortality Database (AHMD)}
\usage{
ReadAHMD(what, regions = NULL, interval = "1x1", save = FALSE, show = TRUE)
}
\arguments{
\item{what}{What type of data are you looking for? The following options
might be available for some or all the countries and regions: \itemize{
  \item{\code{"births"}} -- birth records;
  \item{\code{"Dx_lexis"}} -- deaths by Lexis triangles;
  \item{\code{"Ex_lexis"}} -- exposure-to-risk by Lexis triangles;
  \item{\code{"population"}} -- population size;
  \item{\code{"Dx"}} -- death counts;
  \item{\code{"Ex"}} -- exposure-to-risk;
  \item{\code{"mx"}} -- central death-rates;
  \item{\code{"LT_f"}} -- period life tables for females;
  \item{\code{"LT_m"}} -- period life tables for males;
  \item{\code{"LT_t"}} -- period life tables both sexes combined;
  \item{\code{"e0"}} -- period life expectancy at birth;
  \item{\code{"Exc"}} -- cohort exposures;
  \item{\code{"mxc"}} -- cohort death-rates;
  \item{\code{"LT_fc"}} -- cohort life tables for females;
  \item{\code{"LT_mc"}} -- cohort life tables for males;
  \item{\code{"LT_tc"}} -- cohort life tables both sexes combined;
  \item{\code{"e0c"}} -- cohort life expectancy at birth;
  }}

\item{regions}{Specify the region specific data you want to download by
adding the AHMD region code/s. Options:
\itemize{
  \item{\code{"ACT"}} -- Australian Capital Territory;
  \item{\code{"NSW"}} -- New South Wales;
  \item{\code{"NT"}} -- Northern Territory;
  \item{\code{"QLD"}} -- Queensland;
  \item{\code{"SA"}} -- South Australia;
  \item{\code{"TAS"}} -- Tasmania;
  \item{\code{"VIC"}} -- Victoria;
  \item{\code{"WA"}} -- Western Australia;
  \item{\code{NULL}} -- if \code{NULL} data for all the regions are downloaded.
  }}

\item{interval}{Datasets are given in various age and time formats based on
which the records are aggregated. Interval options:
\itemize{
  \item{\code{"1x1"}} -- by age and year;
  \item{\code{"1x5"}} -- by age and 5-year time interval;
  \item{\code{"1x10"}} -- by age and 10-year time interval;
  \item{\code{"5x1"}} -- by 5-year age group and year;
  \item{\code{"5x5"}} -- by 5-year age group and 5-year time interval;
  \item{\code{"5x10"}} --by 5-year age group and 10-year time interval.
  }}

\item{save}{Do you want to save a copy of the dataset on your local machine?
Logical. Default: \code{FALSE}.}

\item{show}{Choose whether to display a progress bar. Logical.
Default: \code{TRUE}.}
}
\value{
A \code{ReadAHMD} object that contains:
 \item{input}{List with the input values;}
 \item{data}{Data downloaded from AHMD;}
 \item{download.date}{Time stamp;}
 \item{years}{Numerical vector with the years covered in the data;}
 \item{ages}{Numerical vector with ages covered in the data.}
}
\description{
Download detailed mortality and population data for different
provinces and territories in Australia, in a single object from the
\href{https://aushd.org/}{Australian Human Mortality Database}.
}
\details{
(Description taken from the AHMD website).

The Australian Human Mortality Database (AHMD) was created to provide
detailed Australian mortality and population data to researchers, students,
journalists, policy analysts, and others interested in the history of
human longevity. The project is an achievement of the Mortality,
Ageing & Health research team in the ANU School of Demography under the
supervision of Associate Professor Vladimir Canudas-Romo, in collaboration
with demographers at the Max Plank Institute for Demographic Research
(Rostock, Germany) and the Department of Demography, University of
California at Berkeley.

The AHMD is a "satellite" of the Human Mortality Database (HMD),
an international database which currently holds detailed data for multiple
countries or regions. Consequently, the AHMD's underlying methodology
corresponds to the one used for the HMD.

The AHMD gathers all required data (deaths counts, births counts,
population size, exposure-to-risk, death rates) to compute life tables
for Australia, its states and its territories. One of the great advantages
of the database is to include data that is validated and corrected, when
required, and rendered comparable, if possible, for the period ranging
from 1971 thru 2016. For comparison purposes, various life tables published
by governmental organizations are also available for download in PDF format.
}
\examples{
\donttest{
# Download demographic data for Australian Capital Territory and
# Tasmania regions in 5x1 format

# Death counts. We don't want to export data outside R.
AHMD_Dx <- ReadAHMD(what = "Dx",
                    regions = c('ACT', 'TAS'),
                    interval  = "5x1",
                    save = FALSE)
AHMD_Dx

# Download life tables for female population in all the states and export data.
LTF <- ReadAHMD(what = "LT_f", interval  = "5x1", save = FALSE)
LTF
}
}
\seealso{
\code{\link{ReadHMD}}
\code{\link{ReadCHMD}}
}
\author{
Marius D. Pascariu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaws-package.R
\docType{package}
\name{MortalityLaws}
\alias{MortalityLaws-package}
\alias{MortalityLaws}
\title{MortalityLaws: Parametric Mortality Models, Life Tables and HMD}
\description{
Fit the most popular human mortality 'laws', and construct full and abridge life tables given various input indices. A mortality law is a parametric function that describes the dying-out process of individuals in a population during a significant portion of their life spans. For a comprehensive review of the most important mortality laws see Tabeau (2001) \doi{10.1007/0-306-47562-6_1}. Practical functions for downloading data from various human mortality databases are provided as well.
}
\details{
To learn more about the package, start with the vignettes:
\code{browseVignettes(package = "MortalityLaws")}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/mpascariu/MortalityLaws}
  \item Report bugs at \url{https://github.com/mpascariu/MortalityLaws/issues}
}

}
\author{
\strong{Maintainer}: Marius D. Pascariu \email{mpascariu@outlook.com} (\href{https://orcid.org/0000-0002-2568-6489}{ORCID}) [copyright holder]

Other contributors:
\itemize{
  \item Vladimir Canudas-Romo [contributor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_S3.R
\name{print.MortalityLaw}
\alias{print.MortalityLaw}
\title{Print MortalityLaw}
\usage{
\method{print}{MortalityLaw}(x, ...)
}
\arguments{
\item{x}{an object of class \code{"MortalityLaw"}}

\item{...}{further arguments passed to or from other methods.}
}
\value{
Print data on console
}
\description{
Print MortalityLaw
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readJMD.R
\name{ReadJMD}
\alias{ReadJMD}
\title{Download the Japanese Mortality Database (JMD)}
\usage{
ReadJMD(what, regions = NULL, interval = "1x1", save = FALSE, show = TRUE)
}
\arguments{
\item{what}{What type of data are you looking for? The following options
might be available for some or all the countries and regions: \itemize{
  \item{\code{"births"}} -- birth records;
  \item{\code{"Dx_lexis"}} -- deaths by Lexis triangles;
  \item{\code{"Ex_lexis"}} -- exposure-to-risk by Lexis triangles;
  \item{\code{"population"}} -- population size;
  \item{\code{"Dx"}} -- death counts;
  \item{\code{"Ex"}} -- exposure-to-risk;
  \item{\code{"mx"}} -- central death-rates;
  \item{\code{"LT_f"}} -- period life tables for females;
  \item{\code{"LT_m"}} -- period life tables for males;
  \item{\code{"LT_t"}} -- period life tables both sexes combined;
  \item{\code{"e0"}} -- period life expectancy at birth;
  \item{\code{"Exc"}} -- cohort exposures;
  \item{\code{"mxc"}} -- cohort death-rates;
  \item{\code{"LT_fc"}} -- cohort life tables for females;
  \item{\code{"LT_mc"}} -- cohort life tables for males;
  \item{\code{"LT_tc"}} -- cohort life tables both sexes combined;
  \item{\code{"e0c"}} -- cohort life expectancy at birth;
  }}

\item{regions}{Specify the region specific data you want to download by
adding the JMD region code/s. Options: \code{"Japan", "Hokkaido", "Aomori",
"Iwate", "Miyagi","Akita", "Yamagata", "Fukushima", "Ibaraki", "Tochigi",
"Gunma", "Saitama", "Chiba", "Tokyo", "Kanagawa", "Niigata", "Toyama",
"Ishikawa", "Fukui", "Yamanashi", "Nagano", "Gifu", "Shizuoka","Aichi",
"Mie", "Shiga", "Kyoto", "Osaka", "Hyogo", "Nara", "Wakayama", "Tottori",
"Shimane", "Okayama", "Hiroshima", "Yamaguchi", "Tokushima", "Kagawa",
"Ehime", "Kochi", "Fukuoka", "Saga", "Nagasaki", "Kumamoto", "Oita",
"Miyazaki", "Kagoshima", "Okinawa"}.
If \code{NULL} data for all the regions are downloaded at once.}

\item{interval}{Datasets are given in various age and time formats based on
which the records are aggregated. Interval options:
\itemize{
  \item{\code{"1x1"}} -- by age and year;
  \item{\code{"1x5"}} -- by age and 5-year time interval;
  \item{\code{"1x10"}} -- by age and 10-year time interval;
  \item{\code{"5x1"}} -- by 5-year age group and year;
  \item{\code{"5x5"}} -- by 5-year age group and 5-year time interval;
  \item{\code{"5x10"}} --by 5-year age group and 10-year time interval.
  }}

\item{save}{Do you want to save a copy of the dataset on your local machine?
Logical. Default: \code{FALSE}.}

\item{show}{Choose whether to display a progress bar. Logical.
Default: \code{TRUE}.}
}
\value{
A \code{ReadJMD} object that contains:
 \item{input}{List with the input values;}
 \item{data}{Data downloaded from JMD;}
 \item{download.date}{Time stamp;}
 \item{years}{Numerical vector with the years covered in the data;}
 \item{ages}{Numerical vector with ages covered in the data.}
}
\description{
Download detailed mortality and population data of the 47 prefectures in
Japan, in a single object. The source of data is the
\href{https://www.ipss.go.jp/p-toukei/JMD/index-en.asp}{
Japanese Mortality Database}.
}
\details{
(Description taken from the JMD website).

The Japanese Mortality Database is a comprehensively-reorganized mortality
database that is optimized for mortality research and consistent with the
Human Mortality Database. This database is provided as a part of the research
project "Demographic research on the causes and the socio-economic
consequence of longetivity extension in Japan" (2011-2013), "Demographic
research on longevity extension, population aging, and their effects on the
social security and socio-economic structures in Japan" (2014-2016), and
"Comprehensive research from a demographic viewpoint on the longevity
revolution" (2017-2019) at the National Institute of Population and Social
Security Research.

The Japanese Mortality Database is designed to provide the life tables to all
the people who are interested in Japanese mortality including domestic and
foreign mortality researchers for the purpose of mortality research.
Especially because we have structured it to conform with the HMD, our
database is suitable for international comparison, we put emphasis on the
compatibility with the HMD more than our country's particular
characteristics. Therefore, the life tables by JMD do not necessarily
exhibit the same values as ones by the official life tables prepared and
released by the Statistics and Information Department, Minister's
Secretariat, Ministry of Health, Labor and Welfare according to the different
base population or the methods for estimating the tables. When doing things
other than mortality research, if life table that statistically displays our
country's mortality situation is necessary, please use the official life
table that has been prepared by the Statistics and Information Department,
Minister's Secretariat, Ministry of Health, Labor and Welfare.

At the present time, we offer the data for All Japan and by prefecture.
The project team is studying the methodology for estimating life tables
along with data preparation. Therefore, the data may be updated when a
new methodology is adopted. Please refer to "Methods" for further
information.
}
\examples{
\donttest{
# Download demographic data for Fukushima and Tokyo regions in 1x1 format

# Death counts. We don't want to export data outside R.
JMD_Dx <- ReadJMD(what = "Dx",
                  regions = c('Fukushima', 'Tokyo'),
                  interval  = "1x1",
                  save = FALSE)
JMD_Dx

# Download life tables for female population in all the states and export data.
LTF <- ReadJMD(what = "LT_f", interval  = "5x5", save = FALSE)
LTF
}
}
\seealso{
\code{\link{ReadHMD}}
\code{\link{ReadCHMD}}
}
\author{
Marius D. Pascariu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAHMD.R
\name{check_input_ReadAHMD}
\alias{check_input_ReadAHMD}
\title{Check input ReadAHMD}
\usage{
check_input_ReadAHMD(x)
}
\arguments{
\item{x}{a list containing the input arguments from ReadAHMD function}
}
\value{
No return value, called for checking stuff
}
\description{
Check input ReadAHMD
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LawTable.R
\name{LawTable}
\alias{LawTable}
\title{Compute Life Tables from Parameters of a Mortality Law}
\usage{
LawTable(x, par, law, sex = NULL, lx0 = 1e+05, ax = NULL)
}
\arguments{
\item{x}{Numeric vector of ages at the beginning of each age interval. 
For a full life table, use single-year ages (e.g., \code{0:110}). 
For an abridged life table, use the lower bound of each interval 
(e.g., \code{c(0, 1, 5, 10, ..., 110)}).}

\item{par}{The parameters of the mortality model. Can be:
\itemize{
  \item A numeric \strong{vector} containing the coefficients (for a 
        single life table).
  \item A numeric \strong{matrix} or \strong{data.frame} where each row 
        corresponds to a separate set of parameters (producing multiple 
        life tables). Column names should match the parameter names of 
        the chosen law.
}}

\item{law}{The name of the mortality law to be used (e.g., \code{"gompertz"},
\code{"makeham"}). Run \code{\link{availableLaws}} to see all options.}

\item{sex}{Sex of the population. Options are \code{NULL} (default), 
\code{"male"}, \code{"female"}, or \code{"total"}. When specified, 
the first two entries of the \code{ax} column are adjusted using 
Coale-Demeny coefficients, producing more accurate life-table values 
at the youngest ages. The adjustment differs slightly between males 
and females.}

\item{lx0}{Radix, the starting population (or probability scale) at 
age 0. Default is \code{100,000}. All subsequent life-table columns 
(\code{lx}, \code{dx}, \code{Lx}, \code{Tx}) are scaled accordingly.}

\item{ax}{Numeric vector representing the average number of person-years 
lived in the age interval by those who die in that interval. If 
\code{NULL} (the default), \code{ax} is estimated internally using 
a standard formula. You may supply a single value (applied to all 
intervals) or a vector of the same length as \code{x}. A common 
assumption is \code{ax = 0.5}, which places deaths at the midpoint 
of each interval.}
}
\value{
An object of class \code{"LifeTable"} containing the following 
  components:
  \item{lt}{A \code{data.frame} with the complete life table, including 
    columns for age interval (\code{x.int}), exact age (\code{x}), 
    death rate (\code{mx}), death probability (\code{qx}), person-years 
    lived by decedents (\code{ax}), survivorship (\code{lx}), death 
    distribution (\code{dx}), person-years lived (\code{Lx}), total 
    person-years remaining (\code{Tx}), and life expectancy (\code{ex}).}
  \item{call}{The matched function call.}
  \item{process_date}{Timestamp of when the life table was computed.}
}
\description{
Generate a complete life table directly from the fitted parameters of a 
parametric mortality model. This function evaluates the mortality law at 
the given ages and passes the resulting death rates (\code{mx}) or death 
probabilities (\code{qx}) to \code{\link{LifeTable}} for further 
computation of all standard life-table columns (\code{lx}, \code{dx}, 
\code{Lx}, \code{Tx}, \code{ex}, etc.).
}
\details{
This function is designed to work with models that have been fitted 
externally (e.g., via \code{\link{MortalityLaw}} or by hand). The 
\code{par} argument must contain the estimated coefficients of the 
mortality law, and \code{law} must be one of the valid codes listed by 
\code{\link{availableLaws}}.

\strong{Important caveat: age scaling during fitting}

Several mortality laws (e.g., Gompertz, Makeham) internally \emph{scale} 
the age vector during optimisation to ensure numerical stability. If the 
model was fitted using \code{MortalityLaw} over an age range \code{[a, b]}, 
the published coefficients correspond to the \emph{scaled} ages, not the 
original ages. Consequently, \code{LawTable} will only produce valid life 
tables for ages \eqn{\ge a} (the lower bound of the fitting range). 
Attempting to use the same coefficients at younger ages will yield 
incorrect results (e.g., life expectancy at age 25 will equal that at 
age 45).

To determine which models apply age scaling, run:
\preformatted{
A <- availableLaws()$table
A[, c("CODE", "SCALE_X")]
}
Models with \code{SCALE_X = TRUE} rescale the age vector internally. When 
using \code{LawTable} with such a model, make sure the \code{x} argument 
starts from the same lower age bound used during fitting.

For models that do \emph{not} scale (e.g., Heligman-Pollard \code{"HP"}), 
this limitation does not apply, and \code{LawTable} can be used for any 
age range.
}
\examples{
# Example 1 --- Makeham --- multiple life tables from a matrix of parameters

x1 <- 45:100
L1 <- "makeham"
C1 <- matrix(
  c(0.00717, 0.07789, 0.00363,
    0.01018, 0.07229, 0.00001,
    0.00298, 0.09585, 0.00002,
    0.00067, 0.11572, 0.00078),
  nrow = 4,
  dimnames = list(1:4, c("A", "B", "C"))
)

LawTable(x = x1, par = C1, law = L1)

# ---- Important note on age scaling ----

# The Makeham model applies internal age scaling during fitting.
# If the coefficients above were estimated over ages 45-100, the life
# table produced by LawTable is valid only from age 45 onward.

# ---- Example 1B: correct usage ----
LawTable(x = 45:100, par = c(0.00717, 0.07789, 0.00363), law = L1)

# ---- Example 1C: incorrect usage ----
# The code below uses the same coefficients but starts at age 25.
# Because the model was fitted on scaled ages (starting at 45),
# the life table at age 25 will be meaningless (e.g., e25 equals e45).
\dontrun{
LawTable(x = 25:100, par = c(0.00717, 0.07789, 0.00363), law = L1)
}

# ---- How to check which laws apply scaling ----
A <- availableLaws()$table
A[, c("CODE", "SCALE_X")]

# Example 2 --- Heligman-Pollard (no scaling) ---

x2 <- 0:110
L2 <- "HP"
C2 <- c(0.00223, 0.01461, 0.12292, 0.00091,
        2.75201, 29.01877, 0.00002, 1.11411)

LawTable(x = x2, par = C2, law = L2)

# Because "HP" does NOT scale the age vector, the output is valid for
# any starting age. Compare:
LawTable(x = 3:110, par = C2, law = L2)
# Note that e3 = 70.31 in both tables, confirming consistency.

}
\seealso{
\code{\link{LifeTable}} for constructing life tables from raw mortality 
  data; 
\code{\link{MortalityLaw}} for fitting parametric mortality models; 
\code{\link{availableLaws}} for the list of implemented laws and their 
  scaling behaviour.
}
\author{
Marius D. Pascariu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readHMD.R
\name{data_format}
\alias{data_format}
\title{Data formats}
\usage{
data_format()
}
\value{
a vector
}
\description{
Data formats
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{head_tail}
\alias{head_tail}
\title{Summary function - display head and tail in a single data.frame
The original code for this function was first written for 'psych' R package
here we have modified it a bit}
\usage{
head_tail(x, hlength = 4, tlength = 4, digits = 4, ellipsis = TRUE)
}
\arguments{
\item{x}{A matrix or data frame or free text}

\item{hlength}{The number of lines at the beginning to show}

\item{tlength}{The number of lines at the end to show}

\item{digits}{Round off the data to digits}

\item{ellipsis}{separate the head and tail with dots}
}
\value{
Print table's head and tail
}
\description{
Summary function - display head and tail in a single data.frame
The original code for this function was first written for 'psych' R package
here we have modified it a bit
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAHMD.R
\name{AUSregions}
\alias{AUSregions}
\title{region codes}
\usage{
AUSregions()
}
\value{
a vector
}
\description{
region codes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaws-data.R
\docType{data}
\name{JMD_sample}
\alias{JMD_sample}
\title{JMD sample
Data object generated by the \code{ReadJMD()} function.}
\format{
An object of class \code{ReadJMD} of length 5.
}
\usage{
JMD_sample
}
\value{
A sample of demographic data in a data.frame
}
\description{
JMD sample
Data object generated by the \code{ReadJMD()} function.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readHMD.R
\name{ReadHMD}
\alias{ReadHMD}
\title{Download The Human Mortality Database (HMD)}
\usage{
ReadHMD(
  what,
  countries = NULL,
  interval = "1x1",
  username,
  password,
  save = FALSE,
  show = TRUE
)
}
\arguments{
\item{what}{What type of data are you looking for? The following options
might be available for some or all the countries and regions: \itemize{
  \item{\code{"births"}} -- birth records;
  \item{\code{"Dx_lexis"}} -- deaths by Lexis triangles;
  \item{\code{"Ex_lexis"}} -- exposure-to-risk by Lexis triangles;
  \item{\code{"population"}} -- population size;
  \item{\code{"Dx"}} -- death counts;
  \item{\code{"Ex"}} -- exposure-to-risk;
  \item{\code{"mx"}} -- central death-rates;
  \item{\code{"LT_f"}} -- period life tables for females;
  \item{\code{"LT_m"}} -- period life tables for males;
  \item{\code{"LT_t"}} -- period life tables both sexes combined;
  \item{\code{"e0"}} -- period life expectancy at birth;
  \item{\code{"Exc"}} -- cohort exposures;
  \item{\code{"mxc"}} -- cohort death-rates;
  \item{\code{"LT_fc"}} -- cohort life tables for females;
  \item{\code{"LT_mc"}} -- cohort life tables for males;
  \item{\code{"LT_tc"}} -- cohort life tables both sexes combined;
  \item{\code{"e0c"}} -- cohort life expectancy at birth;
  }}

\item{countries}{Specify the country data you want to download by adding the
HMD country code/s. Options:
\code{ 
"AUS"    "AUT",    "BEL",   "BGR", 
"BLR",   "CAN",    "CHL",   "HRV",
"HKG",   "CHE",    "CZE",   "DEUTNP", 
"DEUTE", "DEUTW",  "DNK",   "ESP", 
"EST",   "FIN",    "FRATNP","FRACNP", 
"GRC",   "HUN",    "IRL",   "ISL"    
"ISR",   "ITA",    "JPN",   "KOR", 
"LTU",   "LUX",    "LVA",   "NLD",   
"NOR",   "NZL_NP", "NZL_MA" "NZL_NM", 
"POL",   "PRT"     "RUS",   "SVK", 
"SVN",   "SWE",    "TWN",   "UKR",   
"GBR_NP","GBRTENW","GBRCENW","GBR_SCO",
"GBR_NIR","USA"}.
 If \code{NULL} data for all the countries are downloaded at once;}

\item{interval}{Datasets are given in various age and time formats based on
which the records are aggregated. Interval options:
\itemize{
  \item{\code{"1x1"}} -- by age and year;
  \item{\code{"1x5"}} -- by age and 5-year time interval;
  \item{\code{"1x10"}} -- by age and 10-year time interval;
  \item{\code{"5x1"}} -- by 5-year age group and year;
  \item{\code{"5x5"}} -- by 5-year age group and 5-year time interval;
  \item{\code{"5x10"}} --by 5-year age group and 10-year time interval.
  }}

\item{username}{Your HMD username. If you don't have one you can sign up
for free on the Human Mortality Database website.}

\item{password}{Your HMD password.}

\item{save}{Do you want to save a copy of the dataset on your local machine?
Logical. Default: \code{FALSE}.}

\item{show}{Choose whether to display a progress bar. Logical.
Default: \code{TRUE}.}
}
\value{
A \code{ReadHMD} object that contains:
 \item{input}{List with the input values (except the password).}
 \item{data}{Data downloaded from HMD.}
 \item{download.date}{Time stamp.}
 \item{years}{Numerical vector with the years covered in the data.}
 \item{ages}{Numerical vector with ages covered in the data.}
}
\description{
Download detailed mortality and population data for different countries
and regions in a single object from the \href{https://www.mortality.org/}{
Human Mortality Database}.
}
\details{
The Human Mortality Database (HMD) was created to provide detailed mortality
and population data to researchers, students, journalists, policy analysts,
and others interested in the history of human longevity. The project began
as an outgrowth of earlier projects in the Department of Demography at the
University of California, Berkeley, USA, and at the Max Planck Institute for
Demographic Research in Rostock, Germany (see history). It is the work of two
teams of researchers in the USA and Germany (see research teams), with the
help of financial backers and scientific collaborators from around the world
(see acknowledgements). The Center on the Economics and Development of Aging
(CEDA) French Institute for Demographic Studies (INED) has also supported the
further development of the database in recent years.
}
\examples{
\dontrun{


# Download demographic data for 3 countries in 1x1 format
age_int  <- 1  # age interval: 1,5
year_int <- 1  # year interval: 1,5,10
interval <- paste0(age_int, "x", year_int)  # --> 1x1
# And the 3 countries: Sweden Denmark and USA. We have to use the HMD codes
cntr  <- c('SWE', 'DNK', 'USA')

# Download death counts. We don't want to export data outside R.
HMD_Dx <- ReadHMD(what = "Dx",
                  countries = cntr,
                  interval  = interval,
                  username  = "user@email.com",
                  password  = "password",
                  save = FALSE)
HMD_Dx

# Download life tables for female population and export data.
LTF <- ReadHMD(what = "LT_f",
               countries = cntr,
               interval  = interval,
               username  = "user@email.com",
               password  = "password",
               save = TRUE)
LTF
}
}
\author{
Marius D. Pascariu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LifeTable.R
\name{uxAbove100}
\alias{uxAbove100}
\title{Educate mx or qx on how to behave above age 100 if it gets in trouble
(with NA's, zero's and Inf)}
\usage{
uxAbove100(x, ux, omega = 100, verbose = FALSE)
}
\arguments{
\item{x}{Numeric vector of ages at the beginning of each age interval. 
For a full life table, use single-year ages (e.g., \code{0:110}). 
For an abridged life table, use the lower bound of each interval 
(e.g., \code{c(0, 1, 5, 10, ..., 110)}).}

\item{ux}{A vector of mx or qx.}

\item{omega}{Threshold age. Default: 100.}

\item{verbose}{A logical value. Set \code{verbose = FALSE} to silent
the process that take place inside the function and avoid progress messages.}
}
\value{
A vector of rates
}
\description{
Educate mx or qx on how to behave above age 100 if it gets in trouble
(with NA's, zero's and Inf)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableHMD.R
\name{availableHMD}
\alias{availableHMD}
\title{Check Data Availability in HMD}
\usage{
availableHMD(link = "https://www.mortality.org/Data/DataAvailability")
}
\arguments{
\item{link}{URL to the HMD available data.
Default: "https://www.mortality.org/Data/DataAvailability"}
}
\value{
A tibble.
}
\description{
Returns information about the data available in the Human Mortality 
Database (HMD), including the range of years covered by the life tables 
for each country or region.
}
\examples{
availableHMD()

}
\seealso{
\code{\link{ReadHMD}}
}
\author{
Marius D. Pascariu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaws-data.R
\docType{data}
\name{CHMD_sample}
\alias{CHMD_sample}
\title{CHMD sample
Data object generated by the \code{ReadCHMD()} function.}
\format{
An object of class \code{ReadCHMD} of length 5.
}
\usage{
CHMD_sample
}
\value{
A sample of demographic data in a data.frame
}
\description{
CHMD sample
Data object generated by the \code{ReadCHMD()} function.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convertFx.R
\name{convertFx}
\alias{convertFx}
\title{Convert Life Table Indicators}
\usage{
convertFx(x, data, from, to, ...)
}
\arguments{
\item{x}{Numeric vector of ages at the beginning of each age interval. 
For a full life table, use single-year ages (e.g., \code{0:110}). 
For an abridged life table, use the lower bound of each interval 
(e.g., \code{c(0, 1, 5, 10, ..., 110)}).}

\item{data}{A numeric \code{vector}, \code{matrix}, or \code{data.frame} 
containing the mortality indicator to be converted. Each row should 
correspond to an age, each column to a separate population or time 
period.}

\item{from}{The type of indicator supplied in \code{data}. One of:
\code{"mx"}, \code{"qx"}, \code{"dx"}, or \code{"lx"}.}

\item{to}{The desired output indicator. One of:
\code{"mx"}, \code{"qx"}, \code{"dx"}, \code{"lx"}, \code{"Lx"}, 
\code{"Tx"}, or \code{"ex"}.}

\item{...}{Further arguments passed to \code{\link{LifeTable}} that may 
affect the results, such as \code{sex}, \code{lx0}, or \code{ax}.}
}
\value{
A numeric vector or matrix containing the converted life table 
  indicator. If the input was a named object, the output retains those 
  names.
}
\description{
Easily convert between different life table indicators (e.g., from death 
rates \code{mx} to death probabilities \code{qx}, or from survivorship 
\code{lx} to life expectancy \code{ex}). The function wraps 
\code{\link{LifeTable}} internally, so the conversion relies on the 
same constant-force-of-mortality (CFM) assumption and life-table 
methodology used throughout the package.
}
\details{
This function provides a convenient interface for converting a single 
mortality indicator into another, without having to call 
\code{\link{LifeTable}} directly and extract the desired column.

The supported \strong{input} types (\code{from}) are:
\code{mx}, \code{qx}, \code{dx}, and \code{lx}.

The supported \strong{output} types (\code{to}) are:
\code{mx}, \code{qx}, \code{dx}, \code{lx}, \code{Lx}, \code{Tx}, and 
\code{ex}.

There are 28 possible \code{from}-\code{to} combinations (4 inputs 
\eqn{\times} 7 outputs). All conversions pass through the full life-table 
computation; for example, converting \code{mx} to \code{ex} will 
internally compute \code{qx}, \code{lx}, \code{dx}, \code{Lx}, and 
\code{Tx} in sequence.

When \code{data} is a \code{vector}, the function returns a named vector. 
When \code{data} is a \code{matrix} or \code{data.frame} with multiple 
columns, the function applies the conversion column-wise and returns a 
matrix with the same row and column names as the input.
}
\examples{
# ---- Basic conversions ----

x  <- 0:110
mx <- ahmd$mx

# Convert death rates to death probabilities
qx <- convertFx(x, data = mx, from = "mx", to = "qx")

# Convert death rates to death distribution
dx <- convertFx(x, data = mx, from = "mx", to = "dx")

# Convert death rates to survivorship
lx <- convertFx(x, data = mx, from = "mx", to = "lx")

# ---- All 28 possible conversions ----

from <- c("mx", "qx", "dx", "lx")
to   <- c("mx", "qx", "dx", "lx", "Lx", "Tx", "ex")
K    <- expand.grid(from = from, to = to)

for (i in 1:nrow(K)) {
  In  <- as.character(K[i, "from"])
  Out <- as.character(K[i, "to"])
  N   <- paste0(Out, "_from_", In)
  cat(i, " Create", N, "\n")
  assign(N, convertFx(x = x, data = get(In), from = In, to = Out))
}

}
\seealso{
\code{\link{LifeTable}} for the underlying life-table construction; 
\code{\link{LawTable}} for generating life tables from parametric 
  mortality laws.
}
\author{
Marius D. Pascariu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_models.R
\name{bring_parameters}
\alias{bring_parameters}
\title{Bring or Rename Starting Parameters in the Law Functions}
\usage{
bring_parameters(law, par = NULL)
}
\arguments{
\item{law}{The name of the mortality law to be used (e.g., \code{"gompertz"},
\code{"makeham"}). Run \code{\link{availableLaws}} to see all options.}

\item{par}{parameters of the selected model. If NULL the
default values will be assigned automatically.}
}
\value{
Vector or initial model parameters
}
\description{
Bring or Rename Starting Parameters in the Law Functions
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readCHMD.R
\name{CANregions}
\alias{CANregions}
\title{Country codes}
\usage{
CANregions()
}
\value{
a vector
}
\description{
Country codes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LifeTable.R
\name{LifeTable.core}
\alias{LifeTable.core}
\title{LifeTable.core}
\usage{
LifeTable.core(x, Dx, Ex, mx, qx, lx, dx, sex, lx0, ax)
}
\arguments{
\item{x}{Numeric vector of ages at the beginning of each age interval. 
For a full life table, use single-year ages (e.g., \code{0:110}). 
For an abridged life table, use the lower bound of each interval 
(e.g., \code{c(0, 1, 5, 10, ..., 110)}).}

\item{Dx}{Death counts. Each element represents the total number of 
deaths during the calendar year to persons aged \code{x} to 
\code{x + n} (where \code{n} is the length of the age interval). 
Must be provided together with \code{Ex}.}

\item{Ex}{Exposure-to-risk in the period. This is usually approximated 
by the mid-year population aged \code{x} to \code{x + n}. Must be 
provided together with \code{Dx}.}

\item{mx}{Age-specific death rate in the age interval \code{[x, x+n)}. 
Defined as \code{Dx / Ex}.}

\item{qx}{Probability of dying within the age interval \code{[x, x+n)}.}

\item{lx}{Probability of surviving to exact age \code{x} (if \code{lx0 = 1}), 
or the number of survivors at exact age \code{x} (if \code{lx0 > 1}). 
When \code{lx} is the sole input, the values are re-scaled to the 
chosen radix \code{lx0}.}

\item{dx}{Number of deaths in the life-table population occurring in 
the age interval \code{[x, x+n)}. When \code{dx} is the sole input, 
the values are re-scaled to sum to \code{lx0}.}

\item{sex}{Sex of the population. Options are \code{NULL} (default), 
\code{"male"}, \code{"female"}, or \code{"total"}. When specified, 
the first two entries of the \code{ax} column are adjusted using 
Coale-Demeny coefficients, producing more accurate life-table values 
at the youngest ages. The adjustment differs slightly between males 
and females.}

\item{lx0}{Radix, the starting population (or probability scale) at 
age 0. Default is \code{100,000}. All subsequent life-table columns 
(\code{lx}, \code{dx}, \code{Lx}, \code{Tx}) are scaled accordingly.}

\item{ax}{Numeric vector representing the average number of person-years 
lived in the age interval by those who die in that interval. If 
\code{NULL} (the default), \code{ax} is estimated internally using 
a standard formula. You may supply a single value (applied to all 
intervals) or a vector of the same length as \code{x}. A common 
assumption is \code{ax = 0.5}, which places deaths at the midpoint 
of each interval.}
}
\value{
A data.frame containing life table results
}
\description{
LifeTable.core
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readHMD.R
\name{HMDindices}
\alias{HMDindices}
\title{HMD Indices}
\usage{
HMDindices()
}
\value{
a vector
}
\description{
HMD Indices
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaws-data.R
\docType{data}
\name{HMD_sample}
\alias{HMD_sample}
\title{HMD sample
Data object generated by the \code{ReadHMD()} function.}
\format{
An object of class \code{ReadHMD} of length 5.
}
\usage{
HMD_sample
}
\value{
A sample of demographic data in a data.frame
}
\description{
HMD sample
Data object generated by the \code{ReadHMD()} function.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_S3.R
\name{logLik.MortalityLaw}
\alias{logLik.MortalityLaw}
\title{logLik function for MortalityLaw}
\usage{
\method{logLik}{MortalityLaw}(object, ...)
}
\arguments{
\item{...}{further arguments passed to or from other methods.}
}
\value{
Model log-likelihood value
}
\description{
logLik function for MortalityLaw
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readHMD.R
\name{saveMsg}
\alias{saveMsg}
\title{Print message when saving an object}
\usage{
saveMsg()
}
\value{
No return value, called for side effects
}
\description{
Print message when saving an object
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readJMD.R
\name{check_input_ReadJMD}
\alias{check_input_ReadJMD}
\title{Check input ReadJMD}
\usage{
check_input_ReadJMD(x)
}
\arguments{
\item{x}{a list containing the input arguments from ReadJMD function}
}
\value{
No return value, called for validating input data
}
\description{
Check input ReadJMD
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{substrRight}
\alias{substrRight}
\title{Extracting the last n characters from a string}
\usage{
substrRight(x, n)
}
\arguments{
\item{x}{a string}

\item{n}{number of characters}
}
\value{
A character vector of length n
}
\description{
Extracting the last n characters from a string
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readCHMD.R
\name{ReadCHMD}
\alias{ReadCHMD}
\title{Download the Canadian Human Mortality Database (CHMD)}
\usage{
ReadCHMD(what, regions = NULL, interval = "1x1", save = FALSE, show = TRUE)
}
\arguments{
\item{what}{What type of data are you looking for? The following options are
available: \itemize{
  \item{\code{"births"}} -- birth records;
  \item{\code{"Dx_lexis"}} -- deaths by Lexis triangles;
  \item{\code{"population"}} -- population size;
  \item{\code{"Dx"}} -- death counts;
  \item{\code{"Ex"}} -- exposure-to-risk;
  \item{\code{"mx"}} -- central death-rates;
  \item{\code{"LT_f"}} -- period life tables for females;
  \item{\code{"LT_m"}} -- period life tables for males;
  \item{\code{"LT_t"}} -- period life tables both sexes combined;
  \item{\code{"e0"}} -- period life expectancy at birth;
  }}

\item{regions}{Specify the region specific data you want to download by
adding the CHMD region code/s. Options:
\itemize{
  \item{\code{"CAN"}} -- Canada - Sum of Canadian provinces and territories;
  \item{\code{"NFL"}} -- Newfoundland & Labrador;
  \item{\code{"PEI"}} -- Prince Edward Island;
  \item{\code{"NSC"}} -- Nova Scotia;
  \item{\code{"NBR"}} -- New Brunswick;
  \item{\code{"QUE"}} -- Quebec;
  \item{\code{"ONT"}} -- Ontario;
  \item{\code{"MAN"}} -- Manitoba;
  \item{\code{"SAS"}} -- Saskatchewan;
  \item{\code{"ALB"}} -- Alberta;
  \item{\code{"BCO"}} -- British Columbia;
  \item{\code{"NWT"}} -- Northwest Territories & Nunavut;
  \item{\code{"YUK"}} -- Yukon;
  \item{\code{NULL}} -- if \code{NULL} data for all the regions are downloaded.
  }}

\item{interval}{Datasets are given in various age and time formats based on
which the records are aggregated. Interval options:
\itemize{
  \item{\code{"1x1"}} -- by age and year;
  \item{\code{"1x5"}} -- by age and 5-year time interval;
  \item{\code{"1x10"}} -- by age and 10-year time interval;
  \item{\code{"5x1"}} -- by 5-year age group and year;
  \item{\code{"5x5"}} -- by 5-year age group and 5-year time interval;
  \item{\code{"5x10"}} --by 5-year age group and 10-year time interval.
  }}

\item{save}{Do you want to save a copy of the dataset on your local machine?
Logical. Default: \code{FALSE}.}

\item{show}{Choose whether to display a progress bar. Logical.
Default: \code{TRUE}.}
}
\value{
A \code{ReadCHMD} object that contains:
 \item{input}{List with the input values;}
 \item{data}{Data downloaded from CHMD;}
 \item{download.date}{Time stamp;}
 \item{years}{Numerical vector with the years covered in the data;}
 \item{ages}{Numerical vector with ages covered in the data.}
}
\description{
Download detailed mortality and population data for different
provinces and territories in Canada, in a single object from the
Canadian Human Mortality Database.
}
\details{
(Description taken from the CHMD website).

The Canadian Human Mortality Database (CHMD) was created to provide detailed
Canadian mortality and population data to researchers, students, journalists,
policy analysts, and others interested in the history of human longevity.
The project is an achievement of the Mortality and Longevity research team at
the Department of Demography, Universite de Montreal, under the supervision
of Professor Robert Bourbeau, in collaboration with demographers at the
Max Plank Institute for Demographic Research (Rostock, Germany) and the
Department of Demography, University of California at Berkeley.
Nadine Ouellette, researcher at the Institut national d'etudes demographiques
in Paris and member of the Mortality and Longevity research team at the
Universite de Montreal, is in charge of computing all CHMD life tables and
updating the CHMD web site.

The CHMD is a "satellite" of the Human Mortality Database (HMD), an
international database which currently holds detailed data for multiple
countries or regions. Consequently, the CHMD's underlying methodology
corresponds to the one used for the HMD.

The CHMD gathers all required data (deaths counts, births counts, population
size, exposure-to-risk, death rates) to compute life tables for Canada,
its provinces and its territories. One of the great advantages of the
database is to include data that is validated and corrected, when required,
and rendered comparable, if possible, for the period ranging from 1921
thru 2011. For comparison purposes, various life tables published by
governmental organizations are also available for download in PDF format.
}
\examples{
\donttest{
# Download demographic data for Quebec and Saskatchewan regions in 1x1 format

# Death counts. We don't want to export data outside R.
CHMD_Dx <- ReadCHMD(what = "Dx",
                    regions = c('QUE', 'SAS'),
                    interval  = "1x1",
                    save = FALSE)

# Download life tables for female population. To export data use save = TRUE.
LTF <- ReadCHMD(what = "LT_f",
                regions = c('QUE', 'SAS'),
                interval  = "1x1",
                save = FALSE)
}
}
\seealso{
\code{\link{ReadHMD}}
\code{\link{ReadAHMD}}
}
\author{
Marius D. Pascariu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableLaws.R
\name{print.availableLaws}
\alias{print.availableLaws}
\title{Print availableLaws}
\usage{
\method{print}{availableLaws}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"availableLaws"}}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
print info on the console
}
\description{
Print availableLaws
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_models.R
\name{kannisto_makeham}
\alias{kannisto_makeham}
\title{Kannisto-Makeham Mortality Law - 1998}
\usage{
kannisto_makeham(x, par = NULL)
}
\arguments{
\item{x}{vector of age at the beginning of the age classes}

\item{par}{parameters of the selected model. If NULL the
default values will be assigned automatically.}
}
\value{
A list of rates and model parameters
}
\description{
Kannisto-Makeham Mortality Law - 1998
}
\examples{
kannisto_makeham(x = 85:120)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_main.R
\name{objective_fun}
\alias{objective_fun}
\title{Objective function to minimise during optimisation}
\usage{
objective_fun(par, x, Dx, Ex, mx, qx, law, opt.method, custom.law)
}
\arguments{
\item{par}{Parameter vector on the log scale.}

\item{x}{Numeric vector of ages at the beginning of each age interval. 
For a full life table, use single-year ages (e.g., \code{0:110}). 
For an abridged life table, use the lower bound of each interval 
(e.g., \code{c(0, 1, 5, 10, ..., 110)}).}

\item{Dx}{Death counts. Each element represents the total number of 
deaths during the calendar year to persons aged \code{x} to 
\code{x + n} (where \code{n} is the length of the age interval). 
Must be provided together with \code{Ex}.}

\item{Ex}{Exposure-to-risk in the period. This is usually approximated 
by the mid-year population aged \code{x} to \code{x + n}. Must be 
provided together with \code{Dx}.}

\item{mx}{Age-specific death rate in the age interval \code{[x, x+n)}. 
Defined as \code{Dx / Ex}.}

\item{qx}{Probability of dying within the age interval \code{[x, x+n)}.}

\item{law}{The name of the mortality law to be used (e.g., \code{"gompertz"},
\code{"makeham"}). Run \code{\link{availableLaws}} to see all options.}

\item{opt.method}{The function to optimise. Available options:
\itemize{
  \item{\code{"poissonL"}: Poisson log-likelihood.}
  \item{\code{"binomialL"}: Binomial log-likelihood.}
  \item{\code{"LF1"}: Squared relative error \code{(1 - mu/nu)^2}.}
  \item{\code{"LF2"}: Squared log-ratio \code{log(mu/nu)^2}.}
  \item{\code{"LF3"}: Chi-squared-type \code{((nu - mu)^2)/nu}.}
  \item{\code{"LF4"}: Squared error \code{(nu - mu)^2}.}
  \item{\code{"LF5"}: Deviance-type \code{(nu - mu) * log(nu/mu)}.}
  \item{\code{"LF6"}: Absolute error \code{abs(nu - mu)}.}
}
See \code{\link{availableLF}} for details.}

\item{custom.law}{A user-defined function for fitting a model not included
in the package. The function must accept arguments \code{x} (age vector)
and \code{par} (named parameter vector) and return a list containing at
least an element named \code{hx} (the hazard or force of mortality). See
the examples below.}
}
\value{
A scalar loss value to be minimised.
}
\description{
Given a set of parameters (on the log scale), this function evaluates the
chosen loss function or negative log-likelihood by comparing observed
mortality values (Dx/Ex, mx, or qx) against the hazard rates predicted by
the specified mortality law.
}
\details{
Parameters are transformed back to the original scale via \code{exp(par)}
when calling the mortality law function. Infinite hazard values are capped
to 1, and large penalties are applied for missing or out-of-range values
to guide the optimiser away from invalid regions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LifeTable.R
\name{LifeTable}
\alias{LifeTable}
\title{Compute Life Tables from Mortality Data}
\usage{
LifeTable(x, Dx = NULL, Ex = NULL,
             mx = NULL,
             qx = NULL,
             lx = NULL,
             dx = NULL,
             sex = NULL,
             lx0 = 1e5,
             ax  = NULL)
}
\arguments{
\item{x}{Numeric vector of ages at the beginning of each age interval. 
For a full life table, use single-year ages (e.g., \code{0:110}). 
For an abridged life table, use the lower bound of each interval 
(e.g., \code{c(0, 1, 5, 10, ..., 110)}).}

\item{Dx}{Death counts. Each element represents the total number of 
deaths during the calendar year to persons aged \code{x} to 
\code{x + n} (where \code{n} is the length of the age interval). 
Must be provided together with \code{Ex}.}

\item{Ex}{Exposure-to-risk in the period. This is usually approximated 
by the mid-year population aged \code{x} to \code{x + n}. Must be 
provided together with \code{Dx}.}

\item{mx}{Age-specific death rate in the age interval \code{[x, x+n)}. 
Defined as \code{Dx / Ex}.}

\item{qx}{Probability of dying within the age interval \code{[x, x+n)}.}

\item{lx}{Probability of surviving to exact age \code{x} (if \code{lx0 = 1}), 
or the number of survivors at exact age \code{x} (if \code{lx0 > 1}). 
When \code{lx} is the sole input, the values are re-scaled to the 
chosen radix \code{lx0}.}

\item{dx}{Number of deaths in the life-table population occurring in 
the age interval \code{[x, x+n)}. When \code{dx} is the sole input, 
the values are re-scaled to sum to \code{lx0}.}

\item{sex}{Sex of the population. Options are \code{NULL} (default), 
\code{"male"}, \code{"female"}, or \code{"total"}. When specified, 
the first two entries of the \code{ax} column are adjusted using 
Coale-Demeny coefficients, producing more accurate life-table values 
at the youngest ages. The adjustment differs slightly between males 
and females.}

\item{lx0}{Radix, the starting population (or probability scale) at 
age 0. Default is \code{100,000}. All subsequent life-table columns 
(\code{lx}, \code{dx}, \code{Lx}, \code{Tx}) are scaled accordingly.}

\item{ax}{Numeric vector representing the average number of person-years 
lived in the age interval by those who die in that interval. If 
\code{NULL} (the default), \code{ax} is estimated internally using 
a standard formula. You may supply a single value (applied to all 
intervals) or a vector of the same length as \code{x}. A common 
assumption is \code{ax = 0.5}, which places deaths at the midpoint 
of each interval.}
}
\value{
An object of class \code{"LifeTable"} containing the following 
  components:
  \item{lt}{A \code{data.frame} with the complete life table, including 
    columns for age interval (\code{x.int}), exact age (\code{x}), 
    death rate (\code{mx}), death probability (\code{qx}), person-years 
    lived by decedents (\code{ax}), survivorship (\code{lx}), death 
    distribution (\code{dx}), person-years lived (\code{Lx}), total 
    person-years remaining (\code{Tx}), and life expectancy (\code{ex}).}
  \item{call}{The matched function call.}
  \item{process_date}{Timestamp of when the life table was computed.}
}
\description{
Construct either a full (single-year age intervals) or an abridged 
(wider age intervals) life table from a variety of input data types. 
The function accepts:
\itemize{
  \item Death counts and mid-interval population estimates (\code{Dx, Ex})
  \item Age-specific death rates (\code{mx})
  \item Death probabilities (\code{qx})
  \item Survivorship curve (\code{lx})
  \item Distribution of deaths (\code{dx})
}
Only one of these input options needs to be provided; the others 
are ignored if present. The input can be a numeric \code{vector}, 
\code{matrix}, or \code{data.frame}. When a \code{matrix} or 
\code{data.frame} with multiple columns is supplied, the function 
computes one life table per column.
}
\details{
A life table (also called a mortality table or actuarial table) 
summarises the mortality experience of a population. For each age 
(or age interval) it reports:
\itemize{
  \item Death rates (\code{mx}) and death probabilities (\code{qx})
  \item Survivorship (\code{lx})
  \item Distribution of deaths (\code{dx})
  \item Person-years lived (\code{Lx}) and total person-years remaining 
        (\code{Tx})
  \item Life expectancy (\code{ex})
}
The life table is constructed sequentially: from the input data the 
function derives \code{mx}, then \code{qx}, then \code{lx}, \code{dx}, 
\code{Lx}, \code{Tx}, and finally \code{ex}. The constant-force-of-mortality 
(CFM) assumption is used to convert between \code{mx} and \code{qx}. 
If the \code{sex} argument is supplied, the first two values of the 
\code{ax} column are adjusted using the Coale-Demeny method, which 
accounts for the different infant mortality patterns between males 
and females.
}
\examples{
# Example 1 --- Full life tables with different inputs ------------

y  <- 1900
x  <- as.numeric(rownames(ahmd$mx))
Dx <- ahmd$Dx[, paste(y)]
Ex <- ahmd$Ex[, paste(y)]

LT1 <- LifeTable(x, Dx = Dx, Ex = Ex)
LT2 <- LifeTable(x, mx = LT1$lt$mx)
LT3 <- LifeTable(x, qx = LT1$lt$qx)
LT4 <- LifeTable(x, lx = LT1$lt$lx)
LT5 <- LifeTable(x, dx = LT1$lt$dx)

LT1
LT5
ls(LT5)

# Example 2 --- Compute multiple life tables at once ------------

LTs <- LifeTable(x, mx = ahmd$mx)
LTs
# A warning is printed if the input contains missing values.
# Some of the missing values can be handled automatically.

# Example 3 --- Abridged life table -----------------------------

x  <- c(0, 1, seq(5, 110, by = 5))
mx <- c(.053, .005, .001, .0012, .0018, .002, .003, .004,
        .004, .005, .006, .0093, .0129, .019, .031, .049,
        .084, .129, .180, .2354, .3085, .390, .478, .551)
LT6 <- LifeTable(x, mx = mx, sex = "female")
LT6

# Example 4 --- Abridged life table using a custom 'ax' --------
# This example reuses the ages (x) and death rates (mx) from Example 3.
# Note that 'ax' must have the same length as 'x', otherwise an error
# will be returned.

my_ax <- c(0.1, 1.5, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
           2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1)

LT7 <- LifeTable(x = x, mx = mx, ax = my_ax)

}
\seealso{
\code{\link{LawTable}} for generating life tables from a fitted 
  parametric mortality law; 
  \code{\link{convertFx}} for converting between mortality measures.
}
\author{
Marius D. Pascariu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_main.R
\name{scale_x}
\alias{scale_x}
\title{Scale the age vector for stable optimisation}
\usage{
scale_x(x)
}
\arguments{
\item{x}{A numeric vector of ages.}
}
\value{
A numeric vector of scaled ages, where min(x) == 1.
}
\description{
For mortality laws that cover only a portion of the lifespan (e.g., adult
or old-age mortality), the age vector is rescaled so that the minimum age
becomes 1. This improves numerical stability by keeping the exponentiated
terms in the hazard function within a reasonable range.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphics.R
\name{plot.MortalityLaw}
\alias{plot.MortalityLaw}
\title{Plot Method for MortalityLaw}
\usage{
\method{plot}{MortalityLaw}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"MortalityLaw"}.}

\item{...}{Further arguments passed to graphical methods, such as
parameters (see \code{\link{par}}).}
}
\value{
A plot is generated as a side effect.
}
\description{
Plot Method for MortalityLaw
}
\examples{
# See complete example in MortalityLaw help page
}
\seealso{
\code{\link{MortalityLaw}}
}
\author{
Marius D. Pascariu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readCHMD.R
\name{check_input_ReadCHMD}
\alias{check_input_ReadCHMD}
\title{Check input for ReadCHMD}
\usage{
check_input_ReadCHMD(x)
}
\arguments{
\item{x}{A list with the input values for ReadCHMD}
}
\value{
No return value, called for input validation
}
\description{
Check input for ReadCHMD
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaws-data.R
\docType{data}
\name{ahmd}
\alias{ahmd}
\title{MortalityLaws Test Data}
\format{
An object of class \code{list} of length 3.
}
\source{
\href{https://www.mortality.org}{Human Mortality Database}
}
\usage{
ahmd
}
\description{
Dataset containing altered death rates (mx), death counts (Dx)
and exposures (Ex) for the female population living in
England & Wales in four different years: 1850, 1900, 1950 and 2010.
This dataset is provided for testing purposes only.
Download the actual data free of charge from \url{https://www.mortality.org}.
Once a username and a password are created on the website, the function
\code{\link{ReadHMD}} can be used for downloading.
}
\examples{
head(ahmd$mx)
}
\seealso{
\code{\link{ReadHMD}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LifeTable.R
\name{LifeTable.check}
\alias{LifeTable.check}
\title{Check LifeTable input}
\usage{
LifeTable.check(input)
}
\arguments{
\item{input}{A list containing the input arguments of the LifeTable functions.}
}
\value{
A list of life table validated data
}
\description{
Check LifeTable input
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_S3.R
\name{predict.MortalityLaw}
\alias{predict.MortalityLaw}
\title{Predict function for MortalityLaw}
\usage{
\method{predict}{MortalityLaw}(object, x, ...)
}
\arguments{
\item{object}{An object of class \code{"MortalityLaw"}}

\item{x}{Vector of ages to be considered in prediction}

\item{...}{Additional arguments affecting the predictions produced.}
}
\value{
A vector of predicted hazard rates
}
\description{
Predict function for MortalityLaw
}
\examples{
# Extrapolate old-age mortality with the Kannisto model
# Fit ages 80-94 and extrapolate up to 120.

Mx <- ahmd$mx[paste(80:94), "1950"]
M1 <- MortalityLaw(x = 80:94, mx  = Mx, law = 'kannisto')
fitted(M1)
predict(M1, x = 80:120)

# See more examples in MortalityLaw function help page.
}
\seealso{
\code{\link{MortalityLaw}}
}
\author{
Marius D. Pascariu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readHMD.R
\name{saveOutput}
\alias{saveOutput}
\title{Save Output in the working directory}
\usage{
saveOutput(out, show, prefix)
}
\arguments{
\item{out}{Output file}

\item{show}{Choose whether to display a progress bar. Logical.
Default: \code{TRUE}.}
}
\value{
No return value, called for side effects
}
\description{
Save Output in the working directory
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableHMD.R
\name{make_http_request}
\alias{make_http_request}
\title{Make HTTP request}
\usage{
make_http_request(url)
}
\arguments{
\item{url}{URL}
}
\value{
url response
}
\description{
Make HTTP request
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readHMD.R
\name{ReadHMD.core}
\alias{ReadHMD.core}
\title{Function to Download Data for a one Country}
\usage{
ReadHMD.core(what, country, interval, username, password, link)
}
\arguments{
\item{what}{What type of data are you looking for? The following options
might be available for some or all the countries and regions: \itemize{
  \item{\code{"births"}} -- birth records;
  \item{\code{"Dx_lexis"}} -- deaths by Lexis triangles;
  \item{\code{"Ex_lexis"}} -- exposure-to-risk by Lexis triangles;
  \item{\code{"population"}} -- population size;
  \item{\code{"Dx"}} -- death counts;
  \item{\code{"Ex"}} -- exposure-to-risk;
  \item{\code{"mx"}} -- central death-rates;
  \item{\code{"LT_f"}} -- period life tables for females;
  \item{\code{"LT_m"}} -- period life tables for males;
  \item{\code{"LT_t"}} -- period life tables both sexes combined;
  \item{\code{"e0"}} -- period life expectancy at birth;
  \item{\code{"Exc"}} -- cohort exposures;
  \item{\code{"mxc"}} -- cohort death-rates;
  \item{\code{"LT_fc"}} -- cohort life tables for females;
  \item{\code{"LT_mc"}} -- cohort life tables for males;
  \item{\code{"LT_tc"}} -- cohort life tables both sexes combined;
  \item{\code{"e0c"}} -- cohort life expectancy at birth;
  }}

\item{country}{HMD country code for the selected country. Character;}

\item{interval}{Datasets are given in various age and time formats based on
which the records are aggregated. Interval options:
\itemize{
  \item{\code{"1x1"}} -- by age and year;
  \item{\code{"1x5"}} -- by age and 5-year time interval;
  \item{\code{"1x10"}} -- by age and 10-year time interval;
  \item{\code{"5x1"}} -- by 5-year age group and year;
  \item{\code{"5x5"}} -- by 5-year age group and 5-year time interval;
  \item{\code{"5x10"}} --by 5-year age group and 10-year time interval.
  }}

\item{username}{Your HMD username. If you don't have one you can sign up
for free on the Human Mortality Database website.}

\item{password}{Your HMD password.}

\item{link}{the main link to the database.}
}
\value{
A data.frame containing demographic data
}
\description{
Function to Download Data for a one Country
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_main.R
\name{MortalityLaw}
\alias{MortalityLaw}
\title{Fit Mortality Laws}
\usage{
MortalityLaw(x, Dx = NULL, Ex = NULL, mx = NULL, qx = NULL,
                law = NULL,
                opt.method = "LF2",
                parS = NULL,
                fit.this.x = x,
                custom.law = NULL,
                show = FALSE, ...)
}
\arguments{
\item{x}{Numeric vector of ages at the beginning of each age interval. 
For a full life table, use single-year ages (e.g., \code{0:110}). 
For an abridged life table, use the lower bound of each interval 
(e.g., \code{c(0, 1, 5, 10, ..., 110)}).}

\item{Dx}{Death counts. Each element represents the total number of 
deaths during the calendar year to persons aged \code{x} to 
\code{x + n} (where \code{n} is the length of the age interval). 
Must be provided together with \code{Ex}.}

\item{Ex}{Exposure-to-risk in the period. This is usually approximated 
by the mid-year population aged \code{x} to \code{x + n}. Must be 
provided together with \code{Dx}.}

\item{mx}{Age-specific death rate in the age interval \code{[x, x+n)}. 
Defined as \code{Dx / Ex}.}

\item{qx}{Probability of dying within the age interval \code{[x, x+n)}.}

\item{law}{The name of the mortality law to be used (e.g., \code{"gompertz"},
\code{"makeham"}). Run \code{\link{availableLaws}} to see all options.}

\item{opt.method}{The function to optimise. Available options:
\itemize{
  \item{\code{"poissonL"}: Poisson log-likelihood.}
  \item{\code{"binomialL"}: Binomial log-likelihood.}
  \item{\code{"LF1"}: Squared relative error \code{(1 - mu/nu)^2}.}
  \item{\code{"LF2"}: Squared log-ratio \code{log(mu/nu)^2}.}
  \item{\code{"LF3"}: Chi-squared-type \code{((nu - mu)^2)/nu}.}
  \item{\code{"LF4"}: Squared error \code{(nu - mu)^2}.}
  \item{\code{"LF5"}: Deviance-type \code{(nu - mu) * log(nu/mu)}.}
  \item{\code{"LF6"}: Absolute error \code{abs(nu - mu)}.}
}
See \code{\link{availableLF}} for details.}

\item{parS}{Optional starting parameter values for the optimisation. If
\code{NULL}, sensible defaults are automatically chosen via
\code{\link{bring_parameters}}.}

\item{fit.this.x}{A subset of \code{x} over which to fit the model. The
default is the entire \code{x} vector. Use this to exclude, for example,
advanced ages where data are sparse.}

\item{custom.law}{A user-defined function for fitting a model not included
in the package. The function must accept arguments \code{x} (age vector)
and \code{par} (named parameter vector) and return a list containing at
least an element named \code{hx} (the hazard or force of mortality). See
the examples below.}

\item{show}{Logical. If \code{TRUE}, a progress bar is displayed during
fitting. Default: \code{FALSE}.}

\item{...}{Additional arguments passed to or from other methods.}
}
\value{
An object of class \code{"MortalityLaw"}, which is a list with the following
components:
\item{input}{List of input arguments, stored for reproducibility.}
\item{info}{Model information (name, formula, date of fitting).}
\item{coefficients}{Estimated parameters of the mortality law. A named
vector for a single fit, or a matrix for multiple fits.}
\item{fitted.values}{Fitted hazard rates (or death probabilities) evaluated
at the input ages \code{x}.}
\item{residuals}{Deviance residuals, computed as observed minus fitted
values.}
\item{goodness.of.fit}{List or matrix of goodness-of-fit measures: AIC,
BIC and log-likelihood (available only for likelihood-based methods).}
\item{opt.diagnosis}{Object returned by the optimisation routine, useful
for checking convergence.}
\item{df}{Number of parameters and residual degrees of freedom.}
\item{deviance}{Sum of squared log-residuals, used as a deviance measure.}
}
\description{
Fit parametric mortality models given a set of input data. The data can be
supplied as death counts and mid-interval population estimates
\code{(Dx, Ex)}, age-specific death rates \code{(mx)}, or death
probabilities \code{(qx)}. Use the \code{law} argument to specify the
model to be fitted. Over 30 parametric models are currently implemented;
run \code{\link{availableLaws}} to see the full list. Models can be fitted
using maximum likelihood or by optimising a loss function. See the
\code{\link{availableLF}} function for the implemented options.
}
\details{
\strong{Optimisation:} The PORT routines (via \code{\link{nlminb}}) are used
for unconstrained and box-constrained optimisation. Parameters are estimated
on the log scale to ensure positivity, and the routine is set to allow up to
5000 iterations. When the optimisation method is \code{"poissonL"} or
\code{"binomialL"}, the AIC, BIC and log-likelihood are computed from the
likelihood. Otherwise these are set to \code{NaN}.

\strong{Scaling of the age vector:} For models that cover only a portion
of the lifespan (e.g., adult or old-age mortality), the age vector \code{x}
is automatically re-scaled as \code{x = x - min(x) + 1} before fitting.
This transformation improves numerical stability and helps the optimisation
algorithm converge, especially when the starting age is far from zero.
Models that apply this scaling are flagged with \code{SCALE_X = TRUE} in
the table returned by \code{\link{availableLaws}}. When using
\code{\link{predict.MortalityLaw}} or \code{\link{LawTable}} with such
models, the same scaling is applied internally, so predictions remain
consistent with the fitted coefficients.

\strong{Handling matrix input:} If \code{Dx}, \code{Ex}, \code{mx} or
\code{qx} are provided as matrices (with one column per population or
time period), the function iterates over the columns and fits a separate
model to each, returning a collection of results.
}
\examples{
# Example 1: Fitting the Makeham model --------------------------
x  <- 45:75
Dx <- ahmd$Dx[paste(x), "1950"]
Ex <- ahmd$Ex[paste(x), "1950"]

M1 <- MortalityLaw(x = x, Dx = Dx, Ex = Ex, law = 'makeham')

M1
ls(M1)
coef(M1)
summary(M1)
fitted(M1)
predict(M1, x = 45:95)
plot(M1)


# Example 2: --------------------------
# We can fit the same model using a different data format
# and a different optimization method.
x  <- 45:75
mx <- ahmd$mx[paste(x), ]
M2 <- MortalityLaw(x = x, mx = mx, law = 'makeham', opt.method = 'LF1')
M2
fitted(M2)
predict(M2, x = 55:90)

# Example 3: --------------------------
# Now let's fit a mortality law that is not defined
# in the package, say a reparameterized Gompertz in
# terms of modal age at death
# hx = b*exp(b*(x-m)) (here b and m are the parameters to be estimated)

# A function with 'x' and 'par' as input has to be defined, which returns
# at least an object called 'hx' (hazard rate).
my_gompertz <- function(x, par = c(b = 0.13, M = 45)){
  hx  <- with(as.list(par), b*exp(b*(x - M)) )
  return(as.list(environment()))
}

M3 <- MortalityLaw(x = x, Dx = Dx, Ex = Ex, custom.law = my_gompertz)
summary(M3)
# predict M3 for different ages
predict(M3, x = 85:130)


# Example 4: --------------------------
# Fit Heligman-Pollard model for a single
# year in the dataset between age 0 and 100 and build a life table.

x  <- 0:100
mx <- ahmd$mx[paste(x), "1950"] # select data
M4 <- MortalityLaw(x = x, mx = mx, law = 'HP', opt.method = 'LF2')
M4
plot(M4)

LifeTable(x = x, qx = fitted(M4))
}
\seealso{
\code{\link{availableLaws}} for a list of all implemented models;
\code{\link{availableLF}} for loss function details;
\code{\link{LifeTable}} for life table construction;
\code{\link{ReadHMD}} for downloading data from the Human Mortality Database.
}
\author{
Marius D. Pascariu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_models.R
\name{quadratic}
\alias{quadratic}
\title{Quadratic Model}
\usage{
quadratic(x, par = NULL)
}
\arguments{
\item{x}{vector of age at the beginning of the age classes}

\item{par}{parameters of the selected model. If NULL the
default values will be assigned automatically.}
}
\value{
A list of rates and model parameters
}
\description{
Quadratic Model
}
\examples{
quadratic(x = 0:100)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_S3.R
\name{df.residual.MortalityLaw}
\alias{df.residual.MortalityLaw}
\title{df.residual function for MortalityLaw}
\usage{
\method{df.residual}{MortalityLaw}(object, ...)
}
\arguments{
\item{...}{further arguments passed to or from other methods.}
}
\value{
model residual value
}
\description{
df.residual function for MortalityLaw
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_S3.R
\name{summary.MortalityLaw}
\alias{summary.MortalityLaw}
\title{Summary MortalityLaw}
\usage{
\method{summary}{MortalityLaw}(object, ..., digits = max(3L, getOption("digits") - 3L))
}
\arguments{
\item{object}{an object of class \code{"MortalityLaw"}}

\item{...}{additional arguments affecting the summary produced.}

\item{digits}{number of digits to display.}
}
\value{
A list of model diagnosis
}
\description{
Summary MortalityLaw
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_S3.R
\name{AIC.MortalityLaw}
\alias{AIC.MortalityLaw}
\title{AIC function for MortalityLaw}
\usage{
\method{AIC}{MortalityLaw}(object, ...)
}
\arguments{
\item{...}{further arguments passed to or from other methods.}
}
\value{
model AIC value
}
\description{
AIC function for MortalityLaw
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_models.R
\name{invweibull}
\alias{invweibull}
\title{Inverse-Weibull Mortality Law}
\usage{
invweibull(x, par = NULL)
}
\arguments{
\item{x}{vector of age at the beginning of the age classes}

\item{par}{parameters of the selected model. If NULL the
default values will be assigned automatically.}
}
\value{
A list of rates and model parameters
}
\description{
The Inverse-Weibull proves useful for modelling the childhood and teenage years,
because the logarithm of h(x) is a concave function.
m > 0 is a measure of location
sigma > 0 is measure of dispersion
}
\examples{
invweibull(x = 1:20)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_models.R
\name{invgompertz}
\alias{invgompertz}
\title{Inverse-Gompertz Mortality Law - informative parameterization}
\usage{
invgompertz(x, par = NULL)
}
\arguments{
\item{x}{vector of age at the beginning of the age classes}

\item{par}{parameters of the selected model. If NULL the
default values will be assigned automatically.}
}
\value{
A list of rates and model parameters
}
\description{
m - is a measure of location because it is the mode of the density, m > 0
sigma - represents the dispersion of the density about the mode, sigma > 0
}
\examples{
invgompertz(x = 15:25)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LifeTable.R
\name{mx_qx}
\alias{mx_qx}
\title{mx to qx}
\usage{
mx_qx(x, nx, ux, out = c("qx", "mx"))
}
\arguments{
\item{x}{Numeric vector of ages at the beginning of each age interval. 
For a full life table, use single-year ages (e.g., \code{0:110}). 
For an abridged life table, use the lower bound of each interval 
(e.g., \code{c(0, 1, 5, 10, ..., 110)}).}

\item{nx}{Length of the age-intervals.}

\item{ux}{A vector of mx or qx.}

\item{out}{Type of the output: mx or qx.}
}
\value{
A vector of rates
}
\description{
Function to convert mx into qx and back, using the constant force of
mortality assumption (CFM).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readJMD.R
\name{print.ReadJMD}
\alias{print.ReadJMD}
\title{Print ReadJMD}
\usage{
\method{print}{ReadJMD}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"ReadJMD"}}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Print info on the console
}
\description{
Print ReadJMD
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readCHMD.R
\name{print.ReadCHMD}
\alias{print.ReadCHMD}
\title{Print ReadCHMD}
\usage{
\method{print}{ReadCHMD}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"ReadCHMD"}}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Print data on the console
}
\description{
Print ReadCHMD
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readHMD.R
\name{ageMsg}
\alias{ageMsg}
\title{What age(s) are we looking at?}
\usage{
ageMsg(what, x)
}
\arguments{
\item{x}{An object of class \code{"ReadHMD"}}
}
\value{
A scalar or character indicating age groups
}
\description{
What age(s) are we looking at?
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_main.R
\name{addDetails}
\alias{addDetails}
\title{Retrieve model-specific details for fitting}
\usage{
addDetails(law, custom.law = NULL, parS = NULL)
}
\arguments{
\item{law}{The name of the mortality law to be used (e.g., \code{"gompertz"},
\code{"makeham"}). Run \code{\link{availableLaws}} to see all options.}

\item{custom.law}{A user-defined function for fitting a model not included
in the package. The function must accept arguments \code{x} (age vector)
and \code{par} (named parameter vector) and return a list containing at
least an element named \code{hx} (the hazard or force of mortality). See
the examples below.}

\item{parS}{Optional starting parameter values for the optimisation. If
\code{NULL}, sensible defaults are automatically chosen via
\code{\link{bring_parameters}}.}
}
\value{
A list with components:
\item{law}{Internal law name (e.g., \code{"custom.law"} for user-supplied functions).}
\item{parS}{Starting parameter values for the optimisation.}
\item{model}{Model information data frame (from \code{\link{availableLaws}})
or \code{"Custom Mortality Law"} for user-defined models.}
\item{scale.x}{Logical; whether the age vector should be re-scaled
(\code{x = x - min(x) + 1}) before fitting.}
}
\description{
Based on the chosen mortality law (or a custom law), this function retrieves
the default starting parameters, the model information table, and whether
the age vector should be scaled before fitting.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LifeTable.R
\name{compute.ax}
\alias{compute.ax}
\title{Find ax indicator}
\usage{
compute.ax(x, mx, qx)
}
\arguments{
\item{x}{Numeric vector of ages at the beginning of each age interval. 
For a full life table, use single-year ages (e.g., \code{0:110}). 
For an abridged life table, use the lower bound of each interval 
(e.g., \code{c(0, 1, 5, 10, ..., 110)}).}

\item{mx}{Age-specific death rate in the age interval \code{[x, x+n)}. 
Defined as \code{Dx / Ex}.}

\item{qx}{Probability of dying within the age interval \code{[x, x+n)}.}
}
\value{
\code{ax} - the point in the age interval where 50% of the deaths
have already occurred
}
\description{
Find ax indicator
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_models.R
\name{ggompertz}
\alias{ggompertz}
\title{Gamma-Gompertz Model as in Vaupel et al. (1979)}
\usage{
ggompertz(x, par = NULL)
}
\arguments{
\item{x}{vector of age at the beginning of the age classes}

\item{par}{parameters of the selected model. If NULL the
default values will be assigned automatically.}
}
\value{
A list of rates and model parameters
}
\description{
Gamma-Gompertz Model as in Vaupel et al. (1979)
}
\examples{
ggompertz(x = 50:120)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableLaws.R
\name{availableLaws}
\alias{availableLaws}
\title{Check Available Mortality Laws}
\usage{
availableLaws(law = NULL)
}
\arguments{
\item{law}{Optional. Default: \code{NULL}. One can extract details about
a certain model by specifying its codename.}
}
\value{
The output is of the \code{"availableLaws"} class with the following
components:
 \item{table}{Table with mortality models and codes to be used in \code{\link{MortalityLaw}}.}
 \item{legend}{Table with details about the section of the mortality curve.}
}
\description{
The function returns information about the parametric models that can be
called and fitted in the \code{\link{MortalityLaw}} function.
For a comprehensive review of the most important mortality laws,
Tabeau (2001) is a good starting point.
}
\examples{
availableLaws()
}
\references{
\enumerate{
\item{Gompertz, B. (1825). \href{https://www.jstor.org/stable/107756}{On the
Nature of the Function Expressive of the Law of Human Mortality, and on a
New Mode of Determining the Value of Life Contingencies.}
Philosophical Transactions of the Royal Society of London, 115, 513-583.}
\item{Makeham, W. (1860).
On the Law of Mortality and Construction of Annuity Tables.
The Assurance Magazine and Journal of the Institute of Actuaries, 8(6),
301-310. \doi{10.1017/S204616580000126X}}
\item{Thiele, T. (1871). On a Mathematical Formula to express the
Rate of Mortality throughout the whole of Life, tested by a Series of
Observations made use of by the Danish Life Insurance Company of 1871.
Journal of the Institute of Actuaries and Assurance Magazine, 16(5), 313-329.
\doi{10.1017/S2046167400043688}}
\item{Oppermann, L. H. F. (1870). On the graduation of life tables,
with special application to the rate of mortality in infancy and childhood.
The Insurance Record Minutes from a meeting in the Institute of Actuaries, 42.}
\item{Wittstein, T. and D. Bumsted. (1883).
\href{https://www.cambridge.org/core/journals/journal-of-the-institute-of-actuaries/article/the-mathematical-law-of-mortality/57A7403B578C84769A463EA2BC2F7ECD}{
The Mathematical Law of Mortality.}
Journal of the Institute of Actuaries and Assurance Magazine, 24(3), 153-173.}
\item{Steffensen, J. (1930). Infantile mortality from an actuarial point of
view. Skandinavisk Aktuarietidskrift 13, 272-286.
\doi{10.1080/03461238.1930.10416902}}
\item{Perks, W. (1932).
On Some Experiments in the Graduation of Mortality Statistics.
Journal of the Institute of Actuaries, 63(1), 12-57.
\doi{10.1017/S0020268100046680}}
\item{Harper, F. S. (1936). An actuarial study of infant mortality.
Scandinavian Actuarial Journal 1936 (3-4), 234-270.
\doi{10.1080/03461238.1936.10405113}}
\item{Weibull, W. (1951). A statistical distribution function of wide applicability.
Journal of applied mechanics 103, 293-297.
 \doi{10.1115/1.4010337}}
\item{Beard, R. E. (1971).
\href{http://longevity-science.org/Beard-1971.pdf}{
Some aspects of theories of mortality, cause of
death analysis, forecasting and stochastic processes.}
Biological aspects of demography 999, 57-68.}
\item{Vaupel, J., Manton, K.G., and Stallard, E. (1979).
The impact of heterogeneity in individual frailty on the dynamics of
mortality. Demography 16(3): 439-454.
\doi{10.2307/2061224}}
\item{Siler, W. (1979),
A Competing-Risk Model for Animal Mortality. Ecology, 60: 750-757.
\doi{10.2307/1936612}}
\item{Heligman, L., & Pollard, J. (1980). The age pattern of mortality.
Journal of the Institute of Actuaries, 107(1), 49-80.
\doi{10.1017/S0020268100040257}}
\item{Rogers A and Planck F (1983).
\href{https://pure.iiasa.ac.at/id/eprint/2210/1/WP-83-102.pdf}{
MODEL: A General Program for Estimating Parametrized Model Schedules of Fertility,
Mortality, Migration, and Marital and Labor Force Status Transitions.}
IIASA Working Paper. IIASA, Laxenburg, Austria: WP-83-102}
\item{Martinelle S. (1987). A generalized Perks formula for old-age mortality.
Stockholm, Sweden, Statistiska Centralbyran, 1987. 55 p.
(R&D Report, Research-Methods-Development, U/STM No. 38)}
\item{Carriere J.F. (1992). Parametric models for life tables.
Transactions of the Society of Actuaries. Vol.44}
\item{Kostaki A. (1992).
A nine-parameter version of the Heligman-Pollard formula.
Mathematical Population Studies. Vol. 3 277-288.
\doi{10.1080/08898489209525346}}
\item{Thatcher AR, Kannisto V and Vaupel JW (1998).
The force of mortality at ages 80 to 120. Odense Monographs on
Population Aging Vol. 5, Odense University Press, 1998. 104, 20 p.
Odense, Denmark}
\item{Tabeau E. (2001). A Review of Demographic Forecasting Models for
Mortality. In: Tabeau E., van den Berg Jeths A., Heathcote C. (eds)
Forecasting Mortality in Developed Countries.
European Studies of Population, vol 9. Springer, Dordrecht.
\doi{10.1007/0-306-47562-6_1}}
\item{Finkelstein M. (2012)
Discussing the Strehler-Mildvan model of mortality
Demographic Research, Vol. 26(9), 191-206.
\doi{10.4054/DemRes.2012.26.9}}
}
}
\seealso{
\code{\link{MortalityLaw}}
}
\author{
Marius D. Pascariu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_models.R
\name{weibull}
\alias{weibull}
\title{Weibull Mortality Law - 1939}
\usage{
weibull(x, par = NULL)
}
\arguments{
\item{x}{vector of age at the beginning of the age classes}

\item{par}{parameters of the selected model. If NULL the
default values will be assigned automatically.}
}
\value{
A list of rates and model parameters
}
\description{
Note that if sigma > m, then the mode of the density is 0 and hx is a
non-increasing function of x, while if sigma < m, then the mode is
greater than 0 and hx is an increasing function.
m > 0 is a measure of location
sigma > 0 is measure of dispersion
}
\examples{
weibull(x = 1:20)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_main.R
\name{choose_optim}
\alias{choose_optim}
\title{Run the optimisation routine}
\usage{
choose_optim(input)
}
\arguments{
\item{input}{A list containing all input arguments to \code{\link{MortalityLaw}}.}
}
\value{
A list with components:
\item{x}{Age vector (original).}
\item{new.x}{Age vector after optional scaling.}
\item{opt}{Object returned by \code{\link{nlminb}}.}
\item{C}{Estimated parameters on the original scale.}
\item{hx}{Fitted hazard values evaluated at \code{new.x}.}
\item{logLik, AIC, BIC}{Goodness-of-fit measures (\code{NaN} for
non-likelihood methods).}
}
\description{
This is the core optimisation function for \code{\link{MortalityLaw}}. It:
\enumerate{
  \item Subsets the data to the fitting ages (\code{fit.this.x}).
  \item Scales the age vector if required by the chosen model.
  \item Obtains default starting parameters (if not provided).
  \item Minimises the objective function using \code{\link{nlminb}} (PORT
        routines) with the parameters on the log scale.
  \item Transforms parameters back to the original scale, computes the
        fitted hazard, and derives goodness-of-fit measures (AIC, BIC,
        log-likelihood) where applicable.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableLF.R
\name{availableLF}
\alias{availableLF}
\title{Check Available Loss Functions}
\usage{
availableLF()
}
\value{
A list of class \code{availableLF} with the components:
 \item{table}{Table with loss functions and codes to be used in \code{\link{MortalityLaw}}.}
 \item{legend}{Table with details about the abbreviation used.}
}
\description{
Returns information about the loss functions implemented for use with the
optimisation procedure in the \code{\link{MortalityLaw}} function.
}
\examples{
availableLF()
}
\seealso{
\code{\link{MortalityLaw}}
}
\author{
Marius D. Pascariu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LifeTable.R
\name{coale.demeny.ax}
\alias{coale.demeny.ax}
\title{Find ax[1:2] indicators using Coale-Demeny coefficients
Here we adjust the first two values of ax to account for infant
mortality more accurately}
\usage{
coale.demeny.ax(x, mx, ax, sex)
}
\arguments{
\item{x}{Numeric vector of ages at the beginning of each age interval. 
For a full life table, use single-year ages (e.g., \code{0:110}). 
For an abridged life table, use the lower bound of each interval 
(e.g., \code{c(0, 1, 5, 10, ..., 110)}).}

\item{mx}{Age-specific death rate in the age interval \code{[x, x+n)}. 
Defined as \code{Dx / Ex}.}

\item{ax}{Numeric vector representing the average number of person-years 
lived in the age interval by those who die in that interval. If 
\code{NULL} (the default), \code{ax} is estimated internally using 
a standard formula. You may supply a single value (applied to all 
intervals) or a vector of the same length as \code{x}. A common 
assumption is \code{ax = 0.5}, which places deaths at the midpoint 
of each interval.}

\item{sex}{Sex of the population. Options are \code{NULL} (default), 
\code{"male"}, \code{"female"}, or \code{"total"}. When specified, 
the first two entries of the \code{ax} column are adjusted using 
Coale-Demeny coefficients, producing more accurate life-table values 
at the youngest ages. The adjustment differs slightly between males 
and females.}
}
\value{
A vector of coefficients
}
\description{
Find ax[1:2] indicators using Coale-Demeny coefficients
Here we adjust the first two values of ax to account for infant
mortality more accurately
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LifeTable.R
\name{find.my.case}
\alias{find.my.case}
\title{Function that identifies the case/problem we have to solve}
\usage{
find.my.case(Dx = NULL, Ex = NULL, mx = NULL, qx = NULL, lx = NULL, dx = NULL)
}
\arguments{
\item{Dx}{Death counts. Each element represents the total number of 
deaths during the calendar year to persons aged \code{x} to 
\code{x + n} (where \code{n} is the length of the age interval). 
Must be provided together with \code{Ex}.}

\item{Ex}{Exposure-to-risk in the period. This is usually approximated 
by the mid-year population aged \code{x} to \code{x + n}. Must be 
provided together with \code{Dx}.}

\item{mx}{Age-specific death rate in the age interval \code{[x, x+n)}. 
Defined as \code{Dx / Ex}.}

\item{qx}{Probability of dying within the age interval \code{[x, x+n)}.}

\item{lx}{Probability of surviving to exact age \code{x} (if \code{lx0 = 1}), 
or the number of survivors at exact age \code{x} (if \code{lx0 > 1}). 
When \code{lx} is the sole input, the values are re-scaled to the 
chosen radix \code{lx0}.}

\item{dx}{Number of deaths in the life-table population occurring in 
the age interval \code{[x, x+n)}. When \code{dx} is the sole input, 
the values are re-scaled to sum to \code{lx0}.}
}
\value{
A list containing problem solving details
}
\description{
Function that identifies the case/problem we have to solve
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readJMD.R
\name{JPNregions}
\alias{JPNregions}
\title{region codes}
\usage{
JPNregions()
}
\value{
the vector
}
\description{
region codes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MortalityLaw_check.R
\name{check.MortalityLaw}
\alias{check.MortalityLaw}
\title{Function to check input data in MortalityLaw}
\usage{
check.MortalityLaw(input)
}
\arguments{
\item{input}{A list containing all input arguments to \code{\link{MortalityLaw}}.}
}
\value{
No return value, called for side effects
}
\description{
Function to check input data in MortalityLaw
}
\keyword{internal}
